@@ -1,12 +1,13 @@
-'''CTS: Cluster Testing System: CIB generator
-'''
-__copyright__ = '''
-Author: Andrew Beekhof <abeekhof@suse.de>
-Copyright (C) 2008 Andrew Beekhof
-'''
+""" CIB XML generator for Pacemaker's Cluster Test Suite (CTS)
+"""
+
+# Pacemaker targets compatibility with Python 2.7 and 3.2+
+from __future__ import print_function, unicode_literals, absolute_import, division
+
+__copyright__ = "Copyright 2008-2018 the Pacemaker project contributors"
+__license__ = "GNU General Public License version 2 or later (GPLv2+) WITHOUT ANY WARRANTY"
 
 import sys
-import string
 
 from cts.CTSvars import *
 from cts.CIB     import CibBase
@@ -56,7 +57,7 @@ class InstanceAttributes(XmlBase):
         XmlBase.__init__(self, Factory, "instance_attributes", name)
 
         # Create an <nvpair> for each attribute
-        for (attr, value) in attrs.items():
+        for (attr, value) in list(attrs.items()):
             self.add_child(XmlBase(Factory, "nvpair", "%s-%s" % (name, attr),
                 name=attr, value=value))
 
@@ -320,9 +321,3 @@ class Clone(Group):
             self.children.append(resource)
         else:
             self.Factory.log("Clones can only have a single child. Ignoring %s" % resource.name)
-
-
-class Master(Clone):
-    def __init__(self, Factory, name, child=None):
-        Clone.__init__(self, Factory, name, child)
-        self.tag = "master"
