@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -15,6 +15,7 @@
 #include <unistd.h>
 
 #include <stdlib.h>
+#include <stdint.h>     // uint32_t, uint64_t, UINT64_C()
 #include <errno.h>
 #include <fcntl.h>
 #include <inttypes.h>  /* U64T ~ PRIu64 */
@@ -49,8 +50,8 @@ qb_ipcs_service_t *ipcs_rw = NULL;
 qb_ipcs_service_t *ipcs_shm = NULL;
 
 void send_cib_replace(const xmlNode * sync_request, const char *host);
-static void cib_process_request(xmlNode* request, gboolean force_synchronous,
-                                gboolean privileged, crm_client_t *cib_client);
+static void cib_process_request(xmlNode *request, gboolean privileged,
+                                pcmk__client_t *cib_client);
 
 
 static int cib_process_command(xmlNode *request, xmlNode **reply,
@@ -68,28 +69,22 @@ gboolean cib_legacy_mode(void)
 static int32_t
 cib_ipc_accept(qb_ipcs_connection_t * c, uid_t uid, gid_t gid)
 {
-    crm_trace("Connection %p", c);
     if (cib_shutdown_flag) {
-        crm_info("Ignoring new client [%d] during shutdown", crm_ipcs_client_pid(c));
+        crm_info("Ignoring new IPC client [%d] during shutdown",
+                 pcmk__client_pid(c));
         return -EPERM;
     }
 
-    if (crm_client_new(c, uid, gid) == NULL) {
+    if (pcmk__new_client(c, uid, gid) == NULL) {
         return -EIO;
     }
     return 0;
 }
 
-static void
-cib_ipc_created(qb_ipcs_connection_t * c)
-{
-    crm_trace("Connection %p", c);
-}
-
 static int32_t
 cib_ipc_dispatch_rw(qb_ipcs_connection_t * c, void *data, size_t size)
 {
-    crm_client_t *client = crm_client_get(c);
+    pcmk__client_t *client = pcmk__find_client(c);
 
     crm_trace("%p message from %s", c, client->id);
     return cib_common_callback(c, data, size, TRUE);
@@ -98,7 +93,7 @@ cib_ipc_dispatch_rw(qb_ipcs_connection_t * c, void *data, size_t size)
 static int32_t
 cib_ipc_dispatch_ro(qb_ipcs_connection_t * c, void *data, size_t size)
 {
-    crm_client_t *client = crm_client_get(c);
+    pcmk__client_t *client = pcmk__find_client(c);
 
     crm_trace("%p message from %s", c, client->id);
     return cib_common_callback(c, data, size, FALSE);
@@ -108,13 +103,13 @@ cib_ipc_dispatch_ro(qb_ipcs_connection_t * c, void *data, size_t size)
 static int32_t
 cib_ipc_closed(qb_ipcs_connection_t * c)
 {
-    crm_client_t *client = crm_client_get(c);
+    pcmk__client_t *client = pcmk__find_client(c);
 
     if (client == NULL) {
         return 0;
     }
     crm_trace("Connection %p", c);
-    crm_client_destroy(client);
+    pcmk__free_client(client);
     return 0;
 }
 
@@ -130,7 +125,7 @@ cib_ipc_destroy(qb_ipcs_connection_t * c)
 
 struct qb_ipcs_service_handlers ipc_ro_callbacks = {
     .connection_accept = cib_ipc_accept,
-    .connection_created = cib_ipc_created,
+    .connection_created = NULL,
     .msg_process = cib_ipc_dispatch_ro,
     .connection_closed = cib_ipc_closed,
     .connection_destroyed = cib_ipc_destroy
@@ -138,7 +133,7 @@ struct qb_ipcs_service_handlers ipc_ro_callbacks = {
 
 struct qb_ipcs_service_handlers ipc_rw_callbacks = {
     .connection_accept = cib_ipc_accept,
-    .connection_created = cib_ipc_created,
+    .connection_created = NULL,
     .msg_process = cib_ipc_dispatch_rw,
     .connection_closed = cib_ipc_closed,
     .connection_destroyed = cib_ipc_destroy
@@ -146,26 +141,27 @@ struct qb_ipcs_service_handlers ipc_rw_callbacks = {
 
 void
 cib_common_callback_worker(uint32_t id, uint32_t flags, xmlNode * op_request,
-                           crm_client_t * cib_client, gboolean privileged)
+                           pcmk__client_t *cib_client, gboolean privileged)
 {
     const char *op = crm_element_value(op_request, F_CIB_OPERATION);
 
-    if (crm_str_eq(op, CRM_OP_REGISTER, TRUE)) {
+    if (pcmk__str_eq(op, CRM_OP_REGISTER, pcmk__str_none)) {
         if (flags & crm_ipc_client_response) {
-            xmlNode *ack = create_xml_node(NULL, __FUNCTION__);
+            xmlNode *ack = create_xml_node(NULL, __func__);
 
             crm_xml_add(ack, F_CIB_OPERATION, CRM_OP_REGISTER);
             crm_xml_add(ack, F_CIB_CLIENTID, cib_client->id);
-            crm_ipcs_send(cib_client, id, ack, flags);
+            pcmk__ipc_send_xml(cib_client, id, ack, flags);
             cib_client->request_id = 0;
             free_xml(ack);
         }
         return;
 
-    } else if (crm_str_eq(op, T_CIB_NOTIFY, TRUE)) {
+    } else if (pcmk__str_eq(op, T_CIB_NOTIFY, pcmk__str_none)) {
         /* Update the notify filters for this client */
         int on_off = 0;
-        long long bit = 0;
+        crm_exit_t status = CRM_EX_OK;
+        uint64_t bit = UINT64_C(0);
         const char *type = crm_element_value(op_request, F_CIB_NOTIFY_TYPE);
 
         crm_element_value_int(op_request, F_CIB_NOTIFY_ACTIVATE, &on_off);
@@ -173,36 +169,38 @@ cib_common_callback_worker(uint32_t id, uint32_t flags, xmlNode * op_request,
         crm_debug("Setting %s callbacks for %s (%s): %s",
                   type, cib_client->name, cib_client->id, on_off ? "on" : "off");
 
-        if (safe_str_eq(type, T_CIB_POST_NOTIFY)) {
+        if (pcmk__str_eq(type, T_CIB_POST_NOTIFY, pcmk__str_casei)) {
             bit = cib_notify_post;
 
-        } else if (safe_str_eq(type, T_CIB_PRE_NOTIFY)) {
+        } else if (pcmk__str_eq(type, T_CIB_PRE_NOTIFY, pcmk__str_casei)) {
             bit = cib_notify_pre;
 
-        } else if (safe_str_eq(type, T_CIB_UPDATE_CONFIRM)) {
+        } else if (pcmk__str_eq(type, T_CIB_UPDATE_CONFIRM, pcmk__str_casei)) {
             bit = cib_notify_confirm;
 
-        } else if (safe_str_eq(type, T_CIB_DIFF_NOTIFY)) {
+        } else if (pcmk__str_eq(type, T_CIB_DIFF_NOTIFY, pcmk__str_casei)) {
             bit = cib_notify_diff;
 
-        } else if (safe_str_eq(type, T_CIB_REPLACE_NOTIFY)) {
+        } else if (pcmk__str_eq(type, T_CIB_REPLACE_NOTIFY, pcmk__str_casei)) {
             bit = cib_notify_replace;
-        }
 
-        if (on_off) {
-            set_bit(cib_client->options, bit);
         } else {
-            clear_bit(cib_client->options, bit);
+            status = CRM_EX_INVALID_PARAM;
         }
 
-        if (flags & crm_ipc_client_response) {
-            /* TODO - include rc */
-            crm_ipcs_send_ack(cib_client, id, flags, "ack", __FUNCTION__, __LINE__);
+        if (bit != 0) {
+            if (on_off) {
+                pcmk__set_client_flags(cib_client, bit);
+            } else {
+                pcmk__clear_client_flags(cib_client, bit);
+            }
         }
+
+        pcmk__ipc_send_ack(cib_client, id, flags, "ack", status);
         return;
     }
 
-    cib_process_request(op_request, FALSE, privileged, cib_client);
+    cib_process_request(op_request, privileged, cib_client);
 }
 
 int32_t
@@ -211,8 +209,8 @@ cib_common_callback(qb_ipcs_connection_t * c, void *data, size_t size, gboolean
     uint32_t id = 0;
     uint32_t flags = 0;
     int call_options = 0;
-    crm_client_t *cib_client = crm_client_get(c);
-    xmlNode *op_request = crm_ipcs_recv(cib_client, data, size, &id, &flags);
+    pcmk__client_t *cib_client = pcmk__find_client(c);
+    xmlNode *op_request = pcmk__client_data2xml(cib_client, data, &id, &flags);
 
     if (op_request) {
         crm_element_value_int(op_request, F_CIB_CALLOPTS, &call_options);
@@ -220,7 +218,7 @@ cib_common_callback(qb_ipcs_connection_t * c, void *data, size_t size, gboolean
 
     if (op_request == NULL) {
         crm_trace("Invalid message from %p", c);
-        crm_ipcs_send_ack(cib_client, id, flags, "nack", __FUNCTION__, __LINE__);
+        pcmk__ipc_send_ack(cib_client, id, flags, "nack", CRM_EX_PROTOCOL);
         return 0;
 
     } else if(cib_client == NULL) {
@@ -228,7 +226,7 @@ cib_common_callback(qb_ipcs_connection_t * c, void *data, size_t size, gboolean
         return 0;
     }
 
-    if (is_set(call_options, cib_sync_call)) {
+    if (pcmk_is_set(call_options, cib_sync_call)) {
         CRM_LOG_ASSERT(flags & crm_ipc_client_response);
         CRM_LOG_ASSERT(cib_client->request_id == 0);    /* This means the client has two synchronous events in-flight */
         cib_client->request_id = id;    /* Reply only to the last one */
@@ -242,16 +240,16 @@ cib_common_callback(qb_ipcs_connection_t * c, void *data, size_t size, gboolean
         } else {
             cib_client->name = strdup(value);
             if (crm_is_daemon_name(value)) {
-                set_bit(cib_client->options, cib_is_daemon);
+                pcmk__set_client_flags(cib_client, cib_is_daemon);
             }
         }
     }
 
     /* Allow cluster daemons more leeway before being evicted */
-    if (is_set(cib_client->options, cib_is_daemon)) {
+    if (pcmk_is_set(cib_client->flags, cib_is_daemon)) {
         const char *qmax = cib_config_lookup("cluster-ipc-limit");
 
-        if (crm_set_client_queue_max(cib_client, qmax)) {
+        if (pcmk__set_client_queue_max(cib_client, qmax)) {
             crm_trace("IPC threshold for %s[%u] is now %u",
                       cib_client->name, cib_client->pid, cib_client->queue_max);
         }
@@ -262,11 +260,9 @@ cib_common_callback(qb_ipcs_connection_t * c, void *data, size_t size, gboolean
 
 #if ENABLE_ACL
     CRM_LOG_ASSERT(cib_client->user != NULL);
-    crm_acl_get_set_user(op_request, F_CIB_USER, cib_client->user);
+    pcmk__update_acl_user(op_request, F_CIB_USER, cib_client->user);
 #endif
 
-    crm_log_xml_trace(op_request, "Client[inbound]");
-
     cib_common_callback_worker(id, flags, op_request, cib_client, privileged);
     free_xml(op_request);
 
@@ -315,7 +311,7 @@ process_ping_reply(xmlNode *reply)
     const char *digest = crm_element_value(pong, XML_ATTR_DIGEST);
 
     if (seq_s) {
-        seq = crm_int_helper(seq_s, NULL);
+        seq = (uint64_t) crm_parse_ll(seq_s, NULL);
     }
 
     if(digest == NULL) {
@@ -336,7 +332,7 @@ process_ping_reply(xmlNode *reply)
         }
 
         crm_trace("Processing ping reply %s from %s (%s)", seq_s, host, digest);
-        if(safe_str_eq(ping_digest, digest) == FALSE) {
+        if (!pcmk__str_eq(ping_digest, digest, pcmk__str_casei)) {
             xmlNode *remote_cib = get_message_xml(pong, F_CIB_CALLDATA);
 
             crm_notice("Local CIB %s.%s.%s.%s differs from %s: %s.%s.%s.%s %p",
@@ -352,7 +348,7 @@ process_ping_reply(xmlNode *reply)
             if(remote_cib && remote_cib->children) {
                 /* Additional debug */
                 xml_calculate_changes(the_cib, remote_cib);
-                xml_log_changes(LOG_INFO, __FUNCTION__, remote_cib);
+                xml_log_changes(LOG_INFO, __func__, remote_cib);
                 crm_trace("End of differences");
             }
 
@@ -368,14 +364,13 @@ do_local_notify(xmlNode * notify_src, const char *client_id,
 {
     int rid = 0;
     int call_id = 0;
-    ssize_t rc = 0;
-    crm_client_t *client_obj = NULL;
+    pcmk__client_t *client_obj = NULL;
 
     CRM_ASSERT(notify_src && client_id);
 
     crm_element_value_int(notify_src, F_CIB_CALLID, &call_id);
 
-    client_obj = crm_client_get_by_id(client_id);
+    client_obj = pcmk__find_client_by_id(client_id);
     if (client_obj == NULL) {
         crm_debug("Could not send response %d: client %s not found",
                   call_id, client_id);
@@ -402,24 +397,30 @@ do_local_notify(xmlNode * notify_src, const char *client_id,
                   call_id, client_obj->name, from_peer ? "(originator of delegated request)" : "");
     }
 
-    switch (client_obj->kind) {
-        case CRM_CLIENT_IPC:
-            rc = crm_ipcs_send(client_obj, rid, notify_src, (sync_reply?
-                               crm_ipc_flags_none : crm_ipc_server_event));
-            if (rc < 0) {
-                crm_warn("%s reply to %s failed: %s " CRM_XS " rc=%lld",
-                         (sync_reply? "Synchronous" : "Asynchronous"),
-                         client_obj->name, pcmk_strerror(rc), (long long) rc);
+    switch (PCMK__CLIENT_TYPE(client_obj)) {
+        case pcmk__client_ipc:
+            {
+                int rc = pcmk__ipc_send_xml(client_obj, rid, notify_src,
+                                            (sync_reply? crm_ipc_flags_none
+                                             : crm_ipc_server_event));
+
+                if (rc != pcmk_rc_ok) {
+                    crm_warn("%s reply to %s failed: %s " CRM_XS " rc=%d",
+                             (sync_reply? "Synchronous" : "Asynchronous"),
+                             client_obj->name, pcmk_rc_str(rc), rc);
+                }
             }
             break;
 #ifdef HAVE_GNUTLS_GNUTLS_H
-        case CRM_CLIENT_TLS:
+        case pcmk__client_tls:
 #endif
-        case CRM_CLIENT_TCP:
-            crm_remote_send(client_obj->remote, notify_src);
+        case pcmk__client_tcp:
+            pcmk__remote_send_xml(client_obj->remote, notify_src);
             break;
         default:
-            crm_err("Unknown transport %d for %s", client_obj->kind, client_obj->name);
+            crm_err("Unknown transport for %s " CRM_XS " flags=0x%llx",
+                    pcmk__client_name(client_obj),
+                    (unsigned long long) client_obj->flags);
     }
 }
 
@@ -469,12 +470,15 @@ queue_local_notify(xmlNode * notify_src, const char *client_id, gboolean sync_re
     }
 
     g_hash_table_insert(local_notify_queue, GINT_TO_POINTER(cib_local_bcast_num), notify);
+    // cppcheck doesn't know notify will get freed when hash table is destroyed
+    // cppcheck-suppress memleak
 }
 
 static void
-parse_local_options_v1(crm_client_t * cib_client, int call_type, int call_options, const char *host,
-                    const char *op, gboolean * local_notify, gboolean * needs_reply,
-                    gboolean * process, gboolean * needs_forward)
+parse_local_options_v1(pcmk__client_t *cib_client, int call_type,
+                       int call_options, const char *host, const char *op,
+                       gboolean *local_notify, gboolean *needs_reply,
+                       gboolean *process, gboolean *needs_forward)
 {
     if (cib_op_modifies(call_type)
         && !(call_options & cib_inhibit_bcast)) {
@@ -492,7 +496,7 @@ parse_local_options_v1(crm_client_t * cib_client, int call_type, int call_option
         crm_trace("Processing master %s op locally from %s", op, cib_client->name);
         *local_notify = TRUE;
 
-    } else if (safe_str_eq(host, cib_our_uname)) {
+    } else if (pcmk__str_eq(host, cib_our_uname, pcmk__str_casei)) {
         crm_trace("Processing locally addressed %s op from %s", op, cib_client->name);
         *local_notify = TRUE;
 
@@ -510,12 +514,13 @@ parse_local_options_v1(crm_client_t * cib_client, int call_type, int call_option
 }
 
 static void
-parse_local_options_v2(crm_client_t * cib_client, int call_type, int call_options, const char *host,
-                    const char *op, gboolean * local_notify, gboolean * needs_reply,
-                    gboolean * process, gboolean * needs_forward)
+parse_local_options_v2(pcmk__client_t *cib_client, int call_type,
+                       int call_options, const char *host, const char *op,
+                       gboolean *local_notify, gboolean *needs_reply,
+                       gboolean *process, gboolean *needs_forward)
 {
     if (cib_op_modifies(call_type)) {
-        if(safe_str_eq(op, CIB_OP_MASTER) || safe_str_eq(op, CIB_OP_SLAVE)) {
+        if (pcmk__strcase_any_of(op, CIB_OP_MASTER, CIB_OP_SLAVE, NULL)) {
             /* Always handle these locally */
             *process = TRUE;
             *needs_reply = FALSE;
@@ -546,7 +551,7 @@ parse_local_options_v2(crm_client_t * cib_client, int call_type, int call_option
     } else if (host == NULL) {
         crm_trace("Processing unaddressed %s op from %s", op, cib_client->name);
 
-    } else if (safe_str_eq(host, cib_our_uname)) {
+    } else if (pcmk__str_eq(host, cib_our_uname, pcmk__str_casei)) {
         crm_trace("Processing locally addressed %s op from %s", op, cib_client->name);
 
     } else {
@@ -557,9 +562,10 @@ parse_local_options_v2(crm_client_t * cib_client, int call_type, int call_option
 }
 
 static void
-parse_local_options(crm_client_t * cib_client, int call_type, int call_options, const char *host,
-                    const char *op, gboolean * local_notify, gboolean * needs_reply,
-                    gboolean * process, gboolean * needs_forward)
+parse_local_options(pcmk__client_t *cib_client, int call_type,
+                    int call_options, const char *host, const char *op,
+                    gboolean *local_notify, gboolean *needs_reply,
+                    gboolean *process, gboolean *needs_forward)
 {
     if(cib_legacy_mode()) {
         parse_local_options_v1(cib_client, call_type, call_options, host,
@@ -582,7 +588,7 @@ parse_peer_options_v1(int call_type, xmlNode * request,
     const char *reply_to = crm_element_value(request, F_CIB_ISREPLY);
     const char *update = crm_element_value(request, F_CIB_GLOBAL_UPDATE);
 
-    gboolean is_reply = safe_str_eq(reply_to, cib_our_uname);
+    gboolean is_reply = pcmk__str_eq(reply_to, cib_our_uname, pcmk__str_casei);
 
     if (crm_is_true(update)) {
         *needs_reply = FALSE;
@@ -598,7 +604,7 @@ parse_peer_options_v1(int call_type, xmlNode * request,
 
     op = crm_element_value(request, F_CIB_OPERATION);
     crm_trace("Processing %s request sent by %s", op, originator);
-    if (safe_str_eq(op, "cib_shutdown_req")) {
+    if (pcmk__str_eq(op, "cib_shutdown_req", pcmk__str_casei)) {
         /* Always process these */
         *local_notify = FALSE;
         if (reply_to == NULL || is_reply) {
@@ -610,7 +616,7 @@ parse_peer_options_v1(int call_type, xmlNode * request,
         return *process;
     }
 
-    if (is_reply && safe_str_eq(op, CRM_OP_PING)) {
+    if (is_reply && pcmk__str_eq(op, CRM_OP_PING, pcmk__str_casei)) {
         process_ping_reply(request);
         return FALSE;
     }
@@ -624,11 +630,11 @@ parse_peer_options_v1(int call_type, xmlNode * request,
     }
 
     host = crm_element_value(request, F_CIB_HOST);
-    if (host != NULL && safe_str_eq(host, cib_our_uname)) {
+    if (host != NULL && pcmk__str_eq(host, cib_our_uname, pcmk__str_casei)) {
         crm_trace("Processing %s request sent to us from %s", op, originator);
         return TRUE;
 
-    } else if(is_reply == FALSE && safe_str_eq(op, CRM_OP_PING)) {
+    } else if(is_reply == FALSE && pcmk__str_eq(op, CRM_OP_PING, pcmk__str_casei)) {
         crm_trace("Processing %s request sent to %s by %s", op, host?host:"everyone", originator);
         *needs_reply = TRUE;
         return TRUE;
@@ -650,7 +656,7 @@ parse_peer_options_v1(int call_type, xmlNode * request,
         /* this is for the master instance and we're not it */
         crm_trace("Ignoring reply to %s", crm_str(reply_to));
 
-    } else if (safe_str_eq(op, "cib_shutdown_req")) {
+    } else if (pcmk__str_eq(op, "cib_shutdown_req", pcmk__str_casei)) {
         if (reply_to != NULL) {
             crm_debug("Processing %s from %s", op, originator);
             *needs_reply = FALSE;
@@ -680,22 +686,22 @@ parse_peer_options_v2(int call_type, xmlNode * request,
     const char *reply_to = crm_element_value(request, F_CIB_ISREPLY);
     const char *update = crm_element_value(request, F_CIB_GLOBAL_UPDATE);
 
-    gboolean is_reply = safe_str_eq(reply_to, cib_our_uname);
+    gboolean is_reply = pcmk__str_eq(reply_to, cib_our_uname, pcmk__str_casei);
 
-    if(safe_str_eq(op, CIB_OP_REPLACE)) {
+    if(pcmk__str_eq(op, CIB_OP_REPLACE, pcmk__str_casei)) {
         /* sync_our_cib() sets F_CIB_ISREPLY */
         if (reply_to) {
             delegated = reply_to;
         }
         goto skip_is_reply;
 
-    } else if(safe_str_eq(op, CIB_OP_SYNC)) {
+    } else if(pcmk__str_eq(op, CIB_OP_SYNC, pcmk__str_casei)) {
 
-    } else if (is_reply && safe_str_eq(op, CRM_OP_PING)) {
+    } else if (is_reply && pcmk__str_eq(op, CRM_OP_PING, pcmk__str_casei)) {
         process_ping_reply(request);
         return FALSE;
 
-    } else if (safe_str_eq(op, CIB_OP_UPGRADE)) {
+    } else if (pcmk__str_eq(op, CIB_OP_UPGRADE, pcmk__str_casei)) {
         /* Only the DC (node with the oldest software) should process
          * this operation if F_CIB_SCHEMA_MAX is unset
          *
@@ -741,7 +747,7 @@ parse_peer_options_v2(int call_type, xmlNode * request,
         crm_trace("Ignoring legacy %s reply sent from %s to local clients", op, originator);
         return FALSE;
 
-    } else if (safe_str_eq(op, "cib_shutdown_req")) {
+    } else if (pcmk__str_eq(op, "cib_shutdown_req", pcmk__str_casei)) {
         /* Legacy handling */
         crm_debug("Legacy handling of %s message from %s", op, originator);
         *local_notify = FALSE;
@@ -763,14 +769,14 @@ parse_peer_options_v2(int call_type, xmlNode * request,
     *process = TRUE;
     *needs_reply = FALSE;
 
-    if(safe_str_eq(delegated, cib_our_uname)) {
+    if(pcmk__str_eq(delegated, cib_our_uname, pcmk__str_casei)) {
         *local_notify = TRUE;
     } else {
         *local_notify = FALSE;
     }
 
     host = crm_element_value(request, F_CIB_HOST);
-    if (host != NULL && safe_str_eq(host, cib_our_uname)) {
+    if (host != NULL && pcmk__str_eq(host, cib_our_uname, pcmk__str_casei)) {
         crm_trace("Processing %s request sent to us from %s", op, originator);
         *needs_reply = TRUE;
         return TRUE;
@@ -780,7 +786,7 @@ parse_peer_options_v2(int call_type, xmlNode * request,
         crm_trace("Ignoring %s operation for instance on %s", op, crm_str(host));
         return FALSE;
 
-    } else if(is_reply == FALSE && safe_str_eq(op, CRM_OP_PING)) {
+    } else if(is_reply == FALSE && pcmk__str_eq(op, CRM_OP_PING, pcmk__str_casei)) {
         *needs_reply = TRUE;
     }
 
@@ -810,7 +816,7 @@ parse_peer_options(int call_type, xmlNode * request,
 }
 
 static void
-forward_request(xmlNode * request, crm_client_t * cib_client, int call_options)
+forward_request(xmlNode * request, pcmk__client_t *cib_client, int call_options)
 {
     const char *op = crm_element_value(request, F_CIB_OPERATION);
     const char *host = crm_element_value(request, F_CIB_HOST);
@@ -890,19 +896,27 @@ send_peer_reply(xmlNode * msg, xmlNode * result_diff, const char *originator, gb
     return FALSE;
 }
 
+/*!
+ * \internal
+ * \brief Handle an IPC or CPG message containing a request
+ *
+ * \param[in] request            Request XML
+ * \param[in] privileged         Whether privileged commands may be run
+ *                               (see cib_server_ops[] definition)
+ * \param[in] cib_client         IPC client that sent request (or NULL if CPG)
+ */
 static void
-cib_process_request(xmlNode *request, gboolean force_synchronous,
-                    gboolean privileged, crm_client_t *cib_client)
+cib_process_request(xmlNode *request, gboolean privileged,
+                    pcmk__client_t *cib_client)
 {
     int call_type = 0;
     int call_options = 0;
 
-    gboolean process = TRUE;
-    gboolean is_update = TRUE;
-    gboolean from_peer = TRUE;
-    gboolean needs_reply = TRUE;
-    gboolean local_notify = FALSE;
-    gboolean needs_forward = FALSE;
+    gboolean process = TRUE;        // Whether to process request locally now
+    gboolean is_update = TRUE;      // Whether request would modify CIB
+    gboolean needs_reply = TRUE;    // Whether to build a reply
+    gboolean local_notify = FALSE;  // Whether to notify (local) requester
+    gboolean needs_forward = FALSE; // Whether to forward request somewhere else
     gboolean global_update = crm_is_true(crm_element_value(request, F_CIB_GLOBAL_UPDATE));
 
     xmlNode *op_reply = NULL;
@@ -918,16 +932,9 @@ cib_process_request(xmlNode *request, gboolean force_synchronous,
     const char *client_name = crm_element_value(request, F_CIB_CLIENTNAME);
     const char *reply_to = crm_element_value(request, F_CIB_ISREPLY);
 
-    if (cib_client) {
-        from_peer = FALSE;
-    }
-
     crm_element_value_int(request, F_CIB_CALLOPTS, &call_options);
-    if (force_synchronous) {
-        call_options |= cib_sync_call;
-    }
 
-    if (host != NULL && strlen(host) == 0) {
+    if ((host != NULL) && (*host == '\0')) {
         host = NULL;
     }
 
@@ -941,7 +948,7 @@ cib_process_request(xmlNode *request, gboolean force_synchronous,
         target = "master";
     }
 
-    if (from_peer) {
+    if (cib_client == NULL) {
         crm_trace("Processing peer %s operation from %s/%s on %s intended for %s (reply=%s)",
                   op, client_name, call_id, originator, target, reply_to);
     } else {
@@ -956,7 +963,7 @@ cib_process_request(xmlNode *request, gboolean force_synchronous,
         return;
     }
 
-    if (from_peer == FALSE) {
+    if (cib_client != NULL) {
         parse_local_options(cib_client, call_type, call_options, host, op,
                             &local_notify, &needs_reply, &process, &needs_forward);
 
@@ -968,16 +975,19 @@ cib_process_request(xmlNode *request, gboolean force_synchronous,
     is_update = cib_op_modifies(call_type);
 
     if (call_options & cib_discard_reply) {
-        needs_reply = is_update;
+        /* If the request will modify the CIB, and we are in legacy mode, we
+         * need to build a reply so we can broadcast a diff, even if the
+         * requester doesn't want one.
+         */
+        needs_reply = is_update && cib_legacy_mode();
         local_notify = FALSE;
     }
 
     if (needs_forward) {
-        const char *host = crm_element_value(request, F_CIB_HOST);
         const char *section = crm_element_value(request, F_CIB_SECTION);
         int log_level = LOG_INFO;
 
-        if (safe_str_eq(op, CRM_OP_NOOP)) {
+        if (pcmk__str_eq(op, CRM_OP_NOOP, pcmk__str_casei)) {
             log_level = LOG_DEBUG;
         }
 
@@ -1015,14 +1025,13 @@ cib_process_request(xmlNode *request, gboolean force_synchronous,
 
     } else if (process) {
         time_t finished = 0;
-
         time_t now = time(NULL);
         int level = LOG_INFO;
         const char *section = crm_element_value(request, F_CIB_SECTION);
 
         rc = cib_process_command(request, &op_reply, &result_diff, privileged);
 
-        if (is_update == FALSE) {
+        if (!is_update) {
             level = LOG_TRACE;
 
         } else if (global_update) {
@@ -1039,7 +1048,7 @@ cib_process_request(xmlNode *request, gboolean force_synchronous,
                     level = LOG_ERR;
             }
 
-        } else if (rc != pcmk_ok && is_update) {
+        } else if (rc != pcmk_ok) {
             level = LOG_WARNING;
         }
 
@@ -1065,16 +1074,13 @@ cib_process_request(xmlNode *request, gboolean force_synchronous,
         }
     }
 
-    /* from now on we are the server */
-    if(is_update && cib_legacy_mode() == FALSE) {
+    if (is_update && !cib_legacy_mode()) {
         crm_trace("Completed pre-sync update from %s/%s/%s%s",
                   originator ? originator : "local", client_name, call_id,
                   local_notify?" with local notification":"");
 
-    } else if (needs_reply == FALSE || stand_alone) {
-        /* nothing more to do...
-         * this was a non-originating slave update
-         */
+    } else if (!needs_reply || stand_alone) {
+        // This was a non-originating slave update
         crm_trace("Completed slave update");
 
     } else if (cib_legacy_mode() &&
@@ -1094,14 +1100,16 @@ cib_process_request(xmlNode *request, gboolean force_synchronous,
             crm_trace("Queuing local %ssync notification for %s",
                       (call_options & cib_sync_call) ? "" : "a-", client_id);
 
-            queue_local_notify(op_reply, client_id, (call_options & cib_sync_call), from_peer);
+            queue_local_notify(op_reply, client_id,
+                               pcmk_is_set(call_options, cib_sync_call),
+                               (cib_client == NULL));
             op_reply = NULL;    /* the reply is queued, so don't free here */
         }
 
     } else if (call_options & cib_discard_reply) {
         crm_trace("Caller isn't interested in reply");
 
-    } else if (from_peer) {
+    } else if (cib_client == NULL) {
         if (is_update == FALSE || result_diff == NULL) {
             crm_trace("Request not broadcast: R/O call");
 
@@ -1120,11 +1128,16 @@ cib_process_request(xmlNode *request, gboolean force_synchronous,
 
     if (local_notify && client_id) {
         crm_trace("Performing local %ssync notification for %s",
-                  (call_options & cib_sync_call) ? "" : "a-", client_id);
+                  (pcmk_is_set(call_options, cib_sync_call)? "" : "a"),
+                  client_id);
         if (process == FALSE) {
-            do_local_notify(request, client_id, call_options & cib_sync_call, from_peer);
+            do_local_notify(request, client_id,
+                            pcmk_is_set(call_options, cib_sync_call),
+                            (cib_client == NULL));
         } else {
-            do_local_notify(op_reply, client_id, call_options & cib_sync_call, from_peer);
+            do_local_notify(op_reply, client_id,
+                            pcmk_is_set(call_options, cib_sync_call),
+                            (cib_client == NULL));
         }
     }
 
@@ -1201,7 +1214,7 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
     if (global_update) {
         /* legacy code */
         manage_counters = FALSE;
-        call_options |= cib_force_diff;
+        cib__set_call_options(call_options, "call", cib_force_diff);
         crm_trace("Global update detected");
 
         CRM_CHECK(call_type == 3 || call_type == 4, crm_err("Call type: %d", call_type);
@@ -1216,11 +1229,12 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
             manage_counters = FALSE;
         }
 
-        if (is_not_set(call_options, cib_dryrun) && safe_str_eq(section, XML_CIB_TAG_STATUS)) {
+        if (!pcmk_is_set(call_options, cib_dryrun)
+            && pcmk__str_eq(section, XML_CIB_TAG_STATUS, pcmk__str_casei)) {
             /* Copying large CIBs accounts for a huge percentage of our CIB usage */
-            call_options |= cib_zero_copy;
+            cib__set_call_options(call_options, "call", cib_zero_copy);
         } else {
-            clear_bit(call_options, cib_zero_copy);
+            cib__clear_call_options(call_options, "call", cib_zero_copy);
         }
 
         /* result_cib must not be modified after cib_perform_op() returns */
@@ -1245,18 +1259,18 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
         /* Always write to disk for replace ops,
          * this also negates the need to detect ordering changes
          */
-        if (crm_str_eq(CIB_OP_REPLACE, op, TRUE)) {
+        if (pcmk__str_eq(CIB_OP_REPLACE, op, pcmk__str_none)) {
             config_changed = TRUE;
         }
     }
 
-    if (rc == pcmk_ok && is_not_set(call_options, cib_dryrun)) {
+    if (rc == pcmk_ok && !pcmk_is_set(call_options, cib_dryrun)) {
         crm_trace("Activating %s->%s%s%s",
                   crm_element_value(current_cib, XML_ATTR_NUMUPDATES),
                   crm_element_value(result_cib, XML_ATTR_NUMUPDATES),
-                  (is_set(call_options, cib_zero_copy)? " zero-copy" : ""),
+                  (pcmk_is_set(call_options, cib_zero_copy)? " zero-copy" : ""),
                   (config_changed? " changed" : ""));
-        if(is_not_set(call_options, cib_zero_copy)) {
+        if (!pcmk_is_set(call_options, cib_zero_copy)) {
             rc = activateCibXml(result_cib, config_changed, op);
             crm_trace("Activated %s (%d)",
                       crm_element_value(current_cib, XML_ATTR_NUMUPDATES), rc);
@@ -1266,21 +1280,24 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
             cib_read_config(config_hash, result_cib);
         }
 
-        if (crm_str_eq(CIB_OP_REPLACE, op, TRUE)) {
+        if (pcmk__str_eq(CIB_OP_REPLACE, op, pcmk__str_none)) {
             if (section == NULL) {
                 send_r_notify = TRUE;
 
-            } else if (safe_str_eq(section, XML_TAG_CIB)) {
+            } else if (pcmk__str_eq(section, XML_TAG_CIB, pcmk__str_casei)) {
                 send_r_notify = TRUE;
 
-            } else if (safe_str_eq(section, XML_CIB_TAG_NODES)) {
+            } else if (pcmk__str_eq(section, XML_CIB_TAG_NODES, pcmk__str_casei)) {
                 send_r_notify = TRUE;
 
-            } else if (safe_str_eq(section, XML_CIB_TAG_STATUS)) {
+            } else if (pcmk__str_eq(section, XML_CIB_TAG_STATUS, pcmk__str_casei)) {
+                send_r_notify = TRUE;
+
+            } else if (pcmk__str_eq(section, XML_CIB_TAG_CONFIGURATION, pcmk__str_casei)) {
                 send_r_notify = TRUE;
             }
 
-        } else if (crm_str_eq(CIB_OP_ERASE, op, TRUE)) {
+        } else if (pcmk__str_eq(CIB_OP_ERASE, op, pcmk__str_none)) {
             send_r_notify = TRUE;
         }
 
@@ -1288,7 +1305,7 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
         mainloop_timer_start(digest_timer);
 
     } else if (rc == -pcmk_err_schema_validation) {
-        CRM_ASSERT(is_not_set(call_options, cib_zero_copy));
+        CRM_ASSERT(!pcmk_is_set(call_options, cib_zero_copy));
 
         if (output != NULL) {
             crm_log_xml_info(output, "cib:output");
@@ -1298,8 +1315,10 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
         output = result_cib;
 
     } else {
-        crm_trace("Not activating %d %d %s", rc, is_set(call_options, cib_dryrun), crm_element_value(result_cib, XML_ATTR_NUMUPDATES));
-        if(is_not_set(call_options, cib_zero_copy)) {
+        crm_trace("Not activating %d %d %s", rc,
+                  pcmk_is_set(call_options, cib_dryrun),
+                  crm_element_value(result_cib, XML_ATTR_NUMUPDATES));
+        if (!pcmk_is_set(call_options, cib_zero_copy)) {
             free_xml(result_cib);
         }
     }
@@ -1307,7 +1326,8 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
     if ((call_options & (cib_inhibit_notify|cib_dryrun)) == 0) {
         const char *client = crm_element_value(request, F_CIB_CLIENTNAME);
 
-        crm_trace("Sending notifications %d", is_set(call_options, cib_dryrun));
+        crm_trace("Sending notifications %d",
+                  pcmk_is_set(call_options, cib_dryrun));
         cib_diff_notify(call_options, client, call_id, op, input, rc, *cib_diff);
     }
 
@@ -1319,7 +1339,7 @@ cib_process_command(xmlNode * request, xmlNode ** reply, xmlNode ** cib_diff, gb
 
     xml_log_patchset(LOG_TRACE, "cib:diff", *cib_diff);
   done:
-    if ((call_options & cib_discard_reply) == 0) {
+    if (!pcmk_is_set(call_options, cib_discard_reply) || cib_legacy_mode()) {
         const char *caller = crm_element_value(request, F_CIB_CLIENTID);
 
         *reply = create_xml_node(NULL, "cib-reply");
@@ -1359,7 +1379,7 @@ cib_peer_callback(xmlNode * msg, void *private_data)
     const char *reason = NULL;
     const char *originator = crm_element_value(msg, F_ORIG);
 
-    if (cib_legacy_mode() && (originator == NULL || crm_str_eq(originator, cib_our_uname, TRUE))) {
+    if (cib_legacy_mode() && pcmk__str_eq(originator, cib_our_uname, pcmk__str_null_matches)) {
         /* message is from ourselves */
         int bcast_id = 0;
 
@@ -1378,7 +1398,7 @@ cib_peer_callback(xmlNode * msg, void *private_data)
     }
 
     /* crm_log_xml_trace("Peer[inbound]", msg); */
-    cib_process_request(msg, FALSE, TRUE, NULL);
+    cib_process_request(msg, TRUE, NULL);
     return;
 
   bail:
@@ -1394,14 +1414,14 @@ static gboolean
 cib_force_exit(gpointer data)
 {
     crm_notice("Forcing exit!");
-    terminate_cib(__FUNCTION__, CRM_EX_ERROR);
+    terminate_cib(__func__, CRM_EX_ERROR);
     return FALSE;
 }
 
 static void
 disconnect_remote_client(gpointer key, gpointer value, gpointer user_data)
 {
-    crm_client_t *a_client = value;
+    pcmk__client_t *a_client = value;
 
     crm_err("Disconnecting %s... Not implemented", crm_str(a_client->name));
 }
@@ -1453,22 +1473,22 @@ cib_shutdown(int nsig)
             disconnects++;
         }
 
-        disconnects += crm_hash_table_size(client_connections);
+        disconnects += pcmk__ipc_client_count();
 
-        crm_debug("Disconnecting %d remote clients", crm_hash_table_size(client_connections));
-        g_hash_table_foreach(client_connections, disconnect_remote_client, NULL);
+        crm_debug("Disconnecting %d remote clients", pcmk__ipc_client_count());
+        pcmk__foreach_ipc_client(disconnect_remote_client, NULL);
         crm_info("Disconnected %d clients", disconnects);
     }
 
     qb_ipcs_stats_get(ipcs_rw, &srv_stats, QB_FALSE);
 
-    if (crm_hash_table_size(client_connections) == 0) {
+    if (pcmk__ipc_client_count() == 0) {
         crm_info("All clients disconnected (%d)", srv_stats.active_connections);
         initiate_exit();
 
     } else {
         crm_info("Waiting on %d clients to disconnect (%d)",
-                 crm_hash_table_size(client_connections), srv_stats.active_connections);
+                 pcmk__ipc_client_count(), srv_stats.active_connections);
     }
 }
 
@@ -1480,7 +1500,7 @@ initiate_exit(void)
 
     active = crm_active_peers();
     if (active < 2) {
-        terminate_cib(__FUNCTION__, 0);
+        terminate_cib(__func__, 0);
         return;
     }
 
@@ -1525,7 +1545,7 @@ terminate_cib(const char *caller, int fast)
 
     if (fast > 0) {
         /* Quit fast on error */
-        cib_ipc_servers_destroy(ipcs_ro, ipcs_rw, ipcs_shm);
+        pcmk__stop_based_ipc(ipcs_ro, ipcs_rw, ipcs_shm);
         crm_exit(fast);
 
     } else if ((mainloop != NULL) && g_main_loop_is_running(mainloop)) {
@@ -1543,7 +1563,7 @@ terminate_cib(const char *caller, int fast)
         /* Quit via clean exit. Even the peer status callback can disconnect
          * here, because we're not returning control to the caller. */
         crm_cluster_disconnect(&crm_cluster);
-        cib_ipc_servers_destroy(ipcs_ro, ipcs_rw, ipcs_shm);
+        pcmk__stop_based_ipc(ipcs_ro, ipcs_rw, ipcs_shm);
         crm_exit(CRM_EX_OK);
     }
 }
