@@ -1,5 +1,7 @@
 /*
- * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2020 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU General Public License version 2
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
@@ -22,7 +24,8 @@
 #include <crm/msg_xml.h>
 
 #include <crm/common/xml.h>
-#include <crm/common/ipcs.h>
+#include <crm/common/ipc_internal.h>
+#include <crm/common/xml_internal.h>
 #include <crm/cluster/internal.h>
 
 #include <pacemaker-based.h>
@@ -65,7 +68,7 @@ cib_process_shutdown_req(const char *op, int options, const char *section, xmlNo
     }
 
     crm_info("Peer %s has acknowledged our shutdown request", host);
-    terminate_cib(__FUNCTION__, 0);
+    terminate_cib(__func__, 0);
     return pcmk_ok;
 }
 
@@ -102,7 +105,7 @@ cib_process_readwrite(const char *op, int options, const char *section, xmlNode
 
     crm_trace("Processing \"%s\" event", op);
 
-    if (safe_str_eq(op, CIB_OP_ISMASTER)) {
+    if (pcmk__str_eq(op, CIB_OP_ISMASTER, pcmk__str_casei)) {
         if (cib_is_master == TRUE) {
             result = pcmk_ok;
         } else {
@@ -111,7 +114,7 @@ cib_process_readwrite(const char *op, int options, const char *section, xmlNode
         return result;
     }
 
-    if (safe_str_eq(op, CIB_OP_MASTER)) {
+    if (pcmk__str_eq(op, CIB_OP_MASTER, pcmk__str_casei)) {
         if (cib_is_master == FALSE) {
             crm_info("We are now in R/W mode");
             cib_is_master = TRUE;
@@ -166,7 +169,8 @@ cib_process_ping(const char *op, int options, const char *section, xmlNode * req
     crm_xml_add(*answer, F_CIB_PING_ID, seq);
 
     if (cs == NULL) {
-        cs = qb_log_callsite_get(__func__, __FILE__, __FUNCTION__, LOG_TRACE, __LINE__, crm_trace_nonlog);
+        cs = qb_log_callsite_get(__func__, __FILE__, __func__, LOG_TRACE,
+                                 __LINE__, crm_trace_nonlog);
     }
     if (cs && cs->targets) {
         /* Append additional detail so the reciever can log the differences */
@@ -236,7 +240,7 @@ cib_process_upgrade_server(const char *op, int options, const char *section, xml
 
         rc = update_validation(&scratch, &new_version, 0, TRUE, TRUE);
         if (new_version > current_version) {
-            xmlNode *up = create_xml_node(NULL, __FUNCTION__);
+            xmlNode *up = create_xml_node(NULL, __func__);
 
             rc = pcmk_ok;
             crm_notice("Upgrade request from %s verified", host);
@@ -272,7 +276,7 @@ cib_process_upgrade_server(const char *op, int options, const char *section, xml
                      (origin? origin->uname : "lost"));
 
             if (origin) {
-                xmlNode *up = create_xml_node(NULL, __FUNCTION__);
+                xmlNode *up = create_xml_node(NULL, __func__);
 
                 crm_xml_add(up, F_TYPE, "cib");
                 crm_xml_add(up, F_CIB_OPERATION, CIB_OP_UPGRADE);
@@ -354,7 +358,7 @@ cib_server_process_diff(const char *op, int options, const char *section, xmlNod
     } else if ((rc != pcmk_ok) && !cib_is_master && cib_legacy_mode()) {
         crm_warn("Requesting full CIB refresh because update failed: %s"
                  CRM_XS " rc=%d", pcmk_strerror(rc), rc);
-        xml_log_patchset(LOG_INFO, __FUNCTION__, input);
+        xml_log_patchset(LOG_INFO, __func__, input);
         free_xml(*result_cib);
         *result_cib = NULL;
         send_sync_request(NULL);
@@ -371,7 +375,7 @@ cib_process_replace_svr(const char *op, int options, const char *section, xmlNod
     const char *tag = crm_element_name(input);
     int rc =
         cib_process_replace(op, options, section, req, input, existing_cib, result_cib, answer);
-    if (rc == pcmk_ok && safe_str_eq(tag, XML_TAG_CIB)) {
+    if (rc == pcmk_ok && pcmk__str_eq(tag, XML_TAG_CIB, pcmk__str_casei)) {
         sync_in_progress = 0;
     }
     return rc;
@@ -403,7 +407,8 @@ delete_cib_object(xmlNode * parent, xmlNode * delete_spec)
         equiv_node = find_xml_node(parent, object_name, FALSE);
 
     } else {
-        equiv_node = find_entity(parent, object_name, object_id);
+        equiv_node = pcmk__xe_match(parent, object_name, XML_ATTR_ID,
+                                    object_id);
     }
 
     if (result != pcmk_ok) {
@@ -421,7 +426,8 @@ delete_cib_object(xmlNode * parent, xmlNode * delete_spec)
     } else {
         xmlNode *child = NULL;
 
-        for (child = __xml_first_child(delete_spec); child != NULL; child = __xml_next(child)) {
+        for (child = pcmk__xml_first_child(delete_spec); child != NULL;
+             child = pcmk__xml_next(child)) {
             int tmp_result = delete_cib_object(equiv_node, child);
 
             /*  only the first error is likely to be interesting */
@@ -444,13 +450,13 @@ cib_process_delete_absolute(const char *op, int options, const char *section, xm
     xmlNode *update_section = NULL;
 
     crm_trace("Processing \"%s\" event for section=%s", op, crm_str(section));
-    if (safe_str_eq(XML_CIB_TAG_SECTION_ALL, section)) {
+    if (pcmk__str_eq(XML_CIB_TAG_SECTION_ALL, section, pcmk__str_casei)) {
         section = NULL;
 
-    } else if (safe_str_eq(XML_TAG_CIB, section)) {
+    } else if (pcmk__str_eq(XML_TAG_CIB, section, pcmk__str_casei)) {
         section = NULL;
 
-    } else if (safe_str_eq(crm_element_name(input), XML_TAG_CIB)) {
+    } else if (pcmk__str_eq(crm_element_name(input), XML_TAG_CIB, pcmk__str_casei)) {
         section = NULL;
     }
 
