@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -18,7 +18,7 @@
 #include <crm/pengine/rules.h>
 #include <crm/cluster/internal.h>
 #include <crm/cluster/election.h>
-#include <crm/common/ipcs.h>
+#include <crm/common/ipc_internal.h>
 
 #include <pacemaker-controld.h>
 
@@ -35,6 +35,7 @@ gboolean fsa_has_quorum = FALSE;
 crm_trigger_t *fsa_source = NULL;
 crm_trigger_t *config_read = NULL;
 bool no_quorum_suicide_escalation = FALSE;
+bool controld_shutdown_lock_enabled = false;
 
 /*	 A_HA_CONNECT	*/
 void
@@ -54,7 +55,7 @@ do_ha_control(long long action,
         crm_cluster_disconnect(cluster);
         crm_info("Disconnected from the cluster");
 
-        set_bit(fsa_input_register, R_HA_DISCONNECTED);
+        controld_set_fsa_input_flags(R_HA_DISCONNECTED);
     }
 
     if (action & A_HA_CONNECT) {
@@ -78,18 +79,19 @@ do_ha_control(long long action,
         }
 
         if (registered == FALSE) {
-            set_bit(fsa_input_register, R_HA_DISCONNECTED);
+            controld_set_fsa_input_flags(R_HA_DISCONNECTED);
             register_fsa_error(C_FSA_INTERNAL, I_ERROR, NULL);
             return;
         }
 
-        populate_cib_nodes(node_update_none, __FUNCTION__);
-        clear_bit(fsa_input_register, R_HA_DISCONNECTED);
+        populate_cib_nodes(node_update_none, __func__);
+        controld_clear_fsa_input_flags(R_HA_DISCONNECTED);
         crm_info("Connected to the cluster");
     }
 
     if (action & ~(A_HA_CONNECT | A_HA_DISCONNECT)) {
-        crm_err("Unexpected action %s in %s", fsa_action2string(action), __FUNCTION__);
+        crm_err("Unexpected action %s in %s", fsa_action2string(action),
+                __func__);
     }
 }
 
@@ -100,7 +102,7 @@ do_shutdown(long long action,
             enum crmd_fsa_state cur_state, enum crmd_fsa_input current_input, fsa_data_t * msg_data)
 {
     /* just in case */
-    set_bit(fsa_input_register, R_SHUTDOWN);
+    controld_set_fsa_input_flags(R_SHUTDOWN);
     controld_disconnect_fencer(FALSE);
 }
 
@@ -113,12 +115,12 @@ do_shutdown_req(long long action,
 {
     xmlNode *msg = NULL;
 
-    set_bit(fsa_input_register, R_SHUTDOWN);
+    controld_set_fsa_input_flags(R_SHUTDOWN);
+    //controld_set_fsa_input_flags(R_STAYDOWN);
     crm_info("Sending shutdown request to all peers (DC is %s)",
              (fsa_our_dc? fsa_our_dc : "not set"));
     msg = create_request(CRM_OP_SHUTDOWN_REQ, NULL, NULL, CRM_SYSTEM_CRMD, CRM_SYSTEM_CRMD, NULL);
 
-/* 	set_bit(fsa_input_register, R_STAYDOWN); */
     if (send_cluster_message(NULL, crm_msg_crmd, msg, TRUE) == FALSE) {
         register_fsa_error(C_FSA_INTERNAL, I_ERROR, NULL);
     }
@@ -133,13 +135,13 @@ extern GHashTable *voted;
 void
 crmd_fast_exit(crm_exit_t exit_code)
 {
-    if (is_set(fsa_input_register, R_STAYDOWN)) {
+    if (pcmk_is_set(fsa_input_register, R_STAYDOWN)) {
         crm_warn("Inhibiting respawn "CRM_XS" remapping exit code %d to %d",
                  exit_code, CRM_EX_FATAL);
         exit_code = CRM_EX_FATAL;
 
     } else if ((exit_code == CRM_EX_OK)
-               && is_set(fsa_input_register, R_IN_RECOVERY)) {
+               && pcmk_is_set(fsa_input_register, R_IN_RECOVERY)) {
         crm_err("Could not recover from internal error");
         exit_code = CRM_EX_ERROR;
     }
@@ -170,7 +172,7 @@ crmd_exit(crm_exit_t exit_code)
               exit_code, crm_exit_str(exit_code));
 
     /* Suppress secondary errors resulting from us disconnecting everything */
-    set_bit(fsa_input_register, R_HA_DISCONNECTED);
+    controld_set_fsa_input_flags(R_HA_DISCONNECTED);
 
 /* Close all IPC servers and clients to ensure any and all shared memory files are cleaned up */
 
@@ -214,7 +216,7 @@ crmd_exit(crm_exit_t exit_code)
         delete_fsa_input(fsa_data);
     }
 
-    clear_bit(fsa_input_register, R_MEMBERSHIP);
+    controld_clear_fsa_input_flags(R_MEMBERSHIP);
     g_list_free(fsa_message_queue); fsa_message_queue = NULL;
 
     metadata_cache_fini();
@@ -223,11 +225,13 @@ crmd_exit(crm_exit_t exit_code)
     /* Tear down the CIB manager connection, but don't free it yet -- it could
      * be used when we drain the mainloop later.
      */
+    fsa_cib_conn->cmds->del_notify_callback(fsa_cib_conn, T_CIB_REPLACE_NOTIFY, do_cib_replaced);
+    fsa_cib_conn->cmds->del_notify_callback(fsa_cib_conn, T_CIB_DIFF_NOTIFY, do_cib_updated);
     cib_free_callbacks(fsa_cib_conn);
     fsa_cib_conn->cmds->signoff(fsa_cib_conn);
 
     verify_stopped(fsa_state, LOG_WARNING);
-    clear_bit(fsa_input_register, R_LRM_CONNECTED);
+    controld_clear_fsa_input_flags(R_LRM_CONNECTED);
     lrm_state_destroy_all();
 
     /* This basically will not work, since mainloop has a reference to it */
@@ -236,7 +240,7 @@ crmd_exit(crm_exit_t exit_code)
     mainloop_destroy_trigger(config_read); config_read = NULL;
     mainloop_destroy_trigger(transition_trigger); transition_trigger = NULL;
 
-    crm_client_cleanup();
+    pcmk__client_cleanup();
     crm_peer_destroy();
 
     controld_free_fsa_timers();
@@ -353,52 +357,45 @@ do_startup(long long action,
     }
 }
 
+// \return libqb error code (0 on success, -errno on error)
 static int32_t
-crmd_ipc_accept(qb_ipcs_connection_t * c, uid_t uid, gid_t gid)
+accept_controller_client(qb_ipcs_connection_t *c, uid_t uid, gid_t gid)
 {
-    crm_trace("Connection %p", c);
-    if (crm_client_new(c, uid, gid) == NULL) {
+    crm_trace("Accepting new IPC client connection");
+    if (pcmk__new_client(c, uid, gid) == NULL) {
         return -EIO;
     }
     return 0;
 }
 
-static void
-crmd_ipc_created(qb_ipcs_connection_t * c)
-{
-    crm_trace("Connection %p", c);
-}
-
+// \return libqb error code (0 on success, -errno on error)
 static int32_t
-crmd_ipc_dispatch(qb_ipcs_connection_t * c, void *data, size_t size)
+dispatch_controller_ipc(qb_ipcs_connection_t * c, void *data, size_t size)
 {
     uint32_t id = 0;
     uint32_t flags = 0;
-    crm_client_t *client = crm_client_get(c);
+    pcmk__client_t *client = pcmk__find_client(c);
 
-    xmlNode *msg = crm_ipcs_recv(client, data, size, &id, &flags);
-
-    crm_trace("Invoked: %s", crm_client_name(client));
-    crm_ipcs_send_ack(client, id, flags, "ack", __FUNCTION__, __LINE__);
+    xmlNode *msg = pcmk__client_data2xml(client, data, &id, &flags);
 
     if (msg == NULL) {
+        pcmk__ipc_send_ack(client, id, flags, "ack", CRM_EX_PROTOCOL);
         return 0;
     }
+    pcmk__ipc_send_ack(client, id, flags, "ack", CRM_EX_INDETERMINATE);
 
 #if ENABLE_ACL
     CRM_ASSERT(client->user != NULL);
-    crm_acl_get_set_user(msg, F_CRM_USER, client->user);
+    pcmk__update_acl_user(msg, F_CRM_USER, client->user);
 #endif
 
-    crm_trace("Processing msg from %s", crm_client_name(client));
-    crm_log_xml_trace(msg, "controller[inbound]");
-
     crm_xml_add(msg, F_CRM_SYS_FROM, client->id);
-    if (crmd_authorize_message(msg, client, NULL)) {
+    if (controld_authorize_ipc_message(msg, client, NULL)) {
+        crm_trace("Processing IPC message from %s", pcmk__client_name(client));
         route_message(C_IPC_MESSAGE, msg);
     }
 
-    trigger_fsa(fsa_source);
+    trigger_fsa();
     free_xml(msg);
     return 0;
 }
@@ -406,15 +403,15 @@ crmd_ipc_dispatch(qb_ipcs_connection_t * c, void *data, size_t size)
 static int32_t
 crmd_ipc_closed(qb_ipcs_connection_t * c)
 {
-    crm_client_t *client = crm_client_get(c);
+    pcmk__client_t *client = pcmk__find_client(c);
 
     if (client) {
         crm_trace("Disconnecting %sregistered client %s (%p/%p)",
-                  (client->userdata? "" : "un"), crm_client_name(client),
+                  (client->userdata? "" : "un"), pcmk__client_name(client),
                   c, client);
         free(client->userdata);
-        crm_client_destroy(client);
-        trigger_fsa(fsa_source);
+        pcmk__free_client(client);
+        trigger_fsa();
     }
     return 0;
 }
@@ -444,9 +441,9 @@ do_started(long long action,
            enum crmd_fsa_state cur_state, enum crmd_fsa_input current_input, fsa_data_t * msg_data)
 {
     static struct qb_ipcs_service_handlers crmd_callbacks = {
-        .connection_accept = crmd_ipc_accept,
-        .connection_created = crmd_ipc_created,
-        .msg_process = crmd_ipc_dispatch,
+        .connection_accept = accept_controller_client,
+        .connection_created = NULL,
+        .msg_process = dispatch_controller_ipc,
         .connection_closed = crmd_ipc_closed,
         .connection_destroyed = crmd_ipc_destroy
     };
@@ -455,31 +452,31 @@ do_started(long long action,
         crm_err("Start cancelled... %s", fsa_state2string(cur_state));
         return;
 
-    } else if (is_set(fsa_input_register, R_MEMBERSHIP) == FALSE) {
+    } else if (!pcmk_is_set(fsa_input_register, R_MEMBERSHIP)) {
         crm_info("Delaying start, no membership data (%.16llx)", R_MEMBERSHIP);
 
         crmd_fsa_stall(TRUE);
         return;
 
-    } else if (is_set(fsa_input_register, R_LRM_CONNECTED) == FALSE) {
+    } else if (!pcmk_is_set(fsa_input_register, R_LRM_CONNECTED)) {
         crm_info("Delaying start, not connected to executor (%.16llx)", R_LRM_CONNECTED);
 
         crmd_fsa_stall(TRUE);
         return;
 
-    } else if (is_set(fsa_input_register, R_CIB_CONNECTED) == FALSE) {
+    } else if (!pcmk_is_set(fsa_input_register, R_CIB_CONNECTED)) {
         crm_info("Delaying start, CIB not connected (%.16llx)", R_CIB_CONNECTED);
 
         crmd_fsa_stall(TRUE);
         return;
 
-    } else if (is_set(fsa_input_register, R_READ_CONFIG) == FALSE) {
+    } else if (!pcmk_is_set(fsa_input_register, R_READ_CONFIG)) {
         crm_info("Delaying start, Config not read (%.16llx)", R_READ_CONFIG);
 
         crmd_fsa_stall(TRUE);
         return;
 
-    } else if (is_set(fsa_input_register, R_PEER_DATA) == FALSE) {
+    } else if (!pcmk_is_set(fsa_input_register, R_PEER_DATA)) {
 
         crm_info("Delaying start, No peer data (%.16llx)", R_PEER_DATA);
         crmd_fsa_stall(TRUE);
@@ -487,7 +484,7 @@ do_started(long long action,
     }
 
     crm_debug("Init server comms");
-    ipcs = crmd_ipc_server_init(&crmd_callbacks);
+    ipcs = pcmk__serve_controld_ipc(&crmd_callbacks);
     if (ipcs == NULL) {
         crm_err("Failed to create IPC server: shutting down and inhibiting respawn");
         register_fsa_error(C_FSA_INTERNAL, I_ERROR, NULL);
@@ -496,7 +493,7 @@ do_started(long long action,
     }
     controld_trigger_fencer_connect();
 
-    clear_bit(fsa_input_register, R_STARTING);
+    controld_clear_fsa_input_flags(R_STARTING);
     register_fsa_input(msg_data->fsa_cause, I_PENDING, NULL);
 }
 
@@ -506,48 +503,70 @@ do_recover(long long action,
            enum crmd_fsa_cause cause,
            enum crmd_fsa_state cur_state, enum crmd_fsa_input current_input, fsa_data_t * msg_data)
 {
-    set_bit(fsa_input_register, R_IN_RECOVERY);
+    controld_set_fsa_input_flags(R_IN_RECOVERY);
     crm_warn("Fast-tracking shutdown in response to errors");
 
     register_fsa_input(C_FSA_INTERNAL, I_TERMINATE, NULL);
 }
 
-/* *INDENT-OFF* */
-static pe_cluster_option crmd_opts[] = {
-	/* name, old-name, validate, values, default, short description, long description */
-	{ "dc-version", NULL, "string", NULL, "none", NULL,
-          "Version of Pacemaker on the cluster's DC.",
-          "Includes the hash which identifies the exact changeset it was built from.  Used for diagnostic purposes."
-        },
-	{ "cluster-infrastructure", NULL, "string", NULL, "corosync", NULL,
-          "The messaging stack on which Pacemaker is currently running.",
-          "Used for informational and diagnostic purposes." },
-    { "cluster-name", NULL, "string", NULL, NULL, NULL,
+static pcmk__cluster_option_t crmd_opts[] = {
+    /* name, old name, type, allowed values,
+     * default value, validator,
+     * short description,
+     * long description
+     */
+    {
+        "dc-version", NULL, "string", NULL, "none", NULL,
+        "Pacemaker version on cluster node elected Designated Controller (DC)",
+        "Includes a hash which identifies the exact changeset the code was "
+            "built from. Used for diagnostic purposes."
+    },
+    {
+        "cluster-infrastructure", NULL, "string", NULL, "corosync", NULL,
+        "The messaging stack on which Pacemaker is currently running",
+        "Used for informational and diagnostic purposes."
+    },
+    {
+        "cluster-name", NULL, "string", NULL, NULL, NULL,
         "An arbitrary name for the cluster",
         "This optional value is mostly for users' convenience as desired "
-        "in administration, but may also be used in Pacemaker configuration "
-        "rules via the #cluster-name node attribute, and by higher-level tools "
-        "and resource agents."
+            "in administration, but may also be used in Pacemaker "
+            "configuration rules via the #cluster-name node attribute, and "
+            "by higher-level tools and resource agents."
+    },
+    {
+        XML_CONFIG_ATTR_DC_DEADTIME, NULL, "time",
+        NULL, "20s", pcmk__valid_interval_spec,
+        "How long to wait for a response from other nodes during start-up",
+        "The optimal value will depend on the speed and load of your network "
+            "and the type of switches used."
+    },
+    {
+        XML_CONFIG_ATTR_RECHECK, NULL, "time",
+        "Zero disables polling, while positive values are an interval in seconds"
+            "(unless other units are specified, for example \"5min\")",
+        "15min", pcmk__valid_interval_spec,
+        "Polling interval to recheck cluster state and evaluate rules "
+            "with date specifications",
+        "Pacemaker is primarily event-driven, and looks ahead to know when to "
+            "recheck cluster state for failure timeouts and most time-based "
+            "rules. However, it will also recheck the cluster after this "
+            "amount of inactivity, to evaluate rules with date specifications "
+            "and serve as a fail-safe for certain types of scheduler bugs."
+    },
+    {
+        "load-threshold", NULL, "percentage", NULL,
+        "80%", pcmk__valid_utilization,
+        "Maximum amount of system load that should be used by cluster nodes",
+        "The cluster will slow down its recovery process when the amount of "
+            "system resources used (currently CPU) approaches this limit",
+    },
+    {
+        "node-action-limit", NULL, "integer", NULL,
+        "0", pcmk__valid_number,
+        "Maximum number of jobs that can be scheduled per node "
+            "(defaults to 2x cores)"
     },
-	{ XML_CONFIG_ATTR_DC_DEADTIME, NULL, "time", NULL, "20s", &check_time,
-          "How long to wait for a response from other nodes during startup.",
-          "The \"correct\" value will depend on the speed/load of your network and the type of switches used."
-        },
-	{ XML_CONFIG_ATTR_RECHECK, NULL, "time",
-	  "Zero disables polling.  Positive values are an interval in seconds (unless other SI units are specified. eg. 5min)",
-          "15min", &check_timer,
-	  "Polling interval for time based changes to options, resource parameters and constraints.",
-	  "The Cluster is primarily event driven, however the configuration can have elements that change based on time."
-	  "  To ensure these changes take effect, we can optionally poll the cluster's status for changes."
-        },
-
-	{ "load-threshold", NULL, "percentage", NULL, "80%", &check_utilization,
-	  "The maximum amount of system resources that should be used by nodes in the cluster",
-	  "The cluster will slow down its recovery process when the amount of system resources used"
-          " (currently CPU) approaches this limit",
-        },
-	{ "node-action-limit", NULL, "integer", NULL, "0", &check_number,
-          "The maximum number of jobs that can be scheduled per node. Defaults to 2x cores"},
     { XML_CONFIG_ATTR_FENCE_REACTION, NULL, "string", NULL, "stop", NULL,
         "How a cluster node should react if notified of its own fencing",
         "A cluster node may receive notification of its own fencing if fencing "
@@ -556,61 +575,106 @@ static pe_cluster_option crmd_opts[] = {
         "immediately stop pacemaker and stay stopped, or \"panic\" to attempt "
         "to immediately reboot the local node, falling back to stop on failure."
     },
-	{ XML_CONFIG_ATTR_ELECTION_FAIL, NULL, "time", NULL, "2min", &check_timer,
-          "*** Advanced Use Only ***.", "If need to adjust this value, it probably indicates the presence of a bug."
-        },
-	{ XML_CONFIG_ATTR_FORCE_QUIT, NULL, "time", NULL, "20min", &check_timer,
-          "*** Advanced Use Only ***.", "If need to adjust this value, it probably indicates the presence of a bug."
-        },
-	{
-        "join-integration-timeout", "crmd-integration-timeout",
-        "time", NULL, "3min", &check_timer,
+    {
+        XML_CONFIG_ATTR_ELECTION_FAIL, NULL, "time", NULL,
+        "2min", pcmk__valid_interval_spec,
         "*** Advanced Use Only ***",
-        "If need to adjust this value, it probably indicates the presence of a bug"
+        "Declare an election failed if it is not decided within this much "
+            "time. If you need to adjust this value, it probably indicates "
+            "the presence of a bug."
     },
-	{
-        "join-finalization-timeout", "crmd-finalization-timeout",
-        "time", NULL, "30min", &check_timer,
+    {
+        XML_CONFIG_ATTR_FORCE_QUIT, NULL, "time", NULL,
+        "20min", pcmk__valid_interval_spec,
         "*** Advanced Use Only ***",
-        "If you need to adjust this value, it probably indicates the presence of a bug"
+        "Exit immediately if shutdown does not complete within this much "
+            "time. If you need to adjust this value, it probably indicates "
+            "the presence of a bug."
     },
-	{
-        "transition-delay", "crmd-transition-delay",
-        "time", NULL, "0s", &check_timer,
-        "*** Advanced Use Only *** Enabling this option will slow down cluster recovery under all conditions",
-        "Delay cluster recovery for the configured interval to allow for additional/related events to occur.\n"
-        "Useful if your configuration is sensitive to the order in which ping updates arrive."
+    {
+        "join-integration-timeout", "crmd-integration-timeout", "time", NULL,
+        "3min", pcmk__valid_interval_spec,
+        "*** Advanced Use Only ***",
+        "If you need to adjust this value, it probably indicates "
+            "the presence of a bug."
+    },
+    {
+        "join-finalization-timeout", "crmd-finalization-timeout", "time", NULL,
+        "30min", pcmk__valid_interval_spec,
+        "*** Advanced Use Only ***",
+        "If you need to adjust this value, it probably indicates "
+            "the presence of a bug."
+    },
+    {
+        "transition-delay", "crmd-transition-delay", "time", NULL,
+        "0s", pcmk__valid_interval_spec,
+        "*** Advanced Use Only *** Enabling this option will slow down "
+            "cluster recovery under all conditions",
+        "Delay cluster recovery for this much time to allow for additional "
+            "events to occur. Useful if your configuration is sensitive to "
+            "the order in which ping updates arrive."
+    },
+    {
+        "stonith-watchdog-timeout", NULL, "time", NULL,
+        "0", pcmk__valid_sbd_timeout,
+        "How long to wait before we can assume nodes are safely down "
+            "when watchdog-based self-fencing via SBD is in use",
+        "If nonzero, along with `have-watchdog=true` automatically set by the "
+            "cluster, when fencing is required, watchdog-based self-fencing "
+            "will be performed via SBD without requiring a fencing resource "
+            "explicitly configured. "
+            "If `stonith-watchdog-timeout` is set to a positive value, unseen "
+            "nodes are assumed to self-fence within this much time. +WARNING:+ "
+            "It must be ensured that this value is larger than the "
+            "`SBD_WATCHDOG_TIMEOUT` environment variable on all nodes. "
+            "Pacemaker verifies the settings individually on all nodes and "
+            "prevents startup or shuts down if configured wrongly on the fly. "
+            "It's strongly recommended that `SBD_WATCHDOG_TIMEOUT` is set to "
+            "the same value on all nodes. "
+            "If `stonith-watchdog-timeout` is set to a negative value, and "
+            "`SBD_WATCHDOG_TIMEOUT` is set, twice that value will be used. "
+            "+WARNING:+ In this case, it's essential (currently not verified by "
+            "pacemaker) that `SBD_WATCHDOG_TIMEOUT` is set to the same value on "
+            "all nodes."
+    },
+    {
+        "stonith-max-attempts", NULL, "integer", NULL,
+        "10", pcmk__valid_positive_number,
+        "How many times fencing can fail before it will no longer be "
+            "immediately re-attempted on a target"
+    },
+
+    // Already documented in libpe_status (other values must be kept identical)
+    {
+        "no-quorum-policy", NULL, "enum", "stop, freeze, ignore, demote, suicide",
+        "stop", pcmk__valid_quorum, NULL, NULL
+    },
+    {
+        XML_CONFIG_ATTR_SHUTDOWN_LOCK, NULL, "boolean", NULL,
+        "false", pcmk__valid_boolean, NULL, NULL
     },
-	{ "stonith-watchdog-timeout", NULL, "time", NULL, NULL, &check_sbd_timeout,
-	  "How long to wait before we can assume nodes are safely down", NULL
-        },
-        { "stonith-max-attempts",NULL,"integer",NULL,"10",&check_positive_number,
-          "How many times stonith can fail before it will no longer be attempted on a target"
-        },   
-	{ "no-quorum-policy", NULL, "enum", "stop, freeze, ignore, suicide", "stop", &check_quorum, NULL, NULL },
 };
-/* *INDENT-ON* */
 
 void
 crmd_metadata(void)
 {
-    config_metadata("pacemaker-controld", "1.0",
-                    "controller properties",
-                    "Cluster properties used by Pacemaker's controller,"
-                    " formerly known as crmd",
-                    crmd_opts, DIMOF(crmd_opts));
+    pcmk__print_option_metadata("pacemaker-controld", "1.0",
+                                "Pacemaker controller options",
+                                "Cluster options used by Pacemaker's "
+                                    "controller (formerly called crmd)",
+                                crmd_opts, DIMOF(crmd_opts));
 }
 
 static void
 verify_crmd_options(GHashTable * options)
 {
-    verify_all_options(options, crmd_opts, DIMOF(crmd_opts));
+    pcmk__validate_cluster_options(options, crmd_opts, DIMOF(crmd_opts));
 }
 
 static const char *
 crmd_pref(GHashTable * options, const char *name)
 {
-    return get_cluster_pref(options, crmd_opts, DIMOF(crmd_opts), name);
+    return pcmk__cluster_option(options, crmd_opts, DIMOF(crmd_opts), name);
 }
 
 static void
@@ -630,7 +694,7 @@ config_query_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
 
         if (rc == -EACCES || rc == -pcmk_err_schema_validation) {
             crm_err("The cluster is mis-configured - shutting down and staying down");
-            set_bit(fsa_input_register, R_STAYDOWN);
+            controld_set_fsa_input_flags(R_STAYDOWN);
         }
         goto bail;
     }
@@ -668,7 +732,7 @@ config_query_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
     }
 
     value = crmd_pref(config_hash, "no-quorum-policy");
-    if (safe_str_eq(value, "suicide") && pcmk_locate_sbd()) {
+    if (pcmk__str_eq(value, "suicide", pcmk__str_casei) && pcmk__locate_sbd()) {
         no_quorum_suicide_escalation = TRUE;
     }
 
@@ -698,6 +762,9 @@ config_query_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
     value = crmd_pref(config_hash, "join-finalization-timeout");
     finalization_timer->period_ms = crm_parse_interval_spec(value);
 
+    value = crmd_pref(config_hash, XML_CONFIG_ATTR_SHUTDOWN_LOCK);
+    controld_shutdown_lock_enabled = crm_is_true(value);
+
     free(fsa_cluster_name);
     fsa_cluster_name = NULL;
 
@@ -709,8 +776,8 @@ config_query_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
     alerts = first_named_child(output, XML_CIB_TAG_ALERTS);
     crmd_unpack_alerts(alerts);
 
-    set_bit(fsa_input_register, R_READ_CONFIG);
-    crm_trace("Triggering FSA: %s", __FUNCTION__);
+    controld_set_fsa_input_flags(R_READ_CONFIG);
+    crm_trace("Triggering FSA: %s", __func__);
     mainloop_set_trigger(fsa_source);
 
     g_hash_table_destroy(config_hash);
@@ -745,29 +812,27 @@ do_read_config(long long action,
 void
 crm_shutdown(int nsig)
 {
-    if (crmd_mainloop != NULL && g_main_loop_is_running(crmd_mainloop)) {
-        if (is_set(fsa_input_register, R_SHUTDOWN)) {
-            crm_err("Escalating the shutdown");
-            register_fsa_input_before(C_SHUTDOWN, I_ERROR, NULL);
-
-        } else {
-            set_bit(fsa_input_register, R_SHUTDOWN);
-            register_fsa_input(C_SHUTDOWN, I_SHUTDOWN, NULL);
-
-            if (shutdown_escalation_timer->period_ms == 0) {
-                const char *value = crmd_pref(NULL, XML_CONFIG_ATTR_FORCE_QUIT);
-
-                shutdown_escalation_timer->period_ms = crm_parse_interval_spec(value);
-            }
-
-            /* can't rely on this... */
-            crm_notice("Shutting down cluster resource manager " CRM_XS
-                       " limit=%ums", shutdown_escalation_timer->period_ms);
-            controld_start_timer(shutdown_escalation_timer);
-        }
-
-    } else {
-        crm_info("exit from shutdown");
+    if ((crmd_mainloop == NULL) || !g_main_loop_is_running(crmd_mainloop)) {
         crmd_exit(CRM_EX_OK);
+        return;
     }
+
+    if (pcmk_is_set(fsa_input_register, R_SHUTDOWN)) {
+        crm_err("Escalating shutdown");
+        register_fsa_input_before(C_SHUTDOWN, I_ERROR, NULL);
+        return;
+    }
+
+    controld_set_fsa_input_flags(R_SHUTDOWN);
+    register_fsa_input(C_SHUTDOWN, I_SHUTDOWN, NULL);
+
+    if (shutdown_escalation_timer->period_ms == 0) {
+        const char *value = crmd_pref(NULL, XML_CONFIG_ATTR_FORCE_QUIT);
+
+        shutdown_escalation_timer->period_ms = crm_parse_interval_spec(value);
+    }
+
+    crm_notice("Initiating controller shutdown sequence " CRM_XS
+               " limit=%ums", shutdown_escalation_timer->period_ms);
+    controld_start_timer(shutdown_escalation_timer);
 }
