@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -18,18 +18,20 @@
 #include <crm/common/xml.h>
 #include <crm/cluster/internal.h>
 #include <crm/cib.h>
-#include <crm/common/ipcs.h>
+#include <crm/common/ipc_internal.h>
 
 #include <pacemaker-controld.h>
 
 GListPtr fsa_message_queue = NULL;
 extern void crm_shutdown(int nsig);
 
-void handle_response(xmlNode * stored_msg);
-enum crmd_fsa_input handle_request(xmlNode * stored_msg, enum crmd_fsa_cause cause);
-enum crmd_fsa_input handle_shutdown_request(xmlNode * stored_msg);
-
-#define ROUTER_RESULT(x)	crm_trace("Router result: %s", x)
+static enum crmd_fsa_input handle_message(xmlNode *msg,
+                                          enum crmd_fsa_cause cause);
+static void handle_response(xmlNode *stored_msg);
+static enum crmd_fsa_input handle_request(xmlNode *stored_msg,
+                                          enum crmd_fsa_cause cause);
+static enum crmd_fsa_input handle_shutdown_request(xmlNode *stored_msg);
+static void send_msg_via_ipc(xmlNode * msg, const char *sys);
 
 /* debug only, can wrap all it likes */
 int last_data_id = 0;
@@ -42,7 +44,7 @@ register_fsa_error_adv(enum crmd_fsa_cause cause, enum crmd_fsa_input input,
     if (fsa_actions != A_NOTHING) {
         register_fsa_input_adv(cur_data ? cur_data->fsa_cause : C_FSA_INTERNAL,
                                I_NULL, cur_data ? cur_data->data : NULL,
-                               fsa_actions, TRUE, __FUNCTION__);
+                               fsa_actions, TRUE, __func__);
     }
 
     /* reset the action list */
@@ -56,7 +58,7 @@ register_fsa_error_adv(enum crmd_fsa_cause cause, enum crmd_fsa_input input,
 
 int
 register_fsa_input_adv(enum crmd_fsa_cause cause, enum crmd_fsa_input input,
-                       void *data, long long with_actions,
+                       void *data, uint64_t with_actions,
                        gboolean prepend, const char *raised_from)
 {
     unsigned old_len = g_list_length(fsa_message_queue);
@@ -83,7 +85,7 @@ register_fsa_input_adv(enum crmd_fsa_cause cause, enum crmd_fsa_input input,
         }
 
         if (data == NULL) {
-            fsa_actions |= with_actions;
+            controld_set_fsa_action_flags(with_actions);
             fsa_dump_actions(with_actions, "Restored");
             return 0;
         }
@@ -94,9 +96,10 @@ register_fsa_input_adv(enum crmd_fsa_cause cause, enum crmd_fsa_input input,
     }
 
     last_data_id++;
-    crm_trace("%s %s FSA input %d (%s) (cause=%s) %s data",
-              raised_from, prepend ? "prepended" : "appended", last_data_id,
-              fsa_input2string(input), fsa_cause2string(cause), data ? "with" : "without");
+    crm_trace("%s %s FSA input %d (%s) due to %s, %s data",
+              raised_from, (prepend? "prepended" : "appended"), last_data_id,
+              fsa_input2string(input), fsa_cause2string(cause),
+              (data? "with" : "without"));
 
     fsa_data = calloc(1, sizeof(fsa_data_t));
     fsa_data->id = last_data_id;
@@ -108,7 +111,8 @@ register_fsa_input_adv(enum crmd_fsa_cause cause, enum crmd_fsa_input input,
     fsa_data->actions = with_actions;
 
     if (with_actions != A_NOTHING) {
-        crm_trace("Adding actions %.16llx to input", with_actions);
+        crm_trace("Adding actions %.16llx to input",
+                  (unsigned long long) with_actions);
     }
 
     if (data != NULL) {
@@ -117,10 +121,10 @@ register_fsa_input_adv(enum crmd_fsa_cause cause, enum crmd_fsa_input input,
             case C_CRMD_STATUS_CALLBACK:
             case C_IPC_MESSAGE:
             case C_HA_MESSAGE:
-                crm_trace("Copying %s data from %s as a HA msg",
-                          fsa_cause2string(cause), raised_from);
                 CRM_CHECK(((ha_msg_input_t *) data)->msg != NULL,
                           crm_err("Bogus data from %s", raised_from));
+                crm_trace("Copying %s data from %s as cluster message data",
+                          fsa_cause2string(cause), raised_from);
                 fsa_data->data = copy_ha_msg_input(data);
                 fsa_data->data_type = fsa_dt_ha_msg;
                 break;
@@ -136,23 +140,22 @@ register_fsa_input_adv(enum crmd_fsa_cause cause, enum crmd_fsa_input input,
             case C_SHUTDOWN:
             case C_UNKNOWN:
             case C_STARTUP:
-                crm_err("Copying %s data (from %s)"
-                        " not yet implemented", fsa_cause2string(cause), raised_from);
+                crm_crit("Copying %s data (from %s) is not yet implemented",
+                         fsa_cause2string(cause), raised_from);
                 crmd_exit(CRM_EX_SOFTWARE);
                 break;
         }
-        crm_trace("%s data copied", fsa_cause2string(fsa_data->fsa_cause));
     }
 
     /* make sure to free it properly later */
     if (prepend) {
-        crm_trace("Prepending input");
         fsa_message_queue = g_list_prepend(fsa_message_queue, fsa_data);
     } else {
         fsa_message_queue = g_list_append(fsa_message_queue, fsa_data);
     }
 
-    crm_trace("Queue len: %d", g_list_length(fsa_message_queue));
+    crm_trace("FSA message queue length is %d",
+              g_list_length(fsa_message_queue));
 
     /* fsa_dump_queue(LOG_TRACE); */
 
@@ -161,7 +164,7 @@ register_fsa_input_adv(enum crmd_fsa_cause cause, enum crmd_fsa_input input,
     }
 
     if (fsa_source && input != I_WAIT_FOR_EVENT) {
-        crm_trace("Triggering FSA: %s", __FUNCTION__);
+        crm_trace("Triggering FSA");
         mainloop_set_trigger(fsa_source);
     }
     return last_data_id;
@@ -187,20 +190,11 @@ fsa_dump_queue(int log_level)
 ha_msg_input_t *
 copy_ha_msg_input(ha_msg_input_t * orig)
 {
-    ha_msg_input_t *copy = NULL;
-    xmlNodePtr data = NULL;
+    ha_msg_input_t *copy = calloc(1, sizeof(ha_msg_input_t));
 
-    if (orig != NULL) {
-        crm_trace("Copy msg");
-        data = copy_xml(orig->msg);
-
-    } else {
-        crm_trace("No message to copy");
-    }
-    copy = new_ha_msg_input(data);
-    if (orig && orig->msg != NULL) {
-        CRM_CHECK(copy->msg != NULL, crm_err("copy failed"));
-    }
+    CRM_ASSERT(copy != NULL);
+    copy->msg = (orig && orig->msg)? copy_xml(orig->msg) : NULL;
+    copy->xml = get_message_xml(copy->msg, F_CRM_DATA);
     return copy;
 }
 
@@ -256,13 +250,6 @@ get_message(void)
     return message;
 }
 
-/* returns the current head of the FIFO queue */
-gboolean
-is_message(void)
-{
-    return (g_list_length(fsa_message_queue) > 0);
-}
-
 void *
 fsa_typed_data_adv(fsa_data_t * fsa_data, enum fsa_data_type a_type, const char *caller)
 {
@@ -345,42 +332,38 @@ relay_message(xmlNode * msg, gboolean originated_locally)
     int is_for_crm = 0;
     int is_for_cib = 0;
     int is_local = 0;
-    gboolean processing_complete = FALSE;
     const char *host_to = crm_element_value(msg, F_CRM_HOST_TO);
     const char *sys_to = crm_element_value(msg, F_CRM_SYS_TO);
     const char *sys_from = crm_element_value(msg, F_CRM_SYS_FROM);
     const char *type = crm_element_value(msg, F_TYPE);
     const char *task = crm_element_value(msg, F_CRM_TASK);
-    const char *msg_error = NULL;
+    const char *ref = crm_element_value(msg, XML_ATTR_REFERENCE);
 
-    crm_trace("Routing message %s", crm_element_value(msg, XML_ATTR_REFERENCE));
+    if (ref == NULL) {
+        ref = "without reference ID";
+    }
 
     if (msg == NULL) {
-        msg_error = "Cannot route empty message";
+        crm_warn("Cannot route empty message");
+        return TRUE;
 
-    } else if (safe_str_eq(task, CRM_OP_HELLO)) {
+    } else if (pcmk__str_eq(task, CRM_OP_HELLO, pcmk__str_casei)) {
         /* quietly ignore */
-        processing_complete = TRUE;
+        crm_trace("No routing needed for hello message %s", ref);
+        return TRUE;
 
-    } else if (safe_str_neq(type, T_CRM)) {
-        msg_error = "Bad message type";
+    } else if (!pcmk__str_eq(type, T_CRM, pcmk__str_casei)) {
+        crm_warn("Cannot route message %s: Type is '%s' not '" T_CRM "'",
+                 ref, (type? type : "missing"));
+        crm_log_xml_warn(msg, "[bad message type]");
+        return TRUE;
 
     } else if (sys_to == NULL) {
-        msg_error = "Bad message destination: no subsystem";
-    }
-
-    if (msg_error != NULL) {
-        processing_complete = TRUE;
-        crm_err("%s", msg_error);
-        crm_log_xml_warn(msg, "bad msg");
-    }
-
-    if (processing_complete) {
+        crm_warn("Cannot route message %s: No subsystem specified", ref);
+        crm_log_xml_warn(msg, "[no subsystem]");
         return TRUE;
     }
 
-    processing_complete = TRUE;
-
     is_for_dc = (strcasecmp(CRM_SYSTEM_DC, sys_to) == 0);
     is_for_dcib = (strcasecmp(CRM_SYSTEM_DCIB, sys_to) == 0);
     is_for_te = (strcasecmp(CRM_SYSTEM_TENGINE, sys_to) == 0);
@@ -388,15 +371,16 @@ relay_message(xmlNode * msg, gboolean originated_locally)
     is_for_crm = (strcasecmp(CRM_SYSTEM_CRMD, sys_to) == 0);
 
     is_local = 0;
-    if (host_to == NULL || strlen(host_to) == 0) {
+    if (pcmk__str_empty(host_to)) {
         if (is_for_dc || is_for_te) {
             is_local = 0;
 
         } else if (is_for_crm) {
-            if (safe_str_eq(task, CRM_OP_NODE_INFO)) {
+            if (pcmk__strcase_any_of(task, CRM_OP_NODE_INFO,
+                                     PCMK__CONTROLD_CMD_NODES, NULL)) {
                 /* Node info requests do not specify a host, which is normally
                  * treated as "all hosts", because the whole point is that the
-                 * client doesn't know the local node name. Always handle these
+                 * client may not know the local node name. Always handle these
                  * requests locally.
                  */
                 is_local = 1;
@@ -408,45 +392,54 @@ relay_message(xmlNode * msg, gboolean originated_locally)
             is_local = 1;
         }
 
-    } else if (safe_str_eq(fsa_our_uname, host_to)) {
+    } else if (pcmk__str_eq(fsa_our_uname, host_to, pcmk__str_casei)) {
         is_local = 1;
+    } else if (is_for_crm && pcmk__str_eq(task, CRM_OP_LRM_DELETE, pcmk__str_casei)) {
+        xmlNode *msg_data = get_message_xml(msg, F_CRM_DATA);
+        const char *mode = crm_element_value(msg_data, PCMK__XA_MODE);
+
+        if (pcmk__str_eq(mode, XML_TAG_CIB, pcmk__str_casei)) {
+            // Local delete of an offline node's resource history
+            is_local = 1;
+        }
     }
 
     if (is_for_dc || is_for_dcib || is_for_te) {
         if (AM_I_DC && is_for_te) {
-            ROUTER_RESULT("Message result: Local relay");
+            crm_trace("Route message %s locally as transition request", ref);
             send_msg_via_ipc(msg, sys_to);
 
         } else if (AM_I_DC) {
-            ROUTER_RESULT("Message result: DC/controller process");
-            processing_complete = FALSE;        /* more to be done by caller */
-        } else if (originated_locally && safe_str_neq(sys_from, CRM_SYSTEM_PENGINE)
-                   && safe_str_neq(sys_from, CRM_SYSTEM_TENGINE)) {
+            crm_trace("Route message %s locally as DC request", ref);
+            return FALSE; // More to be done by caller
 
-            /* Neither the TE nor the scheduler should be sending messages
-             * to DCs on other nodes. By definition, if we are no longer the DC,
-             * then the scheduler's or TE's data should be discarded.
-             */
+        } else if (originated_locally && !pcmk__strcase_any_of(sys_from, CRM_SYSTEM_PENGINE,
+                                                               CRM_SYSTEM_TENGINE, NULL)) {
 
 #if SUPPORT_COROSYNC
             if (is_corosync_cluster()) {
                 dest = text2msg_type(sys_to);
             }
 #endif
-            ROUTER_RESULT("Message result: External relay to DC");
+            crm_trace("Relay message %s to DC", ref);
             send_cluster_message(host_to ? crm_get_peer(0, host_to) : NULL, dest, msg, TRUE);
 
         } else {
-            /* discard */
-            ROUTER_RESULT("Message result: Discard, not DC");
+            /* Neither the TE nor the scheduler should be sending messages
+             * to DCs on other nodes. By definition, if we are no longer the DC,
+             * then the scheduler's or TE's data should be discarded.
+             */
+            crm_trace("Discard message %s because we are not DC", ref);
         }
 
     } else if (is_local && (is_for_crm || is_for_cib)) {
-        ROUTER_RESULT("Message result: controller process");
-        processing_complete = FALSE;    /* more to be done by caller */
+        crm_trace("Route message %s locally as controller request", ref);
+        return FALSE; // More to be done by caller
 
     } else if (is_local) {
-        ROUTER_RESULT("Message result: Local relay");
+        crm_trace("Relay message %s locally to %s",
+                  ref, (sys_to? sys_to : "unknown client"));
+        crm_log_xml_trace(msg, "[IPC relay]");
         send_msg_via_ipc(msg, sys_to);
 
     } else {
@@ -465,138 +458,125 @@ relay_message(xmlNode * msg, gboolean originated_locally)
         if (host_to) {
             node_to = crm_find_peer(0, host_to);
             if (node_to == NULL) {
-               crm_err("Cannot route message to unknown node %s", host_to);
-               return TRUE;
+                crm_warn("Cannot route message %s: Unknown node %s",
+                         ref, host_to);
+                return TRUE;
             }
+            crm_trace("Relay message %s to %s",
+                      ref, (node_to->uname? node_to->uname : "peer"));
+        } else {
+            crm_trace("Broadcast message %s to all peers", ref);
         }
-
-        ROUTER_RESULT("Message result: External relay");
         send_cluster_message(host_to ? node_to : NULL, dest, msg, TRUE);
     }
 
-    return processing_complete;
+    return TRUE; // No further processing of message is needed
 }
 
-static gboolean
-process_hello_message(xmlNode * hello,
-                      char **client_name, char **major_version, char **minor_version)
+// Return true if field contains a positive integer
+static bool
+authorize_version(xmlNode *message_data, const char *field,
+                  const char *client_name, const char *ref, const char *uuid)
 {
-    const char *local_client_name;
-    const char *local_major_version;
-    const char *local_minor_version;
+    const char *version = crm_element_value(message_data, field);
 
-    *client_name = NULL;
-    *major_version = NULL;
-    *minor_version = NULL;
+    if (pcmk__str_empty(version)) {
+        crm_warn("IPC hello from %s rejected: No protocol %s",
+                 CRM_XS " ref=%s uuid=%s",
+                 client_name, field, (ref? ref : "none"), uuid);
+        return false;
+    } else {
+        int version_num = crm_parse_int(version, NULL);
 
-    if (hello == NULL) {
-        return FALSE;
+        if (version_num < 0) {
+            crm_warn("IPC hello from %s rejected: Protocol %s '%s' "
+                     "not recognized", CRM_XS " ref=%s uuid=%s",
+                     client_name, field, version, (ref? ref : "none"), uuid);
+            return false;
+        }
     }
-
-    local_client_name = crm_element_value(hello, "client_name");
-    local_major_version = crm_element_value(hello, "major_version");
-    local_minor_version = crm_element_value(hello, "minor_version");
-
-    if (local_client_name == NULL || strlen(local_client_name) == 0) {
-        crm_err("Hello message was not valid (field %s not found)", "client name");
-        return FALSE;
-
-    } else if (local_major_version == NULL || strlen(local_major_version) == 0) {
-        crm_err("Hello message was not valid (field %s not found)", "major version");
-        return FALSE;
-
-    } else if (local_minor_version == NULL || strlen(local_minor_version) == 0) {
-        crm_err("Hello message was not valid (field %s not found)", "minor version");
-        return FALSE;
-    }
-
-    *client_name = strdup(local_client_name);
-    *major_version = strdup(local_major_version);
-    *minor_version = strdup(local_minor_version);
-
-    crm_trace("Hello message ok");
-    return TRUE;
+    return true;
 }
 
-gboolean
-crmd_authorize_message(xmlNode * client_msg, crm_client_t * curr_client, const char *proxy_session)
+/*!
+ * \internal
+ * \brief Check whether a client IPC message is acceptable
+ *
+ * If a given client IPC message is a hello, "authorize" it by ensuring it has
+ * valid information such as a protocol version, and return false indicating
+ * that nothing further needs to be done with the message. If the message is not
+ * a hello, just return true to indicate it needs further processing.
+ *
+ * \param[in] client_msg     XML of IPC message
+ * \param[in] curr_client    If IPC is not proxied, client that sent message
+ * \param[in] proxy_session  If IPC is proxied, the session ID
+ *
+ * \return true if message needs further processing, false if it doesn't
+ */
+bool
+controld_authorize_ipc_message(xmlNode *client_msg, pcmk__client_t *curr_client,
+                               const char *proxy_session)
 {
-    char *client_name = NULL;
-    char *major_version = NULL;
-    char *minor_version = NULL;
-    gboolean auth_result = FALSE;
-
-    xmlNode *xml = NULL;
+    xmlNode *message_data = NULL;
+    const char *client_name = NULL;
     const char *op = crm_element_value(client_msg, F_CRM_TASK);
-    const char *uuid = curr_client ? curr_client->id : proxy_session;
+    const char *ref = crm_element_value(client_msg, XML_ATTR_REFERENCE);
+    const char *uuid = (curr_client? curr_client->id : proxy_session);
 
     if (uuid == NULL) {
-        crm_warn("Message [%s] not authorized", crm_element_value(client_msg, XML_ATTR_REFERENCE));
-        return FALSE;
-
-    } else if (safe_str_neq(CRM_OP_HELLO, op)) {
-        return TRUE;
+        crm_warn("IPC message from client rejected: No client identifier "
+                 CRM_XS " ref=%s", (ref? ref : "none"));
+        goto rejected;
     }
 
-    xml = get_message_xml(client_msg, F_CRM_DATA);
-    auth_result = process_hello_message(xml, &client_name, &major_version, &minor_version);
-
-    if (auth_result == TRUE) {
-        if (client_name == NULL) {
-            crm_err("Bad client details (client_name=%s, uuid=%s)",
-                    crm_str(client_name), uuid);
-            auth_result = FALSE;
-        }
+    if (!pcmk__str_eq(CRM_OP_HELLO, op, pcmk__str_casei)) {
+        // Only hello messages need to be authorized
+        return true;
     }
 
-    if (auth_result == TRUE) {
-        /* check version */
-        int mav = atoi(major_version);
-        int miv = atoi(minor_version);
+    message_data = get_message_xml(client_msg, F_CRM_DATA);
 
-        crm_trace("Checking client version number");
-        if (mav < 0 || miv < 0) {
-            crm_err("Client version (%d:%d) is not acceptable", mav, miv);
-            auth_result = FALSE;
-        }
+    client_name = crm_element_value(message_data, "client_name");
+    if (pcmk__str_empty(client_name)) {
+        crm_warn("IPC hello from client rejected: No client name",
+                 CRM_XS " ref=%s uuid=%s", (ref? ref : "none"), uuid);
+        goto rejected;
+    }
+    if (!authorize_version(message_data, "major_version", client_name, ref,
+                           uuid)) {
+        goto rejected;
+    }
+    if (!authorize_version(message_data, "minor_version", client_name, ref,
+                           uuid)) {
+        goto rejected;
     }
 
-    if (auth_result == TRUE) {
-        crm_trace("Accepted client %s", client_name);
-        if (curr_client) {
-            curr_client->userdata = strdup(client_name);
-        }
-
-        crm_trace("Triggering FSA: %s", __FUNCTION__);
-        mainloop_set_trigger(fsa_source);
-
-    } else {
-        crm_warn("Rejected client logon request");
-        if (curr_client) {
-            qb_ipcs_disconnect(curr_client->ipcs);
-        }
+    crm_trace("Validated IPC hello from client %s", client_name);
+    if (curr_client) {
+        curr_client->userdata = strdup(client_name);
     }
+    mainloop_set_trigger(fsa_source);
+    return false;
 
-    free(minor_version);
-    free(major_version);
-    free(client_name);
-
-    /* hello messages should never be processed further */
-    return FALSE;
+rejected:
+    if (curr_client) {
+        qb_ipcs_disconnect(curr_client->ipcs);
+    }
+    return false;
 }
 
-enum crmd_fsa_input
-handle_message(xmlNode * msg, enum crmd_fsa_cause cause)
+static enum crmd_fsa_input
+handle_message(xmlNode *msg, enum crmd_fsa_cause cause)
 {
     const char *type = NULL;
 
     CRM_CHECK(msg != NULL, return I_NULL);
 
     type = crm_element_value(msg, F_CRM_MSG_TYPE);
-    if (crm_str_eq(type, XML_ATTR_REQUEST, TRUE)) {
+    if (pcmk__str_eq(type, XML_ATTR_REQUEST, pcmk__str_none)) {
         return handle_request(msg, cause);
 
-    } else if (crm_str_eq(type, XML_ATTR_RESPONSE, TRUE)) {
+    } else if (pcmk__str_eq(type, XML_ATTR_RESPONSE, pcmk__str_none)) {
         handle_response(msg);
         return I_NULL;
     }
@@ -611,7 +591,6 @@ handle_failcount_op(xmlNode * stored_msg)
     const char *rsc = NULL;
     const char *uname = NULL;
     const char *op = NULL;
-    const char *interval_ms_s = NULL;
     char *interval_spec = NULL;
     guint interval_ms = 0;
     gboolean is_remote_node = FALSE;
@@ -627,9 +606,9 @@ handle_failcount_op(xmlNode * stored_msg)
         if (xml_attrs) {
             op = crm_element_value(xml_attrs,
                                    CRM_META "_" XML_RSC_ATTR_CLEAR_OP);
-            interval_ms_s = crm_element_value(xml_attrs,
-                                              CRM_META "_" XML_RSC_ATTR_CLEAR_INTERVAL);
-            interval_ms = crm_parse_ms(interval_ms_s);
+            crm_element_value_ms(xml_attrs,
+                                 CRM_META "_" XML_RSC_ATTR_CLEAR_INTERVAL,
+                                 &interval_ms);
         }
     }
     uname = crm_element_value(xml_op, XML_LRM_ATTR_TARGET);
@@ -654,6 +633,93 @@ handle_failcount_op(xmlNode * stored_msg)
     return I_NULL;
 }
 
+static enum crmd_fsa_input
+handle_lrm_delete(xmlNode *stored_msg)
+{
+    const char *mode = NULL;
+    xmlNode *msg_data = get_message_xml(stored_msg, F_CRM_DATA);
+
+    CRM_CHECK(msg_data != NULL, return I_NULL);
+
+    /* CRM_OP_LRM_DELETE has two distinct modes. The default behavior is to
+     * relay the operation to the affected node, which will unregister the
+     * resource from the local executor, clear the resource's history from the
+     * CIB, and do some bookkeeping in the controller.
+     *
+     * However, if the affected node is offline, the client will specify
+     * mode="cib" which means the controller receiving the operation should
+     * clear the resource's history from the CIB and nothing else. This is used
+     * to clear shutdown locks.
+     */
+    mode = crm_element_value(msg_data, PCMK__XA_MODE);
+    if ((mode == NULL) || strcmp(mode, XML_TAG_CIB)) {
+        // Relay to affected node
+        crm_xml_add(stored_msg, F_CRM_SYS_TO, CRM_SYSTEM_LRMD);
+        return I_ROUTER;
+
+    } else {
+        // Delete CIB history locally (compare with do_lrm_delete())
+        const char *from_sys = NULL;
+        const char *user_name = NULL;
+        const char *rsc_id = NULL;
+        const char *node = NULL;
+        xmlNode *rsc_xml = NULL;
+        int rc = pcmk_rc_ok;
+
+        rsc_xml = first_named_child(msg_data, XML_CIB_TAG_RESOURCE);
+        CRM_CHECK(rsc_xml != NULL, return I_NULL);
+
+        rsc_id = ID(rsc_xml);
+        from_sys = crm_element_value(stored_msg, F_CRM_SYS_FROM);
+        node = crm_element_value(msg_data, XML_LRM_ATTR_TARGET);
+#if ENABLE_ACL
+        user_name = pcmk__update_acl_user(stored_msg, F_CRM_USER, NULL);
+#endif
+        crm_debug("Handling " CRM_OP_LRM_DELETE " for %s on %s locally%s%s "
+                  "(clearing CIB resource history only)", rsc_id, node,
+                  (user_name? " for user " : ""), (user_name? user_name : ""));
+#if ENABLE_ACL
+        rc = controld_delete_resource_history(rsc_id, node, user_name,
+                                              cib_dryrun|cib_sync_call);
+#endif
+        if (rc == pcmk_rc_ok) {
+            rc = controld_delete_resource_history(rsc_id, node, user_name,
+                                                  crmd_cib_smart_opt());
+        }
+
+        //Notify client and tengine.(Only notify tengine if mode = "cib" and CRM_OP_LRM_DELETE.)
+        if (from_sys) {
+            lrmd_event_data_t *op = NULL;
+            const char *from_host = crm_element_value(stored_msg,
+                                                      F_CRM_HOST_FROM);
+            const char *transition;
+
+            if (strcmp(from_sys, CRM_SYSTEM_TENGINE)) {
+                transition = crm_element_value(msg_data,
+                                                       XML_ATTR_TRANSITION_KEY);
+            } else {
+                transition = crm_element_value(stored_msg,
+                                                       XML_ATTR_TRANSITION_KEY);
+            }
+
+            crm_info("Notifying %s on %s that %s was%s deleted",
+                     from_sys, (from_host? from_host : "local node"), rsc_id,
+                     ((rc == pcmk_rc_ok)? "" : " not"));
+            op = lrmd_new_event(rsc_id, CRMD_ACTION_DELETE, 0);
+            op->type = lrmd_event_exec_complete;
+            op->user_data = strdup(transition? transition : FAKE_TE_ID);
+            op->params = crm_str_table_new();
+            g_hash_table_insert(op->params, strdup(XML_ATTR_CRM_VERSION),
+                                strdup(CRM_FEATURE_SET));
+            controld_rc2event(op, rc);
+            controld_ack_event_directly(from_host, from_sys, NULL, op, rsc_id);
+            lrmd_free_event(op);
+            controld_trigger_delete_refresh(from_sys, rsc_id);
+        }
+        return I_NULL;
+    }
+}
+
 /*!
  * \brief Handle a CRM_OP_REMOTE_STATE message by updating remote peer cache
  *
@@ -673,7 +739,7 @@ handle_remote_state(xmlNode *msg)
     remote_peer = crm_remote_peer_get(remote_uname);
     CRM_CHECK(remote_peer, return I_NULL);
 
-    crm_update_peer_state(__FUNCTION__, remote_peer,
+    crm_update_peer_state(__func__, remote_peer,
                           crm_is_true(remote_is_up)?
                           CRM_NODE_MEMBER : CRM_NODE_LOST, 0);
     return I_NULL;
@@ -719,6 +785,42 @@ handle_ping(xmlNode *msg)
     return I_NULL;
 }
 
+/*!
+ * \brief Handle a PCMK__CONTROLD_CMD_NODES message
+ *
+ * \return Next FSA input
+ */
+static enum crmd_fsa_input
+handle_node_list(xmlNode *request)
+{
+    GHashTableIter iter;
+    crm_node_t *node = NULL;
+    xmlNode *reply = NULL;
+    xmlNode *reply_data = NULL;
+
+    // Create message data for reply
+    reply_data = create_xml_node(NULL, XML_CIB_TAG_NODES);
+    g_hash_table_iter_init(&iter, crm_peer_cache);
+    while (g_hash_table_iter_next(&iter, NULL, (gpointer *) & node)) {
+        xmlNode *xml = create_xml_node(reply_data, XML_CIB_TAG_NODE);
+
+        crm_xml_add_ll(xml, XML_ATTR_ID, (long long) node->id); // uint32_t
+        crm_xml_add(xml, XML_ATTR_UNAME, node->uname);
+        crm_xml_add(xml, XML_NODE_IN_CLUSTER, node->state);
+    }
+
+    // Create and send reply
+    reply = create_reply(request, reply_data);
+    free_xml(reply_data);
+    if (reply) {
+        (void) relay_message(reply, TRUE);
+        free_xml(reply);
+    }
+
+    // Nothing further to do
+    return I_NULL;
+}
+
 /*!
  * \brief Handle a CRM_OP_NODE_INFO request
  *
@@ -796,13 +898,75 @@ verify_feature_set(xmlNode *msg)
                 CRM_FEATURE_SET, dc_version);
 
         // Nothing is likely to improve without administrator involvement
-        set_bit(fsa_input_register, R_STAYDOWN);
+        controld_set_fsa_input_flags(R_STAYDOWN);
         crmd_exit(CRM_EX_FATAL);
     }
 }
 
-enum crmd_fsa_input
-handle_request(xmlNode * stored_msg, enum crmd_fsa_cause cause)
+// DC gets own shutdown all-clear
+static enum crmd_fsa_input
+handle_shutdown_self_ack(xmlNode *stored_msg)
+{
+    const char *host_from = crm_element_value(stored_msg, F_CRM_HOST_FROM);
+
+    if (pcmk_is_set(fsa_input_register, R_SHUTDOWN)) {
+        // The expected case -- we initiated own shutdown sequence
+        crm_info("Shutting down controller");
+        return I_STOP;
+    }
+
+    if (pcmk__str_eq(host_from, fsa_our_dc, pcmk__str_casei)) {
+        // Must be logic error -- DC confirming its own unrequested shutdown
+        crm_err("Shutting down controller immediately due to "
+                "unexpected shutdown confirmation");
+        return I_TERMINATE;
+    }
+
+    if (fsa_state != S_STOPPING) {
+        // Shouldn't happen -- non-DC confirming unrequested shutdown
+        crm_err("Starting new DC election because %s is "
+                "confirming shutdown we did not request",
+                (host_from? host_from : "another node"));
+        return I_ELECTION;
+    }
+
+    // Shouldn't happen, but we are already stopping anyway
+    crm_debug("Ignoring unexpected shutdown confirmation from %s",
+              (host_from? host_from : "another node"));
+    return I_NULL;
+}
+
+// Non-DC gets shutdown all-clear from DC
+static enum crmd_fsa_input
+handle_shutdown_ack(xmlNode *stored_msg)
+{
+    const char *host_from = crm_element_value(stored_msg, F_CRM_HOST_FROM);
+
+    if (host_from == NULL) {
+        crm_warn("Ignoring shutdown request without origin specified");
+        return I_NULL;
+    }
+
+    if ((fsa_our_dc == NULL) || (strcmp(host_from, fsa_our_dc) == 0)) {
+
+        if (pcmk_is_set(fsa_input_register, R_SHUTDOWN)) {
+            crm_info("Shutting down controller after confirmation from %s",
+                     host_from);
+        } else {
+            crm_err("Shutting down controller after unexpected "
+                    "shutdown request from %s", host_from);
+            controld_set_fsa_input_flags(R_STAYDOWN);
+        }
+        return I_STOP;
+    }
+
+    crm_warn("Ignoring shutdown request from %s because DC is %s",
+             host_from, fsa_our_dc);
+    return I_NULL;
+}
+
+static enum crmd_fsa_input
+handle_request(xmlNode *stored_msg, enum crmd_fsa_cause cause)
 {
     xmlNode *msg = NULL;
     const char *op = crm_element_value(stored_msg, F_CRM_TASK);
@@ -810,7 +974,7 @@ handle_request(xmlNode * stored_msg, enum crmd_fsa_cause cause)
     /* Optimize this for the DC - it has the most to do */
 
     if (op == NULL) {
-        crm_log_xml_err(stored_msg, "Bad message");
+        crm_log_xml_warn(stored_msg, "[request without " F_CRM_TASK "]");
         return I_NULL;
     }
 
@@ -818,7 +982,7 @@ handle_request(xmlNode * stored_msg, enum crmd_fsa_cause cause)
         const char *from = crm_element_value(stored_msg, F_CRM_HOST_FROM);
         crm_node_t *node = crm_find_peer(0, from);
 
-        crm_update_peer_expected(__FUNCTION__, node, CRMD_JOINSTATE_DOWN);
+        crm_update_peer_expected(__func__, node, CRMD_JOINSTATE_DOWN);
         if(AM_I_DC == FALSE) {
             return I_NULL; /* Done */
         }
@@ -836,22 +1000,7 @@ handle_request(xmlNode * stored_msg, enum crmd_fsa_cause cause)
             return I_JOIN_RESULT;
 
         } else if (strcmp(op, CRM_OP_SHUTDOWN) == 0) {
-            const char *host_from = crm_element_value(stored_msg, F_CRM_HOST_FROM);
-            gboolean dc_match = safe_str_eq(host_from, fsa_our_dc);
-
-            if (is_set(fsa_input_register, R_SHUTDOWN)) {
-                crm_info("Shutting ourselves down (DC)");
-                return I_STOP;
-
-            } else if (dc_match) {
-                crm_err("We didn't ask to be shut down, yet our"
-                        " TE is telling us to. Better get out now!");
-                return I_TERMINATE;
-
-            } else if (fsa_state != S_STOPPING) {
-                crm_err("Another node is asking us to shutdown" " but we think we're ok.");
-                return I_ELECTION;
-            }
+            return handle_shutdown_self_ack(stored_msg);
 
         } else if (strcmp(op, CRM_OP_SHUTDOWN_REQ) == 0) {
             /* a slave wants to shut down */
@@ -870,7 +1019,8 @@ handle_request(xmlNode * stored_msg, enum crmd_fsa_cause cause)
 
         fsa_input.msg = stored_msg;
         register_fsa_input_adv(C_HA_MESSAGE, I_NULL, &fsa_input,
-                               A_ELECTION_COUNT | A_ELECTION_CHECK, FALSE, __FUNCTION__);
+                               A_ELECTION_COUNT | A_ELECTION_CHECK, FALSE,
+                               __func__);
 
     } else if (strcmp(op, CRM_OP_THROTTLE) == 0) {
         throttle_update(stored_msg);
@@ -891,7 +1041,8 @@ handle_request(xmlNode * stored_msg, enum crmd_fsa_cause cause)
 
         fsa_input.msg = stored_msg;
         register_fsa_input_adv(C_HA_MESSAGE, I_NULL, &fsa_input,
-                               A_ELECTION_COUNT | A_ELECTION_CHECK, FALSE, __FUNCTION__);
+                               A_ELECTION_COUNT | A_ELECTION_CHECK, FALSE,
+                               __func__);
 
         /* Sometimes we _must_ go into S_ELECTION */
         if (fsa_state == S_HALT) {
@@ -913,9 +1064,12 @@ handle_request(xmlNode * stored_msg, enum crmd_fsa_cause cause)
         crm_debug("Raising I_JOIN_RESULT: join-%s", crm_element_value(stored_msg, F_CRM_JOIN_ID));
         return I_JOIN_RESULT;
 
-    } else if (strcmp(op, CRM_OP_LRM_DELETE) == 0
-               || strcmp(op, CRM_OP_LRM_FAIL) == 0
-               || strcmp(op, CRM_OP_LRM_REFRESH) == 0 || strcmp(op, CRM_OP_REPROBE) == 0) {
+    } else if (strcmp(op, CRM_OP_LRM_DELETE) == 0) {
+        return handle_lrm_delete(stored_msg);
+
+    } else if ((strcmp(op, CRM_OP_LRM_FAIL) == 0)
+               || (strcmp(op, CRM_OP_LRM_REFRESH) == 0)
+               || (strcmp(op, CRM_OP_REPROBE) == 0)) {
 
         crm_xml_add(stored_msg, F_CRM_SYS_TO, CRM_SYSTEM_LRMD);
         return I_ROUTER;
@@ -966,23 +1120,14 @@ handle_request(xmlNode * stored_msg, enum crmd_fsa_cause cause)
 
         remote_ra_process_maintenance_nodes(xml);
 
+    } else if (strcmp(op, PCMK__CONTROLD_CMD_NODES) == 0) {
+        return handle_node_list(stored_msg);
+
         /*========== (NOT_DC)-Only Actions ==========*/
-    } else if (AM_I_DC == FALSE && strcmp(op, CRM_OP_SHUTDOWN) == 0) {
+    } else if (!AM_I_DC) {
 
-        const char *host_from = crm_element_value(stored_msg, F_CRM_HOST_FROM);
-        gboolean dc_match = safe_str_eq(host_from, fsa_our_dc);
-
-        if (dc_match || fsa_our_dc == NULL) {
-            if (is_set(fsa_input_register, R_SHUTDOWN) == FALSE) {
-                crm_err("We didn't ask to be shut down, yet our DC is telling us to.");
-                set_bit(fsa_input_register, R_STAYDOWN);
-                return I_STOP;
-            }
-            crm_info("Shutting down");
-            return I_STOP;
-
-        } else {
-            crm_warn("Discarding %s op from %s", op, host_from);
+        if (strcmp(op, CRM_OP_SHUTDOWN) == 0) {
+            return handle_shutdown_ack(stored_msg);
         }
 
     } else {
@@ -993,8 +1138,8 @@ handle_request(xmlNode * stored_msg, enum crmd_fsa_cause cause)
     return I_NULL;
 }
 
-void
-handle_response(xmlNode * stored_msg)
+static void
+handle_response(xmlNode *stored_msg)
 {
     const char *op = crm_element_value(stored_msg, F_CRM_TASK);
 
@@ -1008,7 +1153,7 @@ handle_response(xmlNode * stored_msg)
         if (msg_ref == NULL) {
             crm_err("%s - Ignoring calculation with no reference", op);
 
-        } else if (safe_str_eq(msg_ref, fsa_pe_ref)) {
+        } else if (pcmk__str_eq(msg_ref, fsa_pe_ref, pcmk__str_casei)) {
             ha_msg_input_t fsa_input;
 
             controld_stop_sched_timer();
@@ -1030,7 +1175,7 @@ handle_response(xmlNode * stored_msg)
     }
 }
 
-enum crmd_fsa_input
+static enum crmd_fsa_input
 handle_shutdown_request(xmlNode * stored_msg)
 {
     /* handle here to avoid potential version issues
@@ -1063,11 +1208,10 @@ handle_shutdown_request(xmlNode * stored_msg)
 /* msg is deleted by the time this returns */
 extern gboolean process_te_message(xmlNode * msg, xmlNode * xml_data);
 
-gboolean
+static void
 send_msg_via_ipc(xmlNode * msg, const char *sys)
 {
-    gboolean send_ok = TRUE;
-    crm_client_t *client_channel = crm_client_get_by_id(sys);
+    pcmk__client_t *client_channel = pcmk__find_client_by_id(sys);
 
     if (crm_element_value(msg, F_CRM_HOST_FROM) == NULL) {
         crm_xml_add(msg, F_CRM_HOST_FROM, fsa_our_uname);
@@ -1075,7 +1219,7 @@ send_msg_via_ipc(xmlNode * msg, const char *sys)
 
     if (client_channel != NULL) {
         /* Transient clients such as crmadmin */
-        send_ok = crm_ipcs_send(client_channel, 0, msg, crm_ipc_server_event);
+        pcmk__ipc_send_xml(client_channel, 0, msg, crm_ipc_server_event);
 
     } else if (sys != NULL && strcmp(sys, CRM_SYSTEM_TENGINE) == 0) {
         xmlNode *data = get_message_xml(msg, F_CRM_DATA);
@@ -1094,12 +1238,9 @@ send_msg_via_ipc(xmlNode * msg, const char *sys)
         fsa_data.data = &fsa_input;
         fsa_data.fsa_input = I_MESSAGE;
         fsa_data.fsa_cause = C_IPC_MESSAGE;
-        fsa_data.origin = __FUNCTION__;
+        fsa_data.origin = __func__;
         fsa_data.data_type = fsa_dt_ha_msg;
 
-#ifdef FSA_TRACE
-        crm_trace("Invoking action A_LRM_INVOKE (%.16llx)", A_LRM_INVOKE);
-#endif
         do_lrm_invoke(A_LRM_INVOKE, C_IPC_MESSAGE, fsa_state, I_MESSAGE, &fsa_data);
 
     } else if (sys != NULL && crmd_is_proxy_session(sys)) {
@@ -1107,21 +1248,7 @@ send_msg_via_ipc(xmlNode * msg, const char *sys)
 
     } else {
         crm_debug("Unknown Sub-system (%s)... discarding message.", crm_str(sys));
-        send_ok = FALSE;
     }
-
-    return send_ok;
-}
-
-ha_msg_input_t *
-new_ha_msg_input(xmlNode * orig)
-{
-    ha_msg_input_t *input_copy = NULL;
-
-    input_copy = calloc(1, sizeof(ha_msg_input_t));
-    input_copy->msg = orig;
-    input_copy->xml = get_message_xml(input_copy->msg, F_CRM_DATA);
-    return input_copy;
 }
 
 void
