@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -43,7 +43,7 @@ te_pseudo_action(crm_graph_t * graph, crm_action_t * pseudo)
     const char *task = crm_element_value(pseudo->xml, XML_LRM_ATTR_TASK);
 
     /* send to peers as well? */
-    if (safe_str_eq(task, CRM_OP_MAINTENANCE_NODES)) {
+    if (pcmk__str_eq(task, CRM_OP_MAINTENANCE_NODES, pcmk__str_casei)) {
         GHashTableIter iter;
         crm_node_t *node = NULL;
 
@@ -51,7 +51,7 @@ te_pseudo_action(crm_graph_t * graph, crm_action_t * pseudo)
         while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &node)) {
             xmlNode *cmd = NULL;
 
-            if (safe_str_eq(fsa_our_uname, node->uname)) {
+            if (pcmk__str_eq(fsa_our_uname, node->uname, pcmk__str_casei)) {
                 continue;
             }
 
@@ -107,13 +107,20 @@ te_crm_command(crm_graph_t * graph, crm_action_t * action)
 
     if (!router_node) {
         router_node = on_node;
+        if (pcmk__str_eq(task, CRM_OP_LRM_DELETE, pcmk__str_casei)) {
+            const char *mode = crm_element_value(action->xml, PCMK__XA_MODE);
+
+            if (pcmk__str_eq(mode, XML_TAG_CIB, pcmk__str_casei)) {
+                router_node = fsa_our_uname;
+            }
+        }
     }
 
     CRM_CHECK(on_node != NULL && strlen(on_node) != 0,
               crm_err("Corrupted command (id=%s) %s: no node", crm_str(id), crm_str(task));
               return FALSE);
 
-    if (safe_str_eq(router_node, fsa_our_uname)) {
+    if (pcmk__str_eq(router_node, fsa_our_uname, pcmk__str_casei)) {
         is_local = TRUE;
     }
 
@@ -126,7 +133,7 @@ te_crm_command(crm_graph_t * graph, crm_action_t * action)
              crm_str(id), (is_local? " locally" : ""),
              (no_wait? " without waiting" : ""), crm_str(task), on_node);
 
-    if (is_local && safe_str_eq(task, CRM_OP_SHUTDOWN)) {
+    if (is_local && pcmk__str_eq(task, CRM_OP_SHUTDOWN, pcmk__str_casei)) {
         /* defer until everything else completes */
         crm_info("crm-event (%s) is a local shutdown", crm_str(id));
         graph->completion_action = tg_shutdown;
@@ -134,15 +141,15 @@ te_crm_command(crm_graph_t * graph, crm_action_t * action)
         te_action_confirmed(action, graph);
         return TRUE;
 
-    } else if (safe_str_eq(task, CRM_OP_SHUTDOWN)) {
+    } else if (pcmk__str_eq(task, CRM_OP_SHUTDOWN, pcmk__str_casei)) {
         crm_node_t *peer = crm_get_peer(0, router_node);
-        crm_update_peer_expected(__FUNCTION__, peer, CRMD_JOINSTATE_DOWN);
+        crm_update_peer_expected(__func__, peer, CRMD_JOINSTATE_DOWN);
     }
 
     cmd = create_request(task, action->xml, router_node, CRM_SYSTEM_CRMD, CRM_SYSTEM_TENGINE, NULL);
 
-    counter =
-        generate_transition_key(transition_graph->id, action->id, get_target_rc(action), te_uuid);
+    counter = pcmk__transition_key(transition_graph->id, action->id,
+                                   get_target_rc(action), te_uuid);
     crm_xml_add(cmd, XML_ATTR_TRANSITION_KEY, counter);
 
     rc = send_cluster_message(crm_get_peer(0, router_node), crm_msg_crmd, cmd, TRUE);
@@ -237,10 +244,11 @@ controld_record_action_timeout(crm_action_t *action)
     op = convert_graph_action(NULL, action, PCMK_LRM_OP_TIMEOUT,
                               PCMK_OCF_UNKNOWN_ERROR);
     op->call_id = -1;
-    op->user_data = generate_transition_key(transition_graph->id, action->id, target_rc, te_uuid);
+    op->user_data = pcmk__transition_key(transition_graph->id, action->id,
+                                         target_rc, te_uuid);
 
     xml_op = pcmk__create_history_xml(rsc, op, CRM_FEATURE_SET, target_rc,
-                                      target, __FUNCTION__, LOG_INFO);
+                                      target, __func__, LOG_INFO);
     lrmd_free_event(op);
 
     crm_log_xml_trace(xml_op, "Action timeout");
@@ -296,11 +304,11 @@ te_rsc_command(crm_graph_t * graph, crm_action_t * action)
         router_node = on_node;
     }
 
-    counter =
-        generate_transition_key(transition_graph->id, action->id, get_target_rc(action), te_uuid);
+    counter = pcmk__transition_key(transition_graph->id, action->id,
+                                   get_target_rc(action), te_uuid);
     crm_xml_add(rsc_op, XML_ATTR_TRANSITION_KEY, counter);
 
-    if (safe_str_eq(router_node, fsa_our_uname)) {
+    if (pcmk__str_eq(router_node, fsa_our_uname, pcmk__str_casei)) {
         is_local = TRUE;
     }
 
@@ -330,7 +338,7 @@ te_rsc_command(crm_graph_t * graph, crm_action_t * action)
             .fsa_input = I_NULL,
             .fsa_cause = C_FSA_INTERNAL,
             .actions = A_LRM_INVOKE,
-            .origin = __FUNCTION__,
+            .origin = __func__,
         };
 
         do_lrm_invoke(A_LRM_INVOKE, C_FSA_INTERNAL, fsa_state, I_NULL, &msg);
@@ -443,8 +451,8 @@ te_update_job_count(crm_action_t * action, int offset)
      * the connection resources */
     target = crm_element_value(action->xml, XML_LRM_ATTR_ROUTER_NODE);
 
-    if ((target == NULL) &&
-        (safe_str_eq(task, CRMD_ACTION_MIGRATE) || safe_str_eq(task, CRMD_ACTION_MIGRATED))) {
+    if ((target == NULL) && pcmk__strcase_any_of(task, CRMD_ACTION_MIGRATE,
+                                                 CRMD_ACTION_MIGRATED, NULL)) {
 
         const char *t1 = crm_meta_value(action->params, XML_LRM_ATTR_MIGRATE_SOURCE);
         const char *t2 = crm_meta_value(action->params, XML_LRM_ATTR_MIGRATE_TARGET);
@@ -490,7 +498,7 @@ te_should_perform_action_on(crm_graph_t * graph, crm_action_t * action, const ch
         return FALSE;
 
     } else if(graph->migration_limit > 0 && r->migrate_jobs >= graph->migration_limit) {
-        if (safe_str_eq(task, CRMD_ACTION_MIGRATE) || safe_str_eq(task, CRMD_ACTION_MIGRATED)) {
+        if (pcmk__strcase_any_of(task, CRMD_ACTION_MIGRATE, CRMD_ACTION_MIGRATED, NULL)) {
             crm_trace("Peer %s is over their migration job limit of %d (%d): deferring %s",
                       target, graph->migration_limit, r->migrate_jobs, id);
             return FALSE;
@@ -519,9 +527,8 @@ te_should_perform_action(crm_graph_t * graph, crm_action_t * action)
      * the connection resources */
     target = crm_element_value(action->xml, XML_LRM_ATTR_ROUTER_NODE);
 
-    if ((target == NULL) &&
-        (safe_str_eq(task, CRMD_ACTION_MIGRATE) || safe_str_eq(task, CRMD_ACTION_MIGRATED))) {
-
+    if ((target == NULL) && pcmk__strcase_any_of(task, CRMD_ACTION_MIGRATE,
+                                                 CRMD_ACTION_MIGRATED, NULL)) {
         target = crm_meta_value(action->params, XML_LRM_ATTR_MIGRATE_SOURCE);
         if(te_should_perform_action_on(graph, action, target) == FALSE) {
             return FALSE;
@@ -605,13 +612,14 @@ notify_crmd(crm_graph_t * graph)
                 }
 
             } else if (fsa_state == S_POLICY_ENGINE) {
-                register_fsa_action(A_PE_INVOKE);
+                controld_set_fsa_action_flags(A_PE_INVOKE);
+                trigger_fsa();
             }
             break;
 
         case tg_shutdown:
             type = "shutdown";
-            if (is_set(fsa_input_register, R_SHUTDOWN)) {
+            if (pcmk_is_set(fsa_input_register, R_SHUTDOWN)) {
                 event = I_STOP;
 
             } else {
@@ -624,7 +632,7 @@ notify_crmd(crm_graph_t * graph)
 
     graph->abort_reason = NULL;
     graph->completion_action = tg_done;
-    clear_bit(fsa_input_register, R_IN_TRANSITION);
+    controld_clear_fsa_input_flags(R_IN_TRANSITION);
 
     if (event != I_NULL) {
         register_fsa_input(C_FSA_INTERNAL, event, NULL);
