@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -13,6 +13,7 @@
 
 #include <crm/crm.h>
 #include <crm/common/xml.h>
+#include <crm/common/xml_internal.h>
 #include <crm/msg_xml.h>
 #include <crm/cluster.h>        /* For ONLINESTATUS etc */
 
@@ -28,6 +29,17 @@ crm_trigger_t *transition_trigger = NULL;
 /* #define RSC_OP_TEMPLATE "//"XML_TAG_DIFF_ADDED"//"XML_TAG_CIB"//"XML_CIB_TAG_STATE"[@uname='%s']"//"XML_LRM_TAG_RSC_OP"[@id='%s]" */
 #define RSC_OP_TEMPLATE "//"XML_TAG_DIFF_ADDED"//"XML_TAG_CIB"//"XML_LRM_TAG_RSC_OP"[@id='%s']"
 
+// An explicit shutdown-lock of 0 means the lock has been cleared
+static bool
+shutdown_lock_cleared(xmlNode *lrm_resource)
+{
+    time_t shutdown_lock = 0;
+
+    return (crm_element_value_epoch(lrm_resource, XML_CONFIG_ATTR_SHUTDOWN_LOCK,
+                                    &shutdown_lock) == pcmk_ok)
+           && (shutdown_lock == 0);
+}
+
 static void
 te_update_diff_v1(const char *event, xmlNode *diff)
 {
@@ -36,7 +48,7 @@ te_update_diff_v1(const char *event, xmlNode *diff)
 
     CRM_CHECK(diff != NULL, return);
 
-    xml_log_patchset(LOG_TRACE, __FUNCTION__, diff);
+    xml_log_patchset(LOG_TRACE, __func__, diff);
     if (cib_config_changed(NULL, NULL, &diff)) {
         abort_transition(INFINITY, tg_restart, "Non-status change", diff);
         goto bail;              /* configuration changed */
@@ -79,7 +91,7 @@ te_update_diff_v1(const char *event, xmlNode *diff)
         const char *name = crm_element_value(attr, XML_NVPAIR_ATTR_NAME);
         const char *value = NULL;
 
-        if (safe_str_eq(CRM_OP_PROBED, name)) {
+        if (pcmk__str_eq(CRM_OP_PROBED, name, pcmk__str_casei)) {
             value = crm_element_value(attr, XML_NVPAIR_ATTR_VALUE);
         }
 
@@ -106,34 +118,43 @@ te_update_diff_v1(const char *event, xmlNode *diff)
     }
     freeXpathObject(xpathObj);
 
+    // Check for lrm_resource entries
+    xpathObj = xpath_search(diff,
+                            "//" F_CIB_UPDATE_RESULT
+                            "//" XML_TAG_DIFF_ADDED
+                            "//" XML_LRM_TAG_RESOURCE);
+    max = numXpathResults(xpathObj);
+
     /*
-     * Updates by, or in response to, TE actions will never contain updates
-     * for more than one resource at a time, so such updates indicate an
-     * LRM refresh.
-     *
-     * In that case, start a new transition rather than check each result
-     * individually, which can result in _huge_ speedups in large clusters.
+     * Updates by, or in response to, graph actions will never affect more than
+     * one resource at a time, so such updates indicate an LRM refresh. In that
+     * case, start a new transition rather than check each result individually,
+     * which can result in _huge_ speedups in large clusters.
      *
      * Unfortunately, we can only do so when there are no pending actions.
      * Otherwise, we could mistakenly throw away those results here, and
      * the cluster will stall waiting for them and time out the operation.
      */
-    if (transition_graph->pending == 0) {
-        xpathObj = xpath_search(diff,
-                                "//" F_CIB_UPDATE_RESULT
-                                "//" XML_TAG_DIFF_ADDED
-                                "//" XML_LRM_TAG_RESOURCE);
-        max = numXpathResults(xpathObj);
-        if (max > 1) {
-            crm_debug("Ignoring resource operation updates due to history refresh of %d resources",
-                      max);
-            crm_log_xml_trace(diff, "lrm-refresh");
-            abort_transition(INFINITY, tg_restart, "History refresh", NULL);
-            goto bail;
-        }
-        freeXpathObject(xpathObj);
+    if ((transition_graph->pending == 0) && (max > 1)) {
+        crm_debug("Ignoring resource operation updates due to history refresh of %d resources",
+                  max);
+        crm_log_xml_trace(diff, "lrm-refresh");
+        abort_transition(INFINITY, tg_restart, "History refresh", NULL);
+        goto bail;
     }
 
+    if (max == 1) {
+        xmlNode *lrm_resource = getXpathResult(xpathObj, 0);
+
+        if (shutdown_lock_cleared(lrm_resource)) {
+            // @TODO would be more efficient to abort once after transition done
+            abort_transition(INFINITY, tg_restart, "Shutdown lock cleared",
+                             lrm_resource);
+            // Still process results, so we stop timers and update failcounts
+        }
+    }
+    freeXpathObject(xpathObj);
+
     /* Process operation updates */
     xpathObj =
         xpath_search(diff,
@@ -201,10 +222,15 @@ te_update_diff_v1(const char *event, xmlNode *diff)
 static void
 process_lrm_resource_diff(xmlNode *lrm_resource, const char *node)
 {
-    for (xmlNode *rsc_op = __xml_first_child(lrm_resource); rsc_op != NULL;
-         rsc_op = __xml_next(rsc_op)) {
+    for (xmlNode *rsc_op = pcmk__xml_first_child(lrm_resource); rsc_op != NULL;
+         rsc_op = pcmk__xml_next(rsc_op)) {
         process_graph_event(rsc_op, node);
     }
+    if (shutdown_lock_cleared(lrm_resource)) {
+        // @TODO would be more efficient to abort once after transition done
+        abort_transition(INFINITY, tg_restart, "Shutdown lock cleared",
+                         lrm_resource);
+    }
 }
 
 static void
@@ -243,7 +269,8 @@ process_resource_updates(const char *node, xmlNode *xml, xmlNode *change,
         return;
     }
 
-    for (rsc = __xml_first_child(xml); rsc != NULL; rsc = __xml_next(rsc)) {
+    for (rsc = pcmk__xml_first_child(xml); rsc != NULL;
+         rsc = pcmk__xml_next(rsc)) {
         crm_trace("Processing %s", ID(rsc));
         process_lrm_resource_diff(rsc, node);
     }
@@ -300,7 +327,7 @@ abort_unless_down(const char *xpath, const char *op, xmlNode *change,
     char *node_uuid = NULL;
     crm_action_t *down = NULL;
 
-    if(safe_str_neq(op, "delete")) {
+    if(!pcmk__str_eq(op, "delete", pcmk__str_casei)) {
         abort_transition(INFINITY, tg_restart, reason, change);
         return;
     }
@@ -382,8 +409,8 @@ static void
 process_status_diff(xmlNode *status, xmlNode *change, const char *op,
                     const char *xpath)
 {
-    for (xmlNode *state = __xml_first_child(status); state != NULL;
-         state = __xml_next(state)) {
+    for (xmlNode *state = pcmk__xml_first_child(status); state != NULL;
+         state = pcmk__xml_next(state)) {
         process_node_state_diff(state, change, op, xpath);
     }
 }
@@ -409,8 +436,8 @@ te_update_diff_v2(xmlNode *diff)
 {
     crm_log_xml_trace(diff, "Patch:Raw");
 
-    for (xmlNode *change = __xml_first_child(diff); change != NULL;
-         change = __xml_next(change)) {
+    for (xmlNode *change = pcmk__xml_first_child(diff); change != NULL;
+         change = pcmk__xml_next(change)) {
 
         xmlNode *match = NULL;
         const char *name = NULL;
@@ -466,12 +493,12 @@ te_update_diff_v2(xmlNode *diff)
             break; // Won't be packaged with operation results we may be waiting for
 
         } else if (strstr(xpath, "/" XML_CIB_TAG_TICKETS)
-                   || safe_str_eq(name, XML_CIB_TAG_TICKETS)) {
+                   || pcmk__str_eq(name, XML_CIB_TAG_TICKETS, pcmk__str_casei)) {
             abort_transition(INFINITY, tg_restart, "Ticket attribute change", change);
             break; // Won't be packaged with operation results we may be waiting for
 
         } else if (strstr(xpath, "/" XML_TAG_TRANSIENT_NODEATTRS "[")
-                   || safe_str_eq(name, XML_TAG_TRANSIENT_NODEATTRS)) {
+                   || pcmk__str_eq(name, XML_TAG_TRANSIENT_NODEATTRS, pcmk__str_casei)) {
             abort_unless_down(xpath, op, change, "Transient attribute change");
             break; // Won't be packaged with operation results we may be waiting for
 
@@ -591,9 +618,9 @@ process_te_message(xmlNode * msg, xmlNode * xml_data)
         crm_trace("Bad sys-to %s", crm_str(sys_to));
         return FALSE;
 
-    } else if (safe_str_eq(op, CRM_OP_INVOKE_LRM)
-               && safe_str_eq(sys_from, CRM_SYSTEM_LRMD)
-/* 		  && safe_str_eq(type, XML_ATTR_RESPONSE) */
+    } else if (pcmk__str_eq(op, CRM_OP_INVOKE_LRM, pcmk__str_casei)
+               && pcmk__str_eq(sys_from, CRM_SYSTEM_LRMD, pcmk__str_casei)
+/* 		  && pcmk__str_eq(type, XML_ATTR_RESPONSE, pcmk__str_casei) */
         ) {
         xmlXPathObject *xpathObj = NULL;
 
@@ -672,7 +699,8 @@ action_timer_callback(gpointer data)
         crm_err("Node %s did not send %s result (via %s) within %dms "
                 "(action timeout plus cluster-delay)",
                 (on_node? on_node : ""), (task? task : "unknown action"),
-                (via_node? via_node : "controller"), timer->timeout);
+                (via_node? via_node : "controller"),
+                timer->timeout + transition_graph->network_delay);
         print_action(LOG_ERR, "Aborting transition, action lost: ", timer->action);
 
         timer->action->failed = TRUE;
