@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -52,7 +52,6 @@ te_graph_trigger(gpointer user_data)
         case S_STOPPING:
         case S_TERMINATE:
             return TRUE;
-            break;
         default:
             break;
     }
@@ -176,7 +175,7 @@ abort_transition_graph(int abort_priority, enum transition_action abort_action,
         xmlNode *search = NULL;
 
         for(search = reason; search; search = search->parent) {
-            if (safe_str_eq(XML_TAG_DIFF, TYPE(search))) {
+            if (pcmk__str_eq(XML_TAG_DIFF, TYPE(search), pcmk__str_casei)) {
                 diff = search;
                 break;
             }
@@ -185,7 +184,7 @@ abort_transition_graph(int abort_priority, enum transition_action abort_action,
         if(diff) {
             xml_patch_versions(diff, add, del);
             for(search = reason; search; search = search->parent) {
-                if (safe_str_eq(XML_DIFF_CHANGE, TYPE(search))) {
+                if (pcmk__str_eq(XML_DIFF_CHANGE, TYPE(search), pcmk__str_casei)) {
                     change = search;
                     break;
                 }
@@ -196,7 +195,7 @@ abort_transition_graph(int abort_priority, enum transition_action abort_action,
     if(reason == NULL) {
         do_crm_log(level, "Transition %d aborted: %s "CRM_XS" source=%s:%d complete=%s",
                    transition_graph->id, abort_text, fn, line,
-                   (transition_graph->complete? "true" : "false"));
+                   pcmk__btoa(transition_graph->complete));
 
     } else if(change == NULL) {
         char *local_path = xml_get_path(reason);
@@ -205,7 +204,7 @@ abort_transition_graph(int abort_priority, enum transition_action abort_action,
                    CRM_XS " cib=%d.%d.%d source=%s:%d path=%s complete=%s",
                    transition_graph->id, TYPE(reason), ID(reason), abort_text,
                    add[0], add[1], add[2], fn, line, local_path,
-                   (transition_graph->complete? "true" : "false"));
+                   pcmk__btoa(transition_graph->complete));
         free(local_path);
 
     } else {
@@ -234,9 +233,9 @@ abort_transition_graph(int abort_priority, enum transition_action abort_action,
                        transition_graph->id,
                        (shortpath? (shortpath + 1) : path), abort_text,
                        add[0], add[1], add[2], fn, line, path,
-                       (transition_graph->complete? "true" : "false"));
+                       pcmk__btoa(transition_graph->complete));
 
-        } else if (safe_str_eq(XML_CIB_TAG_NVPAIR, kind)) { 
+        } else if (pcmk__str_eq(XML_CIB_TAG_NVPAIR, kind, pcmk__str_casei)) { 
             do_crm_log(level, "Transition %d aborted by %s doing %s %s=%s: %s "
                        CRM_XS " cib=%d.%d.%d source=%s:%d path=%s complete=%s",
                        transition_graph->id,
@@ -244,9 +243,9 @@ abort_transition_graph(int abort_priority, enum transition_action abort_action,
                        crm_element_value(reason, XML_NVPAIR_ATTR_NAME),
                        crm_element_value(reason, XML_NVPAIR_ATTR_VALUE),
                        abort_text, add[0], add[1], add[2], fn, line, path,
-                       (transition_graph->complete? "true" : "false"));
+                       pcmk__btoa(transition_graph->complete));
 
-        } else if (safe_str_eq(XML_LRM_TAG_RSC_OP, kind)) {
+        } else if (pcmk__str_eq(XML_LRM_TAG_RSC_OP, kind, pcmk__str_casei)) {
             const char *magic = crm_element_value(reason, XML_ATTR_TRANSITION_MAGIC);
 
             do_crm_log(level, "Transition %d aborted by operation %s '%s' on %s: %s "
@@ -255,10 +254,9 @@ abort_transition_graph(int abort_priority, enum transition_action abort_action,
                        crm_element_value(reason, XML_LRM_ATTR_TASK_KEY), op,
                        crm_element_value(reason, XML_LRM_ATTR_TARGET), abort_text,
                        magic, add[0], add[1], add[2], fn, line,
-                       (transition_graph->complete? "true" : "false"));
+                       pcmk__btoa(transition_graph->complete));
 
-        } else if (safe_str_eq(XML_CIB_TAG_STATE, kind)
-                   || safe_str_eq(XML_CIB_TAG_NODE, kind)) {
+        } else if (pcmk__strcase_any_of(kind, XML_CIB_TAG_STATE, XML_CIB_TAG_NODE, NULL)) {
             const char *uname = crm_peer_uname(ID(reason));
 
             do_crm_log(level, "Transition %d aborted by %s '%s' on %s: %s "
@@ -266,7 +264,7 @@ abort_transition_graph(int abort_priority, enum transition_action abort_action,
                        transition_graph->id,
                        kind, op, (uname? uname : ID(reason)), abort_text,
                        add[0], add[1], add[2], fn, line,
-                       (transition_graph->complete? "true" : "false"));
+                       pcmk__btoa(transition_graph->complete));
 
         } else {
             const char *id = ID(reason);
@@ -276,7 +274,7 @@ abort_transition_graph(int abort_priority, enum transition_action abort_action,
                        transition_graph->id,
                        TYPE(reason), (id? id : ""), (op? op : "change"),
                        abort_text, add[0], add[1], add[2], fn, line, path,
-                       (transition_graph->complete? "true" : "false"));
+                       pcmk__btoa(transition_graph->complete));
         }
     }
 
