@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -10,6 +10,7 @@
 #include <crm_internal.h>
 
 #include <stdlib.h>
+#include <stdint.h>                 // uint64_t
 
 #include <crm/crm.h>
 #include <crm/cib.h>
@@ -454,234 +455,234 @@ fsa_dump_inputs(int log_level, const char *text, long long input_register)
         text = "Input register contents:";
     }
 
-    if (is_set(input_register, R_THE_DC)) {
+    if (pcmk_is_set(input_register, R_THE_DC)) {
         crm_trace("%s %.16llx (R_THE_DC)", text, R_THE_DC);
     }
-    if (is_set(input_register, R_STARTING)) {
+    if (pcmk_is_set(input_register, R_STARTING)) {
         crm_trace("%s %.16llx (R_STARTING)", text, R_STARTING);
     }
-    if (is_set(input_register, R_SHUTDOWN)) {
+    if (pcmk_is_set(input_register, R_SHUTDOWN)) {
         crm_trace("%s %.16llx (R_SHUTDOWN)", text, R_SHUTDOWN);
     }
-    if (is_set(input_register, R_STAYDOWN)) {
+    if (pcmk_is_set(input_register, R_STAYDOWN)) {
         crm_trace("%s %.16llx (R_STAYDOWN)", text, R_STAYDOWN);
     }
-    if (is_set(input_register, R_JOIN_OK)) {
+    if (pcmk_is_set(input_register, R_JOIN_OK)) {
         crm_trace("%s %.16llx (R_JOIN_OK)", text, R_JOIN_OK);
     }
-    if (is_set(input_register, R_READ_CONFIG)) {
+    if (pcmk_is_set(input_register, R_READ_CONFIG)) {
         crm_trace("%s %.16llx (R_READ_CONFIG)", text, R_READ_CONFIG);
     }
-    if (is_set(input_register, R_INVOKE_PE)) {
+    if (pcmk_is_set(input_register, R_INVOKE_PE)) {
         crm_trace("%s %.16llx (R_INVOKE_PE)", text, R_INVOKE_PE);
     }
-    if (is_set(input_register, R_CIB_CONNECTED)) {
+    if (pcmk_is_set(input_register, R_CIB_CONNECTED)) {
         crm_trace("%s %.16llx (R_CIB_CONNECTED)", text, R_CIB_CONNECTED);
     }
-    if (is_set(input_register, R_PE_CONNECTED)) {
+    if (pcmk_is_set(input_register, R_PE_CONNECTED)) {
         crm_trace("%s %.16llx (R_PE_CONNECTED)", text, R_PE_CONNECTED);
     }
-    if (is_set(input_register, R_TE_CONNECTED)) {
+    if (pcmk_is_set(input_register, R_TE_CONNECTED)) {
         crm_trace("%s %.16llx (R_TE_CONNECTED)", text, R_TE_CONNECTED);
     }
-    if (is_set(input_register, R_LRM_CONNECTED)) {
+    if (pcmk_is_set(input_register, R_LRM_CONNECTED)) {
         crm_trace("%s %.16llx (R_LRM_CONNECTED)", text, R_LRM_CONNECTED);
     }
-    if (is_set(input_register, R_CIB_REQUIRED)) {
+    if (pcmk_is_set(input_register, R_CIB_REQUIRED)) {
         crm_trace("%s %.16llx (R_CIB_REQUIRED)", text, R_CIB_REQUIRED);
     }
-    if (is_set(input_register, R_PE_REQUIRED)) {
+    if (pcmk_is_set(input_register, R_PE_REQUIRED)) {
         crm_trace("%s %.16llx (R_PE_REQUIRED)", text, R_PE_REQUIRED);
     }
-    if (is_set(input_register, R_TE_REQUIRED)) {
+    if (pcmk_is_set(input_register, R_TE_REQUIRED)) {
         crm_trace("%s %.16llx (R_TE_REQUIRED)", text, R_TE_REQUIRED);
     }
-    if (is_set(input_register, R_REQ_PEND)) {
+    if (pcmk_is_set(input_register, R_REQ_PEND)) {
         crm_trace("%s %.16llx (R_REQ_PEND)", text, R_REQ_PEND);
     }
-    if (is_set(input_register, R_PE_PEND)) {
+    if (pcmk_is_set(input_register, R_PE_PEND)) {
         crm_trace("%s %.16llx (R_PE_PEND)", text, R_PE_PEND);
     }
-    if (is_set(input_register, R_TE_PEND)) {
+    if (pcmk_is_set(input_register, R_TE_PEND)) {
         crm_trace("%s %.16llx (R_TE_PEND)", text, R_TE_PEND);
     }
-    if (is_set(input_register, R_RESP_PEND)) {
+    if (pcmk_is_set(input_register, R_RESP_PEND)) {
         crm_trace("%s %.16llx (R_RESP_PEND)", text, R_RESP_PEND);
     }
-    if (is_set(input_register, R_CIB_DONE)) {
+    if (pcmk_is_set(input_register, R_CIB_DONE)) {
         crm_trace("%s %.16llx (R_CIB_DONE)", text, R_CIB_DONE);
     }
-    if (is_set(input_register, R_HAVE_CIB)) {
+    if (pcmk_is_set(input_register, R_HAVE_CIB)) {
         crm_trace("%s %.16llx (R_HAVE_CIB)", text, R_HAVE_CIB);
     }
-    if (is_set(input_register, R_CIB_ASKED)) {
+    if (pcmk_is_set(input_register, R_CIB_ASKED)) {
         crm_trace("%s %.16llx (R_CIB_ASKED)", text, R_CIB_ASKED);
     }
-    if (is_set(input_register, R_MEMBERSHIP)) {
+    if (pcmk_is_set(input_register, R_MEMBERSHIP)) {
         crm_trace("%s %.16llx (R_MEMBERSHIP)", text, R_MEMBERSHIP);
     }
-    if (is_set(input_register, R_PEER_DATA)) {
+    if (pcmk_is_set(input_register, R_PEER_DATA)) {
         crm_trace("%s %.16llx (R_PEER_DATA)", text, R_PEER_DATA);
     }
-    if (is_set(input_register, R_IN_RECOVERY)) {
+    if (pcmk_is_set(input_register, R_IN_RECOVERY)) {
         crm_trace("%s %.16llx (R_IN_RECOVERY)", text, R_IN_RECOVERY);
     }
 }
 
 void
-fsa_dump_actions(long long action, const char *text)
+fsa_dump_actions(uint64_t action, const char *text)
 {
-    if (is_set(action, A_READCONFIG)) {
+    if (pcmk_is_set(action, A_READCONFIG)) {
         crm_trace("Action %.16llx (A_READCONFIG) %s", A_READCONFIG, text);
     }
-    if (is_set(action, A_STARTUP)) {
+    if (pcmk_is_set(action, A_STARTUP)) {
         crm_trace("Action %.16llx (A_STARTUP) %s", A_STARTUP, text);
     }
-    if (is_set(action, A_STARTED)) {
+    if (pcmk_is_set(action, A_STARTED)) {
         crm_trace("Action %.16llx (A_STARTED) %s", A_STARTED, text);
     }
-    if (is_set(action, A_HA_CONNECT)) {
+    if (pcmk_is_set(action, A_HA_CONNECT)) {
         crm_trace("Action %.16llx (A_CONNECT) %s", A_HA_CONNECT, text);
     }
-    if (is_set(action, A_HA_DISCONNECT)) {
+    if (pcmk_is_set(action, A_HA_DISCONNECT)) {
         crm_trace("Action %.16llx (A_DISCONNECT) %s", A_HA_DISCONNECT, text);
     }
-    if (is_set(action, A_LRM_CONNECT)) {
+    if (pcmk_is_set(action, A_LRM_CONNECT)) {
         crm_trace("Action %.16llx (A_LRM_CONNECT) %s", A_LRM_CONNECT, text);
     }
-    if (is_set(action, A_LRM_EVENT)) {
+    if (pcmk_is_set(action, A_LRM_EVENT)) {
         crm_trace("Action %.16llx (A_LRM_EVENT) %s", A_LRM_EVENT, text);
     }
-    if (is_set(action, A_LRM_INVOKE)) {
+    if (pcmk_is_set(action, A_LRM_INVOKE)) {
         crm_trace("Action %.16llx (A_LRM_INVOKE) %s", A_LRM_INVOKE, text);
     }
-    if (is_set(action, A_LRM_DISCONNECT)) {
+    if (pcmk_is_set(action, A_LRM_DISCONNECT)) {
         crm_trace("Action %.16llx (A_LRM_DISCONNECT) %s", A_LRM_DISCONNECT, text);
     }
-    if (is_set(action, A_DC_TIMER_STOP)) {
+    if (pcmk_is_set(action, A_DC_TIMER_STOP)) {
         crm_trace("Action %.16llx (A_DC_TIMER_STOP) %s", A_DC_TIMER_STOP, text);
     }
-    if (is_set(action, A_DC_TIMER_START)) {
+    if (pcmk_is_set(action, A_DC_TIMER_START)) {
         crm_trace("Action %.16llx (A_DC_TIMER_START) %s", A_DC_TIMER_START, text);
     }
-    if (is_set(action, A_INTEGRATE_TIMER_START)) {
+    if (pcmk_is_set(action, A_INTEGRATE_TIMER_START)) {
         crm_trace("Action %.16llx (A_INTEGRATE_TIMER_START) %s", A_INTEGRATE_TIMER_START, text);
     }
-    if (is_set(action, A_INTEGRATE_TIMER_STOP)) {
+    if (pcmk_is_set(action, A_INTEGRATE_TIMER_STOP)) {
         crm_trace("Action %.16llx (A_INTEGRATE_TIMER_STOP) %s", A_INTEGRATE_TIMER_STOP, text);
     }
-    if (is_set(action, A_FINALIZE_TIMER_START)) {
+    if (pcmk_is_set(action, A_FINALIZE_TIMER_START)) {
         crm_trace("Action %.16llx (A_FINALIZE_TIMER_START) %s", A_FINALIZE_TIMER_START, text);
     }
-    if (is_set(action, A_FINALIZE_TIMER_STOP)) {
+    if (pcmk_is_set(action, A_FINALIZE_TIMER_STOP)) {
         crm_trace("Action %.16llx (A_FINALIZE_TIMER_STOP) %s", A_FINALIZE_TIMER_STOP, text);
     }
-    if (is_set(action, A_ELECTION_COUNT)) {
+    if (pcmk_is_set(action, A_ELECTION_COUNT)) {
         crm_trace("Action %.16llx (A_ELECTION_COUNT) %s", A_ELECTION_COUNT, text);
     }
-    if (is_set(action, A_ELECTION_VOTE)) {
+    if (pcmk_is_set(action, A_ELECTION_VOTE)) {
         crm_trace("Action %.16llx (A_ELECTION_VOTE) %s", A_ELECTION_VOTE, text);
     }
-    if (is_set(action, A_ELECTION_CHECK)) {
+    if (pcmk_is_set(action, A_ELECTION_CHECK)) {
         crm_trace("Action %.16llx (A_ELECTION_CHECK) %s", A_ELECTION_CHECK, text);
     }
-    if (is_set(action, A_CL_JOIN_ANNOUNCE)) {
+    if (pcmk_is_set(action, A_CL_JOIN_ANNOUNCE)) {
         crm_trace("Action %.16llx (A_CL_JOIN_ANNOUNCE) %s", A_CL_JOIN_ANNOUNCE, text);
     }
-    if (is_set(action, A_CL_JOIN_REQUEST)) {
+    if (pcmk_is_set(action, A_CL_JOIN_REQUEST)) {
         crm_trace("Action %.16llx (A_CL_JOIN_REQUEST) %s", A_CL_JOIN_REQUEST, text);
     }
-    if (is_set(action, A_CL_JOIN_RESULT)) {
+    if (pcmk_is_set(action, A_CL_JOIN_RESULT)) {
         crm_trace("Action %.16llx (A_CL_JOIN_RESULT) %s", A_CL_JOIN_RESULT, text);
     }
-    if (is_set(action, A_DC_JOIN_OFFER_ALL)) {
+    if (pcmk_is_set(action, A_DC_JOIN_OFFER_ALL)) {
         crm_trace("Action %.16llx (A_DC_JOIN_OFFER_ALL) %s", A_DC_JOIN_OFFER_ALL, text);
     }
-    if (is_set(action, A_DC_JOIN_OFFER_ONE)) {
+    if (pcmk_is_set(action, A_DC_JOIN_OFFER_ONE)) {
         crm_trace("Action %.16llx (A_DC_JOIN_OFFER_ONE) %s", A_DC_JOIN_OFFER_ONE, text);
     }
-    if (is_set(action, A_DC_JOIN_PROCESS_REQ)) {
+    if (pcmk_is_set(action, A_DC_JOIN_PROCESS_REQ)) {
         crm_trace("Action %.16llx (A_DC_JOIN_PROCESS_REQ) %s", A_DC_JOIN_PROCESS_REQ, text);
     }
-    if (is_set(action, A_DC_JOIN_PROCESS_ACK)) {
+    if (pcmk_is_set(action, A_DC_JOIN_PROCESS_ACK)) {
         crm_trace("Action %.16llx (A_DC_JOIN_PROCESS_ACK) %s", A_DC_JOIN_PROCESS_ACK, text);
     }
-    if (is_set(action, A_DC_JOIN_FINALIZE)) {
+    if (pcmk_is_set(action, A_DC_JOIN_FINALIZE)) {
         crm_trace("Action %.16llx (A_DC_JOIN_FINALIZE) %s", A_DC_JOIN_FINALIZE, text);
     }
-    if (is_set(action, A_MSG_PROCESS)) {
+    if (pcmk_is_set(action, A_MSG_PROCESS)) {
         crm_trace("Action %.16llx (A_MSG_PROCESS) %s", A_MSG_PROCESS, text);
     }
-    if (is_set(action, A_MSG_ROUTE)) {
+    if (pcmk_is_set(action, A_MSG_ROUTE)) {
         crm_trace("Action %.16llx (A_MSG_ROUTE) %s", A_MSG_ROUTE, text);
     }
-    if (is_set(action, A_RECOVER)) {
+    if (pcmk_is_set(action, A_RECOVER)) {
         crm_trace("Action %.16llx (A_RECOVER) %s", A_RECOVER, text);
     }
-    if (is_set(action, A_DC_RELEASE)) {
+    if (pcmk_is_set(action, A_DC_RELEASE)) {
         crm_trace("Action %.16llx (A_DC_RELEASE) %s", A_DC_RELEASE, text);
     }
-    if (is_set(action, A_DC_RELEASED)) {
+    if (pcmk_is_set(action, A_DC_RELEASED)) {
         crm_trace("Action %.16llx (A_DC_RELEASED) %s", A_DC_RELEASED, text);
     }
-    if (is_set(action, A_DC_TAKEOVER)) {
+    if (pcmk_is_set(action, A_DC_TAKEOVER)) {
         crm_trace("Action %.16llx (A_DC_TAKEOVER) %s", A_DC_TAKEOVER, text);
     }
-    if (is_set(action, A_SHUTDOWN)) {
+    if (pcmk_is_set(action, A_SHUTDOWN)) {
         crm_trace("Action %.16llx (A_SHUTDOWN) %s", A_SHUTDOWN, text);
     }
-    if (is_set(action, A_SHUTDOWN_REQ)) {
+    if (pcmk_is_set(action, A_SHUTDOWN_REQ)) {
         crm_trace("Action %.16llx (A_SHUTDOWN_REQ) %s", A_SHUTDOWN_REQ, text);
     }
-    if (is_set(action, A_STOP)) {
+    if (pcmk_is_set(action, A_STOP)) {
         crm_trace("Action %.16llx (A_STOP  ) %s", A_STOP, text);
     }
-    if (is_set(action, A_EXIT_0)) {
+    if (pcmk_is_set(action, A_EXIT_0)) {
         crm_trace("Action %.16llx (A_EXIT_0) %s", A_EXIT_0, text);
     }
-    if (is_set(action, A_EXIT_1)) {
+    if (pcmk_is_set(action, A_EXIT_1)) {
         crm_trace("Action %.16llx (A_EXIT_1) %s", A_EXIT_1, text);
     }
-    if (is_set(action, A_CIB_START)) {
+    if (pcmk_is_set(action, A_CIB_START)) {
         crm_trace("Action %.16llx (A_CIB_START) %s", A_CIB_START, text);
     }
-    if (is_set(action, A_CIB_STOP)) {
+    if (pcmk_is_set(action, A_CIB_STOP)) {
         crm_trace("Action %.16llx (A_CIB_STOP) %s", A_CIB_STOP, text);
     }
-    if (is_set(action, A_TE_INVOKE)) {
+    if (pcmk_is_set(action, A_TE_INVOKE)) {
         crm_trace("Action %.16llx (A_TE_INVOKE) %s", A_TE_INVOKE, text);
     }
-    if (is_set(action, A_TE_START)) {
+    if (pcmk_is_set(action, A_TE_START)) {
         crm_trace("Action %.16llx (A_TE_START) %s", A_TE_START, text);
     }
-    if (is_set(action, A_TE_STOP)) {
+    if (pcmk_is_set(action, A_TE_STOP)) {
         crm_trace("Action %.16llx (A_TE_STOP) %s", A_TE_STOP, text);
     }
-    if (is_set(action, A_TE_CANCEL)) {
+    if (pcmk_is_set(action, A_TE_CANCEL)) {
         crm_trace("Action %.16llx (A_TE_CANCEL) %s", A_TE_CANCEL, text);
     }
-    if (is_set(action, A_PE_INVOKE)) {
+    if (pcmk_is_set(action, A_PE_INVOKE)) {
         crm_trace("Action %.16llx (A_PE_INVOKE) %s", A_PE_INVOKE, text);
     }
-    if (is_set(action, A_PE_START)) {
+    if (pcmk_is_set(action, A_PE_START)) {
         crm_trace("Action %.16llx (A_PE_START) %s", A_PE_START, text);
     }
-    if (is_set(action, A_PE_STOP)) {
+    if (pcmk_is_set(action, A_PE_STOP)) {
         crm_trace("Action %.16llx (A_PE_STOP) %s", A_PE_STOP, text);
     }
-    if (is_set(action, A_NODE_BLOCK)) {
+    if (pcmk_is_set(action, A_NODE_BLOCK)) {
         crm_trace("Action %.16llx (A_NODE_BLOCK) %s", A_NODE_BLOCK, text);
     }
-    if (is_set(action, A_UPDATE_NODESTATUS)) {
+    if (pcmk_is_set(action, A_UPDATE_NODESTATUS)) {
         crm_trace("Action %.16llx (A_UPDATE_NODESTATUS) %s", A_UPDATE_NODESTATUS, text);
     }
-    if (is_set(action, A_LOG)) {
+    if (pcmk_is_set(action, A_LOG)) {
         crm_trace("Action %.16llx (A_LOG   ) %s", A_LOG, text);
     }
-    if (is_set(action, A_ERROR)) {
+    if (pcmk_is_set(action, A_ERROR)) {
         crm_trace("Action %.16llx (A_ERROR ) %s", A_ERROR, text);
     }
-    if (is_set(action, A_WARN)) {
+    if (pcmk_is_set(action, A_WARN)) {
         crm_trace("Action %.16llx (A_WARN  ) %s", A_WARN, text);
     }
 }
@@ -702,10 +703,10 @@ update_dc(xmlNode * msg)
         CRM_CHECK(dc_version != NULL, return FALSE);
         CRM_CHECK(welcome_from != NULL, return FALSE);
 
-        if (AM_I_DC && safe_str_neq(welcome_from, fsa_our_uname)) {
+        if (AM_I_DC && !pcmk__str_eq(welcome_from, fsa_our_uname, pcmk__str_casei)) {
             invalid = TRUE;
 
-        } else if (fsa_our_dc && safe_str_neq(welcome_from, fsa_our_dc)) {
+        } else if (fsa_our_dc && !pcmk__str_eq(welcome_from, fsa_our_dc, pcmk__str_casei)) {
             invalid = TRUE;
         }
 
@@ -717,7 +718,8 @@ update_dc(xmlNode * msg)
                 crm_warn("New DC %s is not %s", welcome_from, fsa_our_dc);
             }
 
-            register_fsa_action(A_CL_JOIN_QUERY | A_DC_TIMER_START);
+            controld_set_fsa_action_flags(A_CL_JOIN_QUERY | A_DC_TIMER_START);
+            trigger_fsa();
             return FALSE;
         }
     }
@@ -734,14 +736,14 @@ update_dc(xmlNode * msg)
         fsa_our_dc_version = strdup(dc_version);
     }
 
-    if (safe_str_eq(fsa_our_dc, last_dc)) {
+    if (pcmk__str_eq(fsa_our_dc, last_dc, pcmk__str_casei)) {
         /* do nothing */
 
     } else if (fsa_our_dc != NULL) {
         crm_node_t *dc_node = crm_get_peer(0, fsa_our_dc);
 
         crm_info("Set DC to %s (%s)", crm_str(fsa_our_dc), crm_str(fsa_our_dc_version));
-        crm_update_peer_expected(__FUNCTION__, dc_node, CRMD_JOINSTATE_MEMBER);
+        crm_update_peer_expected(__func__, dc_node, CRMD_JOINSTATE_MEMBER);
 
     } else if (last_dc != NULL) {
         crm_info("Unset DC. Was %s", crm_str(last_dc));
@@ -751,42 +753,14 @@ update_dc(xmlNode * msg)
     return TRUE;
 }
 
-#define STATUS_PATH_MAX 512
-static void
-erase_xpath_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void *user_data)
-{
-    char *xpath = user_data;
-
-    do_crm_log_unlikely(rc == 0 ? LOG_DEBUG : LOG_NOTICE,
-                        "Deletion of \"%s\": %s (rc=%d)", xpath, pcmk_strerror(rc), rc);
-}
-
-#define XPATH_STATUS_TAG "//node_state[@uname='%s']/%s"
-
-void
-erase_status_tag(const char *uname, const char *tag, int options)
-{
-    if (fsa_cib_conn && uname) {
-        int call_id;
-        char *xpath = crm_strdup_printf(XPATH_STATUS_TAG, uname, tag);
-
-        crm_info("Deleting %s status entries for %s " CRM_XS " xpath=%s",
-                 tag, uname, xpath);
-        call_id = fsa_cib_conn->cmds->remove(fsa_cib_conn, xpath, NULL,
-                                             cib_quorum_override | cib_xpath | options);
-        fsa_register_cib_callback(call_id, FALSE, xpath, erase_xpath_callback);
-        // CIB library handles freeing xpath
-    }
-}
-
 void crmd_peer_down(crm_node_t *peer, bool full) 
 {
     if(full && peer->state == NULL) {
-        crm_update_peer_state(__FUNCTION__, peer, CRM_NODE_LOST, 0);
-        crm_update_peer_proc(__FUNCTION__, peer, crm_proc_none, NULL);
+        crm_update_peer_state(__func__, peer, CRM_NODE_LOST, 0);
+        crm_update_peer_proc(__func__, peer, crm_proc_none, NULL);
     }
-    crm_update_peer_join(__FUNCTION__, peer, crm_join_none);
-    crm_update_peer_expected(__FUNCTION__, peer, CRMD_JOINSTATE_DOWN);
+    crm_update_peer_join(__func__, peer, crm_join_none);
+    crm_update_peer_expected(__func__, peer, CRMD_JOINSTATE_DOWN);
 }
 
 #define MIN_CIB_OP_TIMEOUT (30)
@@ -892,7 +866,7 @@ get_node_id(xmlNode *lrm_rsc_op)
 {
     xmlNode *node = lrm_rsc_op;
 
-    while (node != NULL && safe_str_neq(XML_CIB_TAG_STATE, TYPE(node))) {
+    while (node != NULL && !pcmk__str_eq(XML_CIB_TAG_STATE, TYPE(node), pcmk__str_casei)) {
         node = node->parent;
     }
 
