@@ -1,5 +1,7 @@
 /*
- * Copyright 2009-2018 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2009-2020 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU General Public License version 2
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
@@ -14,12 +16,13 @@
 #include <crm/crm.h>
 #include <crm/msg_xml.h>
 #include <crm/common/ipc.h>
-#include <crm/common/ipcs.h>
+#include <crm/common/ipc_internal.h>
 #include <crm/cluster/internal.h>
 
 #include <crm/stonith-ng.h>
 #include <crm/fencing/internal.h>
 #include <crm/common/xml.h>
+#include <crm/common/xml_internal.h>
 
 #include <pacemaker-fenced.h>
 
@@ -40,7 +43,7 @@ stonith_send_broadcast_history(xmlNode *history,
                                const char *target)
 {
     xmlNode *bcast = create_xml_node(NULL, "stonith_command");
-    xmlNode *data = create_xml_node(NULL, __FUNCTION__);
+    xmlNode *data = create_xml_node(NULL, __func__);
 
     if (target) {
         crm_xml_add(data, F_STONITH_TARGET, target);
@@ -221,11 +224,12 @@ stonith_xml_history_to_list(xmlNode *history)
 
     CRM_LOG_ASSERT(rv != NULL);
 
-    for (xml_op = __xml_first_child(history); xml_op != NULL;
-         xml_op = __xml_next(xml_op)) {
+    for (xml_op = pcmk__xml_first_child(history); xml_op != NULL;
+         xml_op = pcmk__xml_next(xml_op)) {
         remote_fencing_op_t *op = NULL;
         char *id = crm_element_value_copy(xml_op, F_STONITH_REMOTE_OP_ID);
-        int completed, state;
+        int state;
+        long long completed;
 
         if (!id) {
             crm_warn("History to convert to hashtable has no id in entry");
@@ -242,7 +246,7 @@ stonith_xml_history_to_list(xmlNode *history)
         op->originator = crm_element_value_copy(xml_op, F_STONITH_ORIGIN);
         op->delegate = crm_element_value_copy(xml_op, F_STONITH_DELEGATE);
         op->client_name = crm_element_value_copy(xml_op, F_STONITH_CLIENTNAME);
-        crm_element_value_int(xml_op, F_STONITH_DATE, &completed);
+        crm_element_value_ll(xml_op, F_STONITH_DATE, &completed);
         op->completed = (time_t) completed;
         crm_element_value_int(xml_op, F_STONITH_STATE, &state);
         op->state = (enum op_state) state;
@@ -289,7 +293,7 @@ stonith_local_history_diff(GHashTable *remote_history,
                     continue; /* skip entries broadcasted already */
                 }
 
-                if (target && strcmp(op->target, target) != 0) {
+                if (!pcmk__str_eq(target, op->target, pcmk__str_null_matches)) {
                     continue;
                 }
 
@@ -304,7 +308,7 @@ stonith_local_history_diff(GHashTable *remote_history,
                 crm_xml_add(entry, F_STONITH_ORIGIN, op->originator);
                 crm_xml_add(entry, F_STONITH_DELEGATE, op->delegate);
                 crm_xml_add(entry, F_STONITH_CLIENTNAME, op->client_name);
-                crm_xml_add_int(entry, F_STONITH_DATE, op->completed);
+                crm_xml_add_ll(entry, F_STONITH_DATE, op->completed);
                 crm_xml_add_int(entry, F_STONITH_STATE, op->state);
             }
     }
@@ -350,7 +354,7 @@ stonith_merge_in_history_list(GHashTable *history)
 
         if ((op->state != st_failed) &&
             (op->state != st_done) &&
-            safe_str_eq(op->originator, stonith_our_uname)) {
+            pcmk__str_eq(op->originator, stonith_our_uname, pcmk__str_casei)) {
             crm_warn("received pending action we are supposed to be the "
                      "owner but it's not in our records -> fail it");
             op->state = st_failed;
@@ -359,7 +363,7 @@ stonith_merge_in_history_list(GHashTable *history)
                trigger unexpected results at other places and to prevent
                remote_op_done from setting the delegate if not present
              */
-            stonith_bcast_result_to_peers(op, -EHOSTUNREACH);
+            stonith_bcast_result_to_peers(op, -EHOSTUNREACH, FALSE);
         }
 
         g_hash_table_insert(stonith_remote_op_list, op->id, op);
@@ -398,7 +402,7 @@ stonith_fence_history(xmlNode *msg, xmlNode **output,
 {
     int rc = 0;
     const char *target = NULL;
-    xmlNode *dev = get_xpath_object("//@" F_STONITH_TARGET, msg, LOG_TRACE);
+    xmlNode *dev = get_xpath_object("//@" F_STONITH_TARGET, msg, LOG_NEVER);
     xmlNode *out_history = NULL;
 
     if (dev) {
@@ -438,9 +442,9 @@ stonith_fence_history(xmlNode *msg, xmlNode **output,
                                         st_opt_broadcast | st_opt_discard_reply,
                                         NULL);
         } else if (remote_peer &&
-                   !safe_str_eq(remote_peer, stonith_our_uname)) {
-            xmlNode *history =
-                get_xpath_object("//" F_STONITH_HISTORY_LIST, msg, LOG_TRACE);
+                   !pcmk__str_eq(remote_peer, stonith_our_uname, pcmk__str_casei)) {
+            xmlNode *history = get_xpath_object("//" F_STONITH_HISTORY_LIST,
+                                                msg, LOG_NEVER);
             GHashTable *received_history =
                 history?stonith_xml_history_to_list(history):NULL;
 
