@@ -1,10 +1,11 @@
 /*
- * Copyright 2009-2019 the Pacemaker project contributors
+ * Copyright 2009-2020 the Pacemaker project contributors
  *
  * This source code is licensed under the GNU General Public License version 2
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
  */
 
+#include <stdint.h>                 // uint32_t, uint64_t
 #include <crm/common/mainloop.h>
 
 /*!
@@ -19,13 +20,6 @@
  */
 gboolean stonith_check_fence_tolerance(int tolerance, const char *target, const char *action);
 
-enum st_device_flags
-{
-    st_device_supports_list   = 0x0001,
-    st_device_supports_status = 0x0002,
-    st_device_supports_reboot = 0x0004,
-};
-
 typedef struct stonith_device_s {
     char *id;
     char *agent;
@@ -42,7 +36,7 @@ typedef struct stonith_device_s {
     gboolean automatic_unfencing;
     guint priority;
 
-    enum st_device_flags flags;
+    uint32_t flags; // Group of enum st_device_flags
 
     GHashTable *params;
     GHashTable *aliases;
@@ -70,6 +64,14 @@ enum st_remap_phase {
     st_phase_max = 3
 };
 
+/* These values provide additional information for STONITH's asynchronous reply response.
+ * The st_reply_opt_merged value indicates an operation that has been merged and completed without being executed.
+ */
+enum st_replay_option {
+    st_reply_opt_none            = 0x00000000,
+    st_reply_opt_merged          = 0x00000001,
+};
+
 typedef struct remote_fencing_op_s {
     /* The unique id associated with this operation */
     char *id;
@@ -105,6 +107,10 @@ typedef struct remote_fencing_op_s {
      * values associated with the devices this fencing operation may call */
     gint total_timeout;
 
+    /*! Requested fencing delay.
+     * Value -1 means disable any static/random fencing delays. */
+    int delay;
+
     /*! Delegate is the node being asked to perform a fencing action
      * on behalf of the node that owns the remote operation. Some operations
      * will involve multiple delegates. This value represents the final delegate
@@ -112,8 +118,8 @@ typedef struct remote_fencing_op_s {
     char *delegate;
     /*! The point at which the remote operation completed */
     time_t completed;
-    /*! The stonith_call_options associated with this remote operation */
-    long long call_options;
+    //! Group of enum stonith_call_options associated with this operation
+    uint32_t call_options;
 
     /*! The current state of the remote operation. This indicates
      * what stage the op is in, query, exec, done, duplicate, failed. */
@@ -155,15 +161,16 @@ typedef struct remote_fencing_op_s {
  * \param op, Operation whose result should be broadcast
  * \param rc, Result of the operation
  */
-void stonith_bcast_result_to_peers(remote_fencing_op_t * op, int rc);
+void stonith_bcast_result_to_peers(remote_fencing_op_t * op, int rc, gboolean op_merged);
 
-enum st_callback_flags {
-    st_callback_unknown               = 0x0000,
-    st_callback_notify_fence          = 0x0001,
-    st_callback_device_add            = 0x0004,
-    st_callback_device_del            = 0x0010,
-    st_callback_notify_history        = 0x0020,
-    st_callback_notify_history_synced = 0x0040
+// Fencer-specific client flags
+enum st_client_flags {
+    st_callback_unknown               =  UINT64_C(0),
+    st_callback_notify_fence          = (UINT64_C(1) << 0),
+    st_callback_device_add            = (UINT64_C(1) << 2),
+    st_callback_device_del            = (UINT64_C(1) << 4),
+    st_callback_notify_history        = (UINT64_C(1) << 5),
+    st_callback_notify_history_synced = (UINT64_C(1) << 6)
 };
 
 /*
@@ -202,10 +209,10 @@ void free_stonith_remote_op_list(void);
 void init_stonith_remote_op_hash_table(GHashTable **table);
 void free_metadata_cache(void);
 
-long long get_stonith_flag(const char *name);
+uint64_t get_stonith_flag(const char *name);
 
-void stonith_command(crm_client_t * client, uint32_t id, uint32_t flags,
-                            xmlNode * op_request, const char *remote_peer);
+void stonith_command(pcmk__client_t *client, uint32_t id, uint32_t flags,
+                            xmlNode *op_request, const char *remote_peer);
 
 int stonith_device_register(xmlNode * msg, const char **desc, gboolean from_cib);
 
@@ -232,8 +239,9 @@ void do_stonith_notify(int options, const char *type, int result, xmlNode * data
 void do_stonith_notify_device(int options, const char *op, int rc, const char *desc);
 void do_stonith_notify_level(int options, const char *op, int rc, const char *desc);
 
-remote_fencing_op_t *initiate_remote_stonith_op(crm_client_t * client, xmlNode * request,
-                                                       gboolean manual_ack);
+remote_fencing_op_t *initiate_remote_stonith_op(pcmk__client_t *client,
+                                                xmlNode *request,
+                                                gboolean manual_ack);
 
 int process_remote_stonith_exec(xmlNode * msg);
 
