@@ -13,6 +13,7 @@
 #include <pwd.h>
 #include <grp.h>
 #include <poll.h>
+#include <errno.h>
 #include <stdio.h>
 #include <stdbool.h>
 #include <sys/stat.h>
@@ -24,32 +25,40 @@
 #include <crm/crm.h>  /* indirectly: CRM_EX_* */
 #include <crm/cib/internal.h>  /* cib_channel_ro */
 #include <crm/msg_xml.h>
-#include <crm/common/ipcs_internal.h>
+#include <crm/common/ipc_internal.h>
 #include <crm/common/mainloop.h>
 #include <crm/cluster/internal.h>
 #include <crm/cluster.h>
 
-#include <crm/common/ipc_internal.h>  /* PCMK__SPECIAL_PID*, ... */
-
-#ifdef SUPPORT_COROSYNC
-#include <corosync/cfg.h>
-#endif
-
 #include <dirent.h>
 #include <ctype.h>
 
-static gboolean pcmk_quorate = FALSE;
 static gboolean fatal_error = FALSE;
 static GMainLoop *mainloop = NULL;
 static bool global_keep_tracking = false;
 
 #define PCMK_PROCESS_CHECK_INTERVAL 5
 
-static const char *local_name = NULL;
-static uint32_t local_nodeid = 0;
 static crm_trigger_t *shutdown_trigger = NULL;
+static crm_trigger_t *startup_trigger = NULL;
 static const char *pid_file = PCMK_RUN_DIR "/pacemaker.pid";
 
+/* state we report when asked via pacemakerd-api status-ping */
+static const char *pacemakerd_state = XML_PING_ATTR_PACEMAKERDSTATE_INIT;
+static gboolean running_with_sbd = FALSE; /* local copy */
+/* When contacted via pacemakerd-api by a client having sbd in
+ * the name we assume it is sbd-daemon which wants to know
+ * if pacemakerd shutdown gracefully.
+ * Thus when everything is shutdown properly pacemakerd
+ * waits till it has reported the graceful completion of
+ * shutdown to sbd and just when sbd-client closes the
+ * connection we can assume that the report has arrived
+ * properly so that pacemakerd can finally exit.
+ * Following two variables are used to track that handshake.
+ */
+static unsigned int shutdown_complete_state_reported_to = 0;
+static gboolean shutdown_complete_state_reported_client_closed = FALSE;
+
 typedef struct pcmk_child_s {
     pid_t pid;
     long flag;
@@ -107,23 +116,6 @@ static pcmk_child_t pcmk_children[] = {
 static gboolean check_active_before_startup_processes(gpointer user_data);
 static int child_liveness(pcmk_child_t *child);
 static gboolean start_child(pcmk_child_t * child);
-static gboolean update_node_processes(uint32_t id, const char *uname,
-                                      uint32_t procs);
-void update_process_clients(pcmk__client_t *client);
-
-static uint32_t
-get_process_list(void)
-{
-    int lpc = 0;
-    uint32_t procs = crm_get_cluster_proc();
-
-    for (lpc = 0; lpc < SIZEOF(pcmk_children); lpc++) {
-        if (pcmk_children[lpc].pid != 0) {
-            procs |= pcmk_children[lpc].flag;
-        }
-    }
-    return procs;
-}
 
 static void
 pcmk_process_exit(pcmk_child_t * child)
@@ -131,16 +123,6 @@ pcmk_process_exit(pcmk_child_t * child)
     child->pid = 0;
     child->active_before_startup = FALSE;
 
-    /* Broadcast the fact that one of our processes died ASAP
-     *
-     * Try to get some logging of the cause out first though
-     * because we're probably about to get fenced
-     *
-     * Potentially do this only if respawn_count > N
-     * to allow for local recovery
-     */
-    update_node_processes(local_nodeid, NULL, get_process_list());
-
     child->respawn_count += 1;
     if (child->respawn_count > MAX_RESPAWN) {
         crm_err("Child respawn count exceeded by %s", child->name);
@@ -150,15 +132,13 @@ pcmk_process_exit(pcmk_child_t * child)
     if (shutdown_trigger) {
         /* resume step-wise shutdown (returned TRUE yields no parallelizing) */
         mainloop_set_trigger(shutdown_trigger);
-        /* intended to speed up propagating expected lay-off of the daemons? */
-        update_node_processes(local_nodeid, NULL, get_process_list());
 
     } else if (!child->respawn) {
         /* nothing to do */
 
     } else if (crm_is_true(getenv("PCMK_fail_fast"))) {
         crm_err("Rebooting system because of %s", child->name);
-        pcmk_panic(__FUNCTION__);
+        pcmk__panic(__func__);
 
     } else if (child_liveness(child) == pcmk_rc_ok) {
         crm_warn("One-off suppressing strict respawning of a child process %s,"
@@ -178,28 +158,6 @@ pcmk_process_exit(pcmk_child_t * child)
     }
 }
 
-static void pcmk_exit_with_cluster(int exitcode)
-{
-#ifdef SUPPORT_COROSYNC
-    corosync_cfg_handle_t cfg_handle;
-    cs_error_t err;
-
-    if (exitcode == CRM_EX_FATAL) {
-	    crm_info("Asking Corosync to shut down");
-	    err = corosync_cfg_initialize(&cfg_handle, NULL);
-	    if (err != CS_OK) {
-		    crm_warn("Unable to open handle to corosync to close it down. err=%d", err);
-	    }
-	    err = corosync_cfg_try_shutdown(cfg_handle, COROSYNC_CFG_SHUTDOWN_FLAG_IMMEDIATE);
-	    if (err != CS_OK) {
-		    crm_warn("Corosync shutdown failed. err=%d", err);
-	    }
-	    corosync_cfg_finalize(cfg_handle);
-    }
-#endif
-    crm_exit(exitcode);
-}
-
 static void
 pcmk_child_exit(mainloop_child_t * p, pid_t pid, int core, int signo, int exitcode)
 {
@@ -230,7 +188,7 @@ pcmk_child_exit(mainloop_child_t * p, pid_t pid, int core, int signo, int exitco
                 crm_emerg("%s[%d] instructed the machine to reset", name, pid);
                 child->respawn = FALSE;
                 fatal_error = TRUE;
-                pcmk_panic(__FUNCTION__);
+                pcmk__panic(__func__);
                 pcmk_shutdown(SIGTERM);
                 break;
 
@@ -343,7 +301,6 @@ start_child(pcmk_child_t * child)
         crm_info("Forked child %lld for process %s%s",
                  (long long) child->pid, child->name,
                  use_valgrind ? " (valgrind enabled: " VALGRIND_BIN ")" : "");
-        update_node_processes(local_nodeid, NULL, get_process_list());
         return TRUE;
 
     } else {
@@ -379,7 +336,7 @@ start_child(pcmk_child_t * child)
 
             // Drop root group access if not needed
             if (!need_root_group && (setgid(gid) < 0)) {
-                crm_perror(LOG_ERR, "Could not set group to %d", gid);
+                crm_warn("Could not set group to %d: %s", gid, strerror(errno));
             }
 
             /* Initialize supplementary groups to only those always granted to
@@ -391,7 +348,8 @@ start_child(pcmk_child_t * child)
         }
 
         if (uid && setuid(uid) < 0) {
-            crm_perror(LOG_ERR, "Could not set user to %d (%s)", uid, child->uid);
+            crm_warn("Could not set user to %s (id %d): %s",
+                     child->uid, uid, strerror(errno));
         }
 
         pcmk__close_fds_in_child(true);
@@ -405,7 +363,7 @@ start_child(pcmk_child_t * child)
         } else {
             (void)execvp(child->command, opts_default);
         }
-        crm_perror(LOG_ERR, "FATAL: Cannot exec %s", child->command);
+        crm_crit("Could not execute %s: %s", child->command, strerror(errno));
         crm_exit(CRM_EX_FATAL);
     }
     return TRUE;                /* never reached */
@@ -433,21 +391,20 @@ escalate_shutdown(gpointer data)
 static gboolean
 pcmk_shutdown_worker(gpointer user_data)
 {
-    static int phase = 0;
+    static int phase = SIZEOF(pcmk_children);
     static time_t next_log = 0;
-    static int max = SIZEOF(pcmk_children);
 
     int lpc = 0;
 
-    if (phase == 0) {
+    if (phase == SIZEOF(pcmk_children)) {
         crm_notice("Shutting down Pacemaker");
-        phase = max;
+        pacemakerd_state = XML_PING_ATTR_PACEMAKERDSTATE_SHUTTINGDOWN;
     }
 
     for (; phase > 0; phase--) {
         /* Don't stop anything with start_seq < 1 */
 
-        for (lpc = max - 1; lpc >= 0; lpc--) {
+        for (lpc = SIZEOF(pcmk_children) - 1; lpc >= 0; lpc--) {
             pcmk_child_t *child = &(pcmk_children[lpc]);
 
             if (phase != child->start_seq) {
@@ -494,8 +451,14 @@ pcmk_shutdown_worker(gpointer user_data)
         }
     }
 
-    /* send_cluster_id(); */
     crm_notice("Shutdown complete");
+    pacemakerd_state = XML_PING_ATTR_PACEMAKERDSTATE_SHUTDOWNCOMPLETE;
+    if (!fatal_error && running_with_sbd &&
+        pcmk__get_sbd_sync_resource_startup() &&
+        !shutdown_complete_state_reported_client_closed) {
+        crm_notice("Waiting for SBD to pick up shutdown-complete-state.");
+        return TRUE;
+    }
 
     {
         const char *delay = pcmk__env_option("shutdown_delay");
@@ -509,7 +472,10 @@ pcmk_shutdown_worker(gpointer user_data)
 
     if (fatal_error) {
         crm_notice("Shutting down and staying down after fatal error");
-        pcmk_exit_with_cluster(CRM_EX_FATAL);
+#ifdef SUPPORT_COROSYNC
+        pcmkd_shutdown_corosync();
+#endif
+        crm_exit(CRM_EX_FATAL);
     }
 
     return TRUE;
@@ -524,7 +490,7 @@ pcmk_ignore(int nsig)
 static void
 pcmk_sigquit(int nsig)
 {
-    pcmk_panic(__FUNCTION__);
+    pcmk__panic(__func__);
 }
 
 void
@@ -546,6 +512,51 @@ pcmk_ipc_accept(qb_ipcs_connection_t * c, uid_t uid, gid_t gid)
     return 0;
 }
 
+static void
+pcmk_handle_ping_request(pcmk__client_t *c, xmlNode *msg, uint32_t id)
+{
+    const char *value = NULL;
+    xmlNode *ping = NULL;
+    xmlNode *reply = NULL;
+    time_t pinged = time(NULL);
+    const char *from = crm_element_value(msg, F_CRM_SYS_FROM);
+
+    /* Pinged for status */
+    crm_trace("Pinged from %s.%s",
+              crm_str(crm_element_value(msg, F_CRM_ORIGIN)),
+              from?from:"unknown");
+    ping = create_xml_node(NULL, XML_CRM_TAG_PING);
+    value = crm_element_value(msg, F_CRM_SYS_TO);
+    crm_xml_add(ping, XML_PING_ATTR_SYSFROM, value);
+    crm_xml_add(ping, XML_PING_ATTR_PACEMAKERDSTATE, pacemakerd_state);
+    crm_xml_add_ll(ping, XML_ATTR_TSTAMP, (long long) pinged);
+    crm_xml_add(ping, XML_PING_ATTR_STATUS, "ok");
+    reply = create_reply(msg, ping);
+    free_xml(ping);
+    if (reply) {
+        if (pcmk__ipc_send_xml(c, id, reply, crm_ipc_server_event) !=
+                pcmk_rc_ok) {
+            crm_err("Failed sending ping-reply");
+        }
+        free_xml(reply);
+    } else {
+        crm_err("Failed building ping-reply");
+    }
+    /* just proceed state on sbd pinging us */
+    if (from && strstr(from, "sbd")) {
+        if (pcmk__str_eq(pacemakerd_state, XML_PING_ATTR_PACEMAKERDSTATE_SHUTDOWNCOMPLETE, pcmk__str_none)) {
+            if (pcmk__get_sbd_sync_resource_startup()) {
+                crm_notice("Shutdown-complete-state passed to SBD.");
+            }
+            shutdown_complete_state_reported_to = c->pid;
+        } else if (pcmk__str_eq(pacemakerd_state, XML_PING_ATTR_PACEMAKERDSTATE_WAITPING, pcmk__str_none)) {
+            crm_notice("Received startup-trigger from SBD.");
+            pacemakerd_state = XML_PING_ATTR_PACEMAKERDSTATE_STARTINGDAEMONS;
+            mainloop_set_trigger(startup_trigger);
+        }
+    }
+}
+
 /* Exit code means? */
 static int32_t
 pcmk_ipc_dispatch(qb_ipcs_connection_t * qbc, void *data, size_t size)
@@ -553,38 +564,55 @@ pcmk_ipc_dispatch(qb_ipcs_connection_t * qbc, void *data, size_t size)
     uint32_t id = 0;
     uint32_t flags = 0;
     const char *task = NULL;
+    xmlNode *msg = NULL;
     pcmk__client_t *c = pcmk__find_client(qbc);
-    xmlNode *msg = pcmk__client_data2xml(c, data, &id, &flags);
 
-    pcmk__ipc_send_ack(c, id, flags, "ack");
+    CRM_CHECK(c != NULL, return 0);
+
+    msg = pcmk__client_data2xml(c, data, &id, &flags);
     if (msg == NULL) {
+        pcmk__ipc_send_ack(c, id, flags, "ack", CRM_EX_PROTOCOL);
         return 0;
     }
 
     task = crm_element_value(msg, F_CRM_TASK);
-    if (crm_str_eq(task, CRM_OP_QUIT, TRUE)) {
-        /* Time to quit */
-        crm_notice("Shutting down in response to ticket %s (%s)",
-                   crm_element_value(msg, F_CRM_REFERENCE), crm_element_value(msg, F_CRM_ORIGIN));
-        pcmk_shutdown(15);
+    if (pcmk__str_eq(task, CRM_OP_QUIT, pcmk__str_none)) {
+#if ENABLE_ACL
+        /* Only allow privileged users (i.e. root or hacluster)
+         * to shut down Pacemaker from the command line (or direct IPC).
+         *
+         * We only check when ACLs are enabled, because without them, any client
+         * with IPC access could shut down Pacemaker via the CIB anyway.
+         */
+        bool allowed = pcmk_is_set(c->flags, pcmk__client_privileged);
+#else
+        bool allowed = true;
+#endif
+        if (allowed) {
+            crm_notice("Shutting down in response to IPC request %s from %s",
+                       crm_element_value(msg, F_CRM_REFERENCE),
+                       crm_element_value(msg, F_CRM_ORIGIN));
+            pcmk__ipc_send_ack(c, id, flags, "ack", CRM_EX_OK);
+            pcmk_shutdown(15);
+        } else {
+            crm_warn("Ignoring shutdown request from unprivileged client %s",
+                     pcmk__client_name(c));
+            pcmk__ipc_send_ack(c, id, flags, "ack", CRM_EX_INSUFFICIENT_PRIV);
+        }
 
-    } else if (crm_str_eq(task, CRM_OP_RM_NODE_CACHE, TRUE)) {
-        /* Send to everyone */
-        struct iovec *iov;
-        int id = 0;
-        const char *name = NULL;
+    } else if (pcmk__str_eq(task, CRM_OP_RM_NODE_CACHE, pcmk__str_none)) {
+        crm_trace("Ignoring IPC request to purge node "
+                  "because peer cache is not used");
+        pcmk__ipc_send_ack(c, id, flags, "ack", CRM_EX_OK);
 
-        crm_element_value_int(msg, XML_ATTR_ID, &id);
-        name = crm_element_value(msg, XML_ATTR_UNAME);
-        crm_notice("Instructing peers to remove references to node %s/%u", name, id);
-
-        iov = calloc(1, sizeof(struct iovec));
-        iov->iov_base = dump_xml_unformatted(msg);
-        iov->iov_len = 1 + strlen(iov->iov_base);
-        send_cpg_iov(iov);
+    } else if (pcmk__str_eq(task, CRM_OP_PING, pcmk__str_none)) {
+        pcmk__ipc_send_ack(c, id, flags, "ack", CRM_EX_INDETERMINATE);
+        pcmk_handle_ping_request(c, msg, id);
 
     } else {
-        update_process_clients(c);
+        crm_debug("Unrecognized IPC command '%s' sent to pacemakerd",
+                  crm_str(task));
+        pcmk__ipc_send_ack(c, id, flags, "ack", CRM_EX_INVALID_PARAM);
     }
 
     free_xml(msg);
@@ -601,6 +629,12 @@ pcmk_ipc_closed(qb_ipcs_connection_t * c)
         return 0;
     }
     crm_trace("Connection %p", c);
+    if (shutdown_complete_state_reported_to == client->pid) {
+        shutdown_complete_state_reported_client_closed = TRUE;
+        if (shutdown_trigger) {
+            mainloop_set_trigger(shutdown_trigger);
+        }
+    }
     pcmk__free_client(client);
     return 0;
 }
@@ -620,113 +654,6 @@ struct qb_ipcs_service_handlers mcp_ipc_callbacks = {
     .connection_destroyed = pcmk_ipc_destroy
 };
 
-static void
-send_xml_to_client(gpointer key, gpointer value, gpointer user_data)
-{
-    pcmk__ipc_send_xml((pcmk__client_t *) value, 0, (xmlNode *) user_data,
-                       crm_ipc_server_event);
-}
-
-/*!
- * \internal
- * \brief Send an XML message with process list of all known peers to client(s)
- *
- * \param[in] client  Send message to this client, or all clients if NULL
- */
-void
-update_process_clients(pcmk__client_t *client)
-{
-    GHashTableIter iter;
-    crm_node_t *node = NULL;
-    xmlNode *update = create_xml_node(NULL, "nodes");
-
-    if (is_corosync_cluster()) {
-        crm_xml_add_int(update, "quorate", pcmk_quorate);
-    }
-
-    g_hash_table_iter_init(&iter, crm_peer_cache);
-    while (g_hash_table_iter_next(&iter, NULL, (gpointer *) & node)) {
-        xmlNode *xml = create_xml_node(update, "node");
-
-        crm_xml_add_int(xml, "id", node->id);
-        crm_xml_add(xml, "uname", node->uname);
-        crm_xml_add(xml, "state", node->state);
-        crm_xml_add_int(xml, "processes", node->processes);
-    }
-
-    if(client) {
-        crm_trace("Sending process list to client %s", client->id);
-        send_xml_to_client(NULL, client, update);
-
-    } else {
-        crm_trace("Sending process list to %d clients",
-                  pcmk__ipc_client_count());
-        pcmk__foreach_ipc_client(send_xml_to_client, update);
-    }
-    free_xml(update);
-}
-
-/*!
- * \internal
- * \brief Send a CPG message with local node's process list to all peers
- */
-static void
-update_process_peers(void)
-{
-    /* Do nothing for corosync-2 based clusters */
-
-    struct iovec *iov = calloc(1, sizeof(struct iovec));
-
-    CRM_ASSERT(iov);
-    if (local_name) {
-        iov->iov_base = crm_strdup_printf("<node uname=\"%s\" proclist=\"%u\"/>",
-                                          local_name, get_process_list());
-    } else {
-        iov->iov_base = crm_strdup_printf("<node proclist=\"%u\"/>",
-                                          get_process_list());
-    }
-    iov->iov_len = strlen(iov->iov_base) + 1;
-    crm_trace("Sending %s", (char*) iov->iov_base);
-    send_cpg_iov(iov);
-}
-
-/*!
- * \internal
- * \brief Update a node's process list, notifying clients and peers if needed
- *
- * \param[in] id     Node ID of affected node
- * \param[in] uname  Uname of affected node
- * \param[in] procs  Affected node's process list mask
- *
- * \return TRUE if the process list changed, FALSE otherwise
- */
-static gboolean
-update_node_processes(uint32_t id, const char *uname, uint32_t procs)
-{
-    gboolean changed = FALSE;
-    crm_node_t *node = crm_get_peer(id, uname);
-
-    if (procs != 0) {
-        if (procs != node->processes) {
-            crm_debug("Node %s now has process list: %.32x (was %.32x)",
-                      node->uname, procs, node->processes);
-            node->processes = procs;
-            changed = TRUE;
-
-            /* If local node's processes have changed, notify clients/peers */
-            if (id == local_nodeid) {
-                update_process_clients(NULL);
-                update_process_peers();
-            }
-
-        } else {
-            crm_trace("Node %s still has process list: %.32x", node->uname, procs);
-        }
-    }
-    return changed;
-}
-
-
 static pcmk__cli_option_t long_options[] = {
     // long option, argument type, storage, short option, description, flags
     {
@@ -1099,8 +1026,8 @@ find_and_track_existing_processes(void)
     return pcmk_rc_ok;
 }
 
-static void
-init_children_processes(void)
+static gboolean
+init_children_processes(void *user_data)
 {
     int start_seq = 1, lpc = 0;
     static int max = SIZEOF(pcmk_children);
@@ -1126,97 +1053,94 @@ init_children_processes(void)
      * This may be useful for the daemons to know
      */
     setenv("PCMK_respawned", "true", 1);
-}
-
-static void
-mcp_cpg_destroy(gpointer user_data)
-{
-    crm_crit("Lost connection to cluster layer, shutting down");
-    crm_exit(CRM_EX_DISCONNECT);
-}
-
-/*!
- * \internal
- * \brief Process a CPG message (process list or manual peer cache removal)
- *
- * \param[in] handle     CPG connection (ignored)
- * \param[in] groupName  CPG group name (ignored)
- * \param[in] nodeid     ID of affected node
- * \param[in] pid        Process ID (ignored)
- * \param[in] msg        CPG XML message
- * \param[in] msg_len    Length of msg in bytes (ignored)
- */
-static void
-mcp_cpg_deliver(cpg_handle_t handle,
-                 const struct cpg_name *groupName,
-                 uint32_t nodeid, uint32_t pid, void *msg, size_t msg_len)
-{
-    xmlNode *xml = string2xml(msg);
-    const char *task = crm_element_value(xml, F_CRM_TASK);
-
-    crm_trace("Received CPG message (%s): %.200s",
-              (task? task : "process list"), (char*)msg);
-
-    if (task == NULL) {
-        if (nodeid == local_nodeid) {
-            crm_debug("Ignoring message with local node's process list");
-        } else {
-            uint32_t procs = 0;
-            const char *uname = crm_element_value(xml, "uname");
-
-            crm_element_value_int(xml, "proclist", (int *)&procs);
-            if (update_node_processes(nodeid, uname, procs)) {
-                update_process_clients(NULL);
-            }
-        }
-
-    } else if (crm_str_eq(task, CRM_OP_RM_NODE_CACHE, TRUE)) {
-        int id = 0;
-        const char *name = NULL;
-
-        crm_element_value_int(xml, XML_ATTR_ID, &id);
-        name = crm_element_value(xml, XML_ATTR_UNAME);
-        reap_crm_member(id, name);
-    }
-
-    if (xml != NULL) {
-        free_xml(xml);
-    }
-}
-
-static void
-mcp_cpg_membership(cpg_handle_t handle,
-                    const struct cpg_name *groupName,
-                    const struct cpg_address *member_list, size_t member_list_entries,
-                    const struct cpg_address *left_list, size_t left_list_entries,
-                    const struct cpg_address *joined_list, size_t joined_list_entries)
-{
-    /* Update peer cache if needed */
-    pcmk_cpg_membership(handle, groupName, member_list, member_list_entries,
-                        left_list, left_list_entries,
-                        joined_list, joined_list_entries);
-
-    /* Always broadcast our own presence after any membership change */
-    update_process_peers();
-}
-
-static gboolean
-mcp_quorum_callback(unsigned long long seq, gboolean quorate)
-{
-    pcmk_quorate = quorate;
+    pacemakerd_state = XML_PING_ATTR_PACEMAKERDSTATE_RUNNING;
     return TRUE;
 }
 
 static void
-mcp_quorum_destroy(gpointer user_data)
+remove_core_file_limit(void)
 {
-    crm_info("connection lost");
+    struct rlimit cores;
+    int rc = getrlimit(RLIMIT_CORE, &cores);
+
+    if (rc < 0) {
+        crm_warn("Cannot determine current maximum core file size: %s",
+                 strerror(errno));
+        return;
+    }
+
+    if ((cores.rlim_max == 0) && (geteuid() == 0)) {
+        cores.rlim_max = RLIM_INFINITY;
+    } else {
+        crm_info("Maximum core file size is %llu bytes",
+                 (unsigned long long) cores.rlim_max);
+    }
+    cores.rlim_cur = cores.rlim_max;
+
+    rc = setrlimit(RLIMIT_CORE, &cores);
+    if (rc < 0) {
+        crm_warn("Cannot raise system limit on core file size "
+                 "(consider doing so manually)");
+    }
+}
+
+static crm_exit_t
+request_shutdown(crm_ipc_t *ipc)
+{
+    xmlNode *request = NULL;
+    xmlNode *reply = NULL;
+    int rc = 0;
+    crm_exit_t status = CRM_EX_OK;
+
+    request = create_request(CRM_OP_QUIT, NULL, NULL, CRM_SYSTEM_MCP,
+                             CRM_SYSTEM_MCP, NULL);
+    if (request == NULL) {
+        crm_err("Unable to create shutdown request"); // Probably memory error
+        status = CRM_EX_TEMPFAIL;
+        goto done;
+    }
+
+    crm_notice("Requesting shutdown of existing Pacemaker instance");
+    rc = crm_ipc_send(ipc, request, crm_ipc_client_response, 0, &reply);
+    if (rc < 0) {
+        crm_err("Could not send shutdown request");
+        status = crm_errno2exit(rc);
+        goto done;
+    }
+
+    if ((rc == 0) || (reply == NULL)) {
+        crm_err("Unrecognized response to shutdown request");
+        status = CRM_EX_PROTOCOL;
+        goto done;
+    }
+
+    if ((crm_element_value_int(reply, "status", &rc) == 0)
+        && (rc != CRM_EX_OK)) {
+        crm_err("Shutdown request failed: %s", crm_exit_str(rc));
+        status = rc;
+        goto done;
+    }
+
+    // Wait for pacemakerd to shut down IPC (with 30-minute timeout)
+    status = CRM_EX_TIMEOUT;
+    for (int i = 0; i < 900; ++i) {
+        if (!crm_ipc_connected(ipc)) {
+            status = CRM_EX_OK;
+            break;
+        }
+        sleep(2);
+    }
+
+done:
+    free_xml(request);
+    crm_ipc_close(ipc);
+    crm_ipc_destroy(ipc);
+    return status;
 }
 
 int
 main(int argc, char **argv)
 {
-    int rc;
     int flag;
     int argerr = 0;
 
@@ -1225,10 +1149,8 @@ main(int argc, char **argv)
 
     uid_t pcmk_uid = 0;
     gid_t pcmk_gid = 0;
-    struct rlimit cores;
     crm_ipc_t *old_instance = NULL;
     qb_ipcs_service_t *ipcs = NULL;
-    static crm_cluster_t cluster;
 
     crm_log_preinit(NULL, argc, argv);
     pcmk__set_cli_options(NULL, "[options]", long_options,
@@ -1295,20 +1217,7 @@ main(int argc, char **argv)
     (void) crm_ipc_connect(old_instance);
 
     if (shutdown) {
-        crm_debug("Shutting down existing Pacemaker instance by request");
-        while (crm_ipc_connected(old_instance)) {
-            xmlNode *cmd =
-                create_request(CRM_OP_QUIT, NULL, NULL, CRM_SYSTEM_MCP, CRM_SYSTEM_MCP, NULL);
-
-            crm_debug(".");
-            crm_ipc_send(old_instance, cmd, 0, 0, NULL);
-            free_xml(cmd);
-
-            sleep(2);
-        }
-        crm_ipc_close(old_instance);
-        crm_ipc_destroy(old_instance);
-        crm_exit(CRM_EX_OK);
+        crm_exit(request_shutdown(old_instance));
 
     } else if (crm_ipc_connected(old_instance)) {
         crm_ipc_close(old_instance);
@@ -1320,16 +1229,17 @@ main(int argc, char **argv)
     crm_ipc_close(old_instance);
     crm_ipc_destroy(old_instance);
 
+#ifdef SUPPORT_COROSYNC
     if (mcp_read_config() == FALSE) {
-        crm_notice("Could not obtain corosync config data, exiting");
         crm_exit(CRM_EX_UNAVAILABLE);
     }
+#endif
 
     // OCF shell functions and cluster-glue need facility under different name
     {
         const char *facility = pcmk__env_option("logfacility");
 
-        if (facility && safe_str_neq(facility, "none")) {
+        if (facility && !pcmk__str_eq(facility, "none", pcmk__str_casei)) {
             setenv("HA_LOGFACILITY", facility, 1);
         }
     }
@@ -1338,25 +1248,7 @@ main(int argc, char **argv)
                PACEMAKER_VERSION, BUILD_VERSION, CRM_FEATURES);
     mainloop = g_main_loop_new(NULL, FALSE);
 
-    rc = getrlimit(RLIMIT_CORE, &cores);
-    if (rc < 0) {
-        crm_perror(LOG_ERR, "Cannot determine current maximum core size.");
-    } else {
-        if (cores.rlim_max == 0 && geteuid() == 0) {
-            cores.rlim_max = RLIM_INFINITY;
-        } else {
-            crm_info("Maximum core file size is: %lu", (unsigned long)cores.rlim_max);
-        }
-        cores.rlim_cur = cores.rlim_max;
-
-        rc = setrlimit(RLIMIT_CORE, &cores);
-        if (rc < 0) {
-            crm_perror(LOG_ERR,
-                       "Core file generation will remain disabled."
-                       " Core files are an important diagnostic tool, so"
-                       " please consider enabling them by default.");
-        }
-    }
+    remove_core_file_limit();
 
     if (pcmk_daemon_user(&pcmk_uid, &pcmk_gid) < 0) {
         crm_err("Cluster user %s does not exist, aborting Pacemaker startup", CRM_DAEMON_USER);
@@ -1398,14 +1290,16 @@ main(int argc, char **argv)
         crm_exit(CRM_EX_OSERR);
     }
 
+#ifdef SUPPORT_COROSYNC
     /* Allows us to block shutdown */
-    if (cluster_connect_cfg(&local_nodeid) == FALSE) {
-        crm_err("Couldn't connect to Corosync's CFG service");
+    if (!cluster_connect_cfg()) {
         crm_exit(CRM_EX_PROTOCOL);
     }
+#endif
 
-    if(pcmk_locate_sbd() > 0) {
+    if (pcmk__locate_sbd() > 0) {
         setenv("PCMK_watchdog", "true", 1);
+        running_with_sbd = TRUE;
     } else {
         setenv("PCMK_watchdog", "false", 1);
     }
@@ -1419,35 +1313,26 @@ main(int argc, char **argv)
             crm_exit(CRM_EX_FATAL);
     };
 
-    cluster.destroy = mcp_cpg_destroy;
-    cluster.cpg.cpg_deliver_fn = mcp_cpg_deliver;
-    cluster.cpg.cpg_confchg_fn = mcp_cpg_membership;
-
-    crm_set_autoreap(FALSE);
-
-    rc = pcmk_ok;
-
-    if (cluster_connect_cpg(&cluster) == FALSE) {
-        crm_err("Couldn't connect to Corosync's CPG service");
-        rc = -ENOPROTOOPT;
-
-    } else if (cluster_connect_quorum(mcp_quorum_callback, mcp_quorum_destroy)
-               == FALSE) {
-        rc = -ENOTCONN;
+    mainloop_add_signal(SIGTERM, pcmk_shutdown);
+    mainloop_add_signal(SIGINT, pcmk_shutdown);
 
+    if ((running_with_sbd) && pcmk__get_sbd_sync_resource_startup()) {
+        crm_notice("Waiting for startup-trigger from SBD.");
+        pacemakerd_state = XML_PING_ATTR_PACEMAKERDSTATE_WAITPING;
+        startup_trigger = mainloop_add_trigger(G_PRIORITY_HIGH, init_children_processes, NULL);
     } else {
-        local_name = get_local_node_name();
-        update_node_processes(local_nodeid, local_name, get_process_list());
-
-        mainloop_add_signal(SIGTERM, pcmk_shutdown);
-        mainloop_add_signal(SIGINT, pcmk_shutdown);
-
-        init_children_processes();
-
-        crm_notice("Pacemaker daemon successfully started and accepting connections");
-        g_main_loop_run(mainloop);
+        if (running_with_sbd) {
+            crm_warn("Enabling SBD_SYNC_RESOURCE_STARTUP would (if supported "
+                     "by your SBD version) improve reliability of "
+                     "interworking between SBD & pacemaker.");
+        }
+        pacemakerd_state = XML_PING_ATTR_PACEMAKERDSTATE_STARTINGDAEMONS;
+        init_children_processes(NULL);
     }
 
+    crm_notice("Pacemaker daemon successfully started and accepting connections");
+    g_main_loop_run(mainloop);
+
     if (ipcs) {
         crm_trace("Closing IPC server");
         mainloop_del_ipc_server(ipcs);
@@ -1455,9 +1340,8 @@ main(int argc, char **argv)
     }
 
     g_main_loop_unref(mainloop);
-
-    cluster_disconnect_cpg(&cluster);
+#ifdef SUPPORT_COROSYNC
     cluster_disconnect_cfg();
-
-    crm_exit(crm_errno2exit(rc));
+#endif
+    crm_exit(CRM_EX_OK);
 }
