@@ -1,5 +1,7 @@
 /*
- * Copyright 2009-2018 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2009-2020 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -18,6 +20,7 @@ extern "C" {
  * \ingroup core
  */
 
+#  include <signal.h> // sighandler_t
 #  include <glib.h>
 
 enum mainloop_child_flags {
@@ -41,7 +44,11 @@ void mainloop_trigger_complete(crm_trigger_t * trig);
 
 gboolean mainloop_destroy_trigger(crm_trigger_t * source);
 
-gboolean crm_signal(int sig, void (*dispatch) (int sig));
+#  ifndef HAVE_SIGHANDLER_T
+typedef void (*sighandler_t)(int);
+#  endif
+
+sighandler_t crm_signal_handler(int sig, sighandler_t dispatch);
 
 gboolean mainloop_add_signal(int sig, void (*dispatch) (int sig));
 
@@ -71,6 +78,30 @@ struct ipc_client_callbacks {
 qb_ipcs_service_t *mainloop_add_ipc_server(const char *name, enum qb_ipc_type type,
                                            struct qb_ipcs_service_handlers *callbacks);
 
+/*!
+ * \brief Start server-side API end-point, hooked into the internal event loop
+ *
+ * \param[in] name    name of the IPC end-point ("address" for the client)
+ * \param[in] type    selects libqb's IPC back-end (or use #QB_IPC_NATIVE)
+ * \param[in] callbacks  defines libqb's IPC service-level handlers
+ * \param[in] priority  priority relative to other events handled in the
+ *                      abstract handling loop, use #QB_LOOP_MED when unsure
+ *
+ * \return libqb's opaque handle to the created service abstraction
+ *
+ * \note For portability concerns, do not use this function if you keep
+ *       \p priority as #QB_LOOP_MED, stick with #mainloop_add_ipc_server
+ *       (with exactly such semantics) instead (once you link with this new
+ *       symbol employed, you can't downgrade the library freely anymore).
+ *
+ * \note The intended effect will only get fully reflected when run-time
+ *       linked to patched libqb: https://github.com/ClusterLabs/libqb/pull/352
+ */
+qb_ipcs_service_t *mainloop_add_ipc_server_with_prio(const char *name,
+                                                    enum qb_ipc_type type,
+                                                    struct qb_ipcs_service_handlers *callbacks,
+                                                    enum qb_loop_priority prio);
+
 void mainloop_del_ipc_server(qb_ipcs_service_t * server);
 
 mainloop_io_t *mainloop_add_ipc_client(const char *name, int priority, size_t max_size,
@@ -115,11 +146,19 @@ pid_t mainloop_child_pid(mainloop_child_t * child);
 void mainloop_clear_child_userdata(mainloop_child_t * child);
 gboolean mainloop_child_kill(pid_t pid);
 
+void pcmk_quit_main_loop(GMainLoop *mloop, unsigned int n);
 void pcmk_drain_main_loop(GMainLoop *mloop, guint timer_ms,
                           bool (*check)(guint));
 
 #  define G_PRIORITY_MEDIUM (G_PRIORITY_HIGH/2)
 
+#ifndef PCMK__NO_COMPAT
+/* Everything here is deprecated and kept only for public API backward
+ * compatibility. It will be moved to compatibility.h in a future release.
+ */
+gboolean crm_signal(int sig, void (*dispatch) (int sig)); // deprecated
+#endif
+
 #ifdef __cplusplus
 }
 #endif
