@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -31,6 +31,7 @@ extern "C" {
 #  include <libxml/tree.h>
 
 #  include <crm/lrmd.h>
+#  include <crm/common/acl.h>
 #  include <crm/common/results.h>
 
 #  define ONLINESTATUS  "online"  // Status of an online client
@@ -50,15 +51,14 @@ gboolean crm_is_true(const char *s);
 int crm_str_to_boolean(const char *s, int *ret);
 long long crm_parse_ll(const char *text, const char *default_text);
 int crm_parse_int(const char *text, const char *default_text);
+long long crm_get_msec(const char *input);
 char * crm_strip_trailing_newline(char *str);
-gboolean crm_str_eq(const char *a, const char *b, gboolean use_case);
-gboolean safe_str_neq(const char *a, const char *b);
 gboolean crm_strcase_equal(gconstpointer a, gconstpointer b);
 guint crm_strcase_hash(gconstpointer v);
 guint g_str_hash_traditional(gconstpointer v);
 char *crm_strdup_printf(char const *format, ...) __attribute__ ((__format__ (__printf__, 1, 2)));
+int pcmk_numeric_strcasecmp(const char *s1, const char *s2);
 
-#  define safe_str_eq(a, b) crm_str_eq(a, b, FALSE)
 #  define crm_str_hash g_str_hash_traditional
 
 static inline char *
@@ -87,7 +87,7 @@ crm_ttoa(time_t epoch_time)
  *       g_hash_table_destroy().
  */
 static inline GHashTable *
-crm_str_table_new()
+crm_str_table_new(void)
 {
     return g_hash_table_new_full(crm_str_hash, g_str_equal, free, free);
 }
@@ -100,7 +100,7 @@ crm_str_table_new()
  *       g_hash_table_destroy().
  */
 static inline GHashTable *
-crm_strcase_table_new()
+crm_strcase_table_new(void)
 {
     return g_hash_table_new_full(crm_strcase_hash, crm_strcase_equal, free, free);
 }
@@ -112,20 +112,16 @@ GHashTable *crm_str_table_dup(GHashTable *old_table);
 /* public I/O functions (from io.c) */
 void crm_build_path(const char *path_c, mode_t mode);
 
-long long crm_get_msec(const char *input);
 guint crm_parse_interval_spec(const char *input);
 int char2score(const char *score);
 char *score2char(int score);
 char *score2char_stack(int score, char *buf, size_t len);
 
-// deprecated
-#define crm_get_interval crm_parse_interval_spec
-
 /* public operation functions (from operations.c) */
 gboolean parse_op_key(const char *key, char **rsc_id, char **op_type,
                       guint *interval_ms);
-gboolean decode_transition_key(const char *key, char **uuid, int *action,
-                               int *transition_id, int *target_rc);
+gboolean decode_transition_key(const char *key, char **uuid, int *transition_id,
+                               int *action_id, int *target_rc);
 gboolean decode_transition_magic(const char *magic, char **uuid,
                                  int *transition_id, int *action_id,
                                  int *op_status, int *op_rc, int *target_rc);
@@ -141,12 +137,14 @@ xmlNode *crm_create_op_xml(xmlNode *parent, const char *prefix,
 
 // Capabilities supported by a resource agent standard
 enum pcmk_ra_caps {
-    pcmk_ra_cap_none         = 0x000,
-    pcmk_ra_cap_provider     = 0x001, // Requires provider
-    pcmk_ra_cap_status       = 0x002, // Supports status instead of monitor
-    pcmk_ra_cap_params       = 0x004, // Supports parameters
-    pcmk_ra_cap_unique       = 0x008, // Supports unique clones
-    pcmk_ra_cap_promotable   = 0x010, // Supports promotable clones
+    pcmk_ra_cap_none         = 0,
+    pcmk_ra_cap_provider     = (1 << 0), // Requires provider
+    pcmk_ra_cap_status       = (1 << 1), // Supports status instead of monitor
+    pcmk_ra_cap_params       = (1 << 2), // Supports parameters
+    pcmk_ra_cap_unique       = (1 << 3), // Supports unique clones
+    pcmk_ra_cap_promotable   = (1 << 4), // Supports promotable clones
+    pcmk_ra_cap_stdin        = (1 << 5), // Reads from standard input
+    pcmk_ra_cap_fence_params = (1 << 6), // Supports pcmk_monitor_timeout, etc.
 };
 
 uint32_t pcmk_get_ra_caps(const char *standard);
@@ -154,7 +152,6 @@ char *crm_generate_ra_key(const char *standard, const char *provider,
                           const char *type);
 int crm_parse_agent_spec(const char *spec, char **standard, char **provider,
                          char **type);
-bool crm_provider_required(const char *standard); // deprecated
 
 
 int compare_version(const char *version1, const char *version2);
@@ -163,23 +160,40 @@ int compare_version(const char *version1, const char *version2);
 void crm_abort(const char *file, const char *function, int line,
                const char *condition, gboolean do_core, gboolean do_fork);
 
-static inline gboolean
-is_not_set(long long word, long long bit)
+/*!
+ * \brief Check whether any of specified flags are set in a flag group
+ *
+ * \param[in] flag_group        The flag group being examined
+ * \param[in] flags_to_check    Which flags in flag_group should be checked
+ *
+ * \return true if \p flags_to_check is nonzero and any of its flags are set in
+ *         \p flag_group, or false otherwise
+ */
+static inline bool
+pcmk_any_flags_set(uint64_t flag_group, uint64_t flags_to_check)
 {
-    return ((word & bit) == 0);
+    return (flag_group & flags_to_check) != 0;
 }
 
-static inline gboolean
-is_set(long long word, long long bit)
+/*!
+ * \brief Check whether all of specified flags are set in a flag group
+ *
+ * \param[in] flag_group        The flag group being examined
+ * \param[in] flags_to_check    Which flags in flag_group should be checked
+ *
+ * \return true if \p flags_to_check is zero or all of its flags are set in
+ *         \p flag_group, or false otherwise
+ */
+static inline bool
+pcmk_all_flags_set(uint64_t flag_group, uint64_t flags_to_check)
 {
-    return ((word & bit) == bit);
+    return (flag_group & flags_to_check) == flags_to_check;
 }
 
-static inline gboolean
-is_set_any(long long word, long long bit)
-{
-    return ((word & bit) != 0);
-}
+/*!
+ * \brief Convenience alias for pcmk_all_flags_set(), to check single flag
+ */
+#define pcmk_is_set(g, f)   pcmk_all_flags_set((g), (f))
 
 static inline guint
 crm_hash_table_size(GHashTable * hashtable)
@@ -196,18 +210,65 @@ const char *crm_meta_value(GHashTable * hash, const char *field);
 char *crm_md5sum(const char *buffer);
 
 char *crm_generate_uuid(void);
+
+// This belongs in ipc.h but is here for backward compatibility
 bool crm_is_daemon_name(const char *name);
 
 int crm_user_lookup(const char *name, uid_t * uid, gid_t * gid);
+int pcmk_daemon_user(uid_t *uid, gid_t *gid);
 
 #ifdef HAVE_GNUTLS_GNUTLS_H
 void crm_gnutls_global_init(void);
 #endif
 
-bool pcmk_acl_required(const char *user);
-
 char *pcmk_hostname(void);
 
+bool pcmk_str_is_infinity(const char *s);
+bool pcmk_str_is_minus_infinity(const char *s);
+
+#ifndef PCMK__NO_COMPAT
+/* Everything here is deprecated and kept only for public API backward
+ * compatibility. It will be moved to compatibility.h in a future release.
+ */
+
+//! \deprecated Use crm_parse_interval_spec() instead
+#define crm_get_interval crm_parse_interval_spec
+
+//! \deprecated Use !pcmk_is_set() or !pcmk_all_flags_set() instead
+static inline gboolean
+is_not_set(long long word, long long bit)
+{
+    return ((word & bit) == 0);
+}
+
+//! \deprecated Use pcmk_is_set() or pcmk_all_flags_set() instead
+static inline gboolean
+is_set(long long word, long long bit)
+{
+    return ((word & bit) == bit);
+}
+
+//! \deprecated Use pcmk_any_flags_set() instead
+static inline gboolean
+is_set_any(long long word, long long bit)
+{
+    return ((word & bit) != 0);
+}
+
+//! \deprecated Use pcmk_get_ra_caps() instead
+bool crm_provider_required(const char *standard);
+
+//! \deprecated Use strcmp or strcasecmp instead
+gboolean crm_str_eq(const char *a, const char *b, gboolean use_case);
+
+//! \deprecated Use strcmp instead
+gboolean safe_str_neq(const char *a, const char *b);
+
+//! \deprecated Use strcasecmp instead
+#define safe_str_eq(a, b) crm_str_eq(a, b, FALSE)
+
+#endif // PCMK__NO_COMPAT
+
 #ifdef __cplusplus
 }
 #endif
