@@ -1,5 +1,7 @@
 /*
- * Copyright 2011-2018 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2011-2020 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -8,8 +10,41 @@
 #ifndef STONITH_NG_INTERNAL__H
 #  define STONITH_NG_INTERNAL__H
 
+#  include <glib.h>
 #  include <crm/common/ipc.h>
 #  include <crm/common/xml.h>
+#  include <crm/common/output_internal.h>
+#  include <crm/stonith-ng.h>
+
+enum st_device_flags
+{
+    st_device_supports_list   = 0x0001,
+    st_device_supports_status = 0x0002,
+    st_device_supports_reboot = 0x0004,
+    st_device_supports_parameter_plug = 0x0008,
+    st_device_supports_parameter_port = 0x0010,
+};
+
+#define stonith__set_device_flags(device_flags, device_id, flags_to_set) do { \
+        device_flags = pcmk__set_flags_as(__func__, __LINE__, LOG_TRACE,      \
+                                          "Fence device", device_id,          \
+                                          (device_flags), (flags_to_set),     \
+                                          #flags_to_set);                     \
+    } while (0)
+
+#define stonith__set_call_options(st_call_opts, call_for, flags_to_set) do { \
+        st_call_opts = pcmk__set_flags_as(__func__, __LINE__, LOG_TRACE,     \
+                                          "Fencer call", (call_for),         \
+                                          (st_call_opts), (flags_to_set),    \
+                                          #flags_to_set);                    \
+    } while (0)
+
+#define stonith__clear_call_options(st_call_opts, call_for, flags_to_clear) do { \
+        st_call_opts = pcmk__clear_flags_as(__func__, __LINE__, LOG_TRACE,     \
+                                            "Fencer call", (call_for),         \
+                                            (st_call_opts), (flags_to_clear),  \
+                                            #flags_to_clear);                  \
+    } while (0)
 
 struct stonith_action_s;
 typedef struct stonith_action_s stonith_action_t;
@@ -19,16 +54,19 @@ stonith_action_t *stonith_action_create(const char *agent,
                                         const char *victim,
                                         uint32_t victim_nodeid,
                                         int timeout,
-                                        GHashTable * device_args, GHashTable * port_map);
+                                        GHashTable * device_args,
+                                        GHashTable * port_map,
+                                        const char * host_arg);
 void stonith__destroy_action(stonith_action_t *action);
 void stonith__action_result(stonith_action_t *action, int *rc, char **output,
                             char **error_output);
 
-GPid
+int
 stonith_action_execute_async(stonith_action_t * action,
                              void *userdata,
                              void (*done) (GPid pid, int rc, const char *output,
-                                           gpointer user_data));
+                                           gpointer user_data),
+                             void (*fork_cb) (GPid pid, gpointer user_data));
 
 int stonith__execute(stonith_action_t *action);
 
@@ -43,6 +81,17 @@ xmlNode *create_device_registration_xml(const char *id,
                                         stonith_key_value_t *params,
                                         const char *rsc_provides);
 
+void stonith__register_messages(pcmk__output_t *out);
+
+GList *stonith__parse_targets(const char *hosts);
+
+gboolean stonith__later_succeeded(stonith_history_t *event, stonith_history_t *top_history);
+stonith_history_t *stonith__sort_history(stonith_history_t *history);
+
+void stonith__device_parameter_flags(uint32_t *device_flags,
+                                     const char *device_name,
+                                     xmlNode *metadata);
+
 #  define ST_LEVEL_MAX 10
 
 #  define F_STONITH_CLIENTID      "st_clientid"
@@ -52,10 +101,12 @@ xmlNode *create_device_registration_xml(const char *id,
 #  define F_STONITH_OPERATION     "st_op"
 #  define F_STONITH_TARGET        "st_target"
 #  define F_STONITH_REMOTE_OP_ID  "st_remote_op"
+#  define F_STONITH_REMOTE_OP_ID_RELAY  "st_remote_op_relay"
 #  define F_STONITH_RC            "st_rc"
 /*! Timeout period per a device execution */
 #  define F_STONITH_TIMEOUT       "st_timeout"
 #  define F_STONITH_TOLERANCE     "st_tolerance"
+#  define F_STONITH_DELAY         "st_delay"
 /*! Action specific timeout period returned in query of fencing devices. */
 #  define F_STONITH_ACTION_TIMEOUT       "st_action_timeout"
 /*! Host in query result is not allowed to run this action */
@@ -91,6 +142,7 @@ xmlNode *create_device_registration_xml(const char *id,
 #  define F_STONITH_DEVICE        "st_device_id"
 #  define F_STONITH_ACTION        "st_device_action"
 #  define F_STONITH_MODE          "st_mode"
+#  define F_STONITH_MERGED        "st_op_merged"
 
 #  define T_STONITH_NG        "stonith-ng"
 #  define T_STONITH_REPLY     "st-reply"
@@ -138,7 +190,29 @@ int stonith__list_rhcs_agents(stonith_key_value_t **devices);
 int stonith__rhcs_metadata(const char *agent, int timeout, char **output);
 bool stonith__agent_is_rhcs(const char *agent);
 int stonith__rhcs_validate(stonith_t *st, int call_options, const char *target,
-                           const char *agent, GHashTable *params,
+                           const char *agent, GHashTable *params, const char *host_arg,
                            int timeout, char **output, char **error_output);
 
+int stonith__failed_history(pcmk__output_t *out, va_list args);
+int stonith__history(pcmk__output_t *out, va_list args);
+int stonith__full_history(pcmk__output_t *out, va_list args);
+int stonith__full_history_xml(pcmk__output_t *out, va_list args);
+int stonith__last_fenced_html(pcmk__output_t *out, va_list args);
+int stonith__last_fenced_text(pcmk__output_t *out, va_list args);
+int stonith__last_fenced_xml(pcmk__output_t *out, va_list args);
+int stonith__pending_actions(pcmk__output_t *out, va_list args);
+int stonith__event_html(pcmk__output_t *out, va_list args);
+int stonith__event_text(pcmk__output_t *out, va_list args);
+int stonith__event_xml(pcmk__output_t *out, va_list args);
+int stonith__validate_agent_html(pcmk__output_t *out, va_list args);
+int stonith__validate_agent_text(pcmk__output_t *out, va_list args);
+int stonith__validate_agent_xml(pcmk__output_t *out, va_list args);
+
+stonith_history_t *stonith__first_matching_event(stonith_history_t *history,
+                                                 bool (*matching_fn)(stonith_history_t *, void *),
+                                                 void *user_data);
+bool stonith__event_state_pending(stonith_history_t *history, void *user_data);
+bool stonith__event_state_eq(stonith_history_t *history, void *user_data);
+bool stonith__event_state_neq(stonith_history_t *history, void *user_data);
+
 #endif
