@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -15,7 +15,6 @@ extern "C" {
 #endif
 
 #  include <glib.h>
-#  include <regex.h>
 
 #  include <crm/crm.h>
 #  include <crm/common/iso8601.h>
@@ -28,21 +27,11 @@ enum expression_type {
     loc_expr,
     role_expr,
     time_expr,
-    version_expr
+    version_expr,
+    rsc_expr,
+    op_expr
 };
 
-typedef struct pe_re_match_data {
-    char *string;
-    int nregs;
-    regmatch_t *pmatch;
-} pe_re_match_data_t;
-
-typedef struct pe_match_data {
-    pe_re_match_data_t *re;
-    GHashTable *params;
-    GHashTable *meta;
-} pe_match_data_t;
-
 enum expression_type find_expression_type(xmlNode * expr);
 
 gboolean pe_evaluate_rules(xmlNode *ruleset, GHashTable *node_hash,
@@ -57,21 +46,38 @@ gboolean pe_test_expression(xmlNode *expr, GHashTable *node_hash,
                             crm_time_t *next_change,
                             pe_match_data_t *match_data);
 
+void pe_eval_nvpairs(xmlNode *top, xmlNode *xml_obj, const char *set_name,
+                     pe_rule_eval_data_t *rule_data, GHashTable *hash,
+                     const char *always_first, gboolean overwrite,
+                     crm_time_t *next_change);
+
 void pe_unpack_nvpairs(xmlNode *top, xmlNode *xml_obj, const char *set_name,
                        GHashTable *node_hash, GHashTable *hash,
                        const char *always_first, gboolean overwrite,
                        crm_time_t *now, crm_time_t *next_change);
 
 #if ENABLE_VERSIONED_ATTRS
-void pe_unpack_versioned_attributes(xmlNode *top, xmlNode *xml_obj,
-                                    const char *set_name, GHashTable *node_hash,
-                                    xmlNode *hash, crm_time_t *now,
-                                    crm_time_t *next_change);
+void pe_eval_versioned_attributes(xmlNode *top, xmlNode *xml_obj,
+                                  const char *set_name, pe_rule_eval_data_t *rule_data,
+                                  xmlNode *hash, crm_time_t *next_change);
+
 GHashTable *pe_unpack_versioned_parameters(xmlNode *versioned_params, const char *ra_version);
 #endif
 
 char *pe_expand_re_matches(const char *string, pe_re_match_data_t * match_data);
 
+gboolean pe_eval_rules(xmlNode *ruleset, pe_rule_eval_data_t *rule_data,
+                       crm_time_t *next_change);
+gboolean pe_eval_expr(xmlNode *rule, pe_rule_eval_data_t *rule_data,
+                      crm_time_t *next_change);
+gboolean pe_eval_subexpr(xmlNode *expr, pe_rule_eval_data_t *rule_data,
+                         crm_time_t *next_change);
+
+#ifndef PCMK__NO_COMPAT
+/* Everything here is deprecated and kept only for public API backward
+ * compatibility. It will be moved to compatibility.h in a future release.
+ */
+
 //! \deprecated Use pe_evaluate_rules() instead
 gboolean test_ruleset(xmlNode *ruleset, GHashTable *node_hash, crm_time_t *now);
 
@@ -108,6 +114,7 @@ void unpack_instance_attributes(xmlNode *top, xmlNode *xml_obj,
                                 const char *set_name, GHashTable *node_hash,
                                 GHashTable *hash, const char *always_first,
                                 gboolean overwrite, crm_time_t *now);
+#endif
 
 #ifdef __cplusplus
 }
