@@ -1,5 +1,6 @@
 /*
- * Copyright 2004-2018 International Business Machines
+ * Original copyright 2004 International Business Machines
+ * Later changes copyright 2008-2020 the Pacemaker project contributors
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -9,6 +10,7 @@
 #include <unistd.h>
 #include <limits.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
@@ -23,16 +25,36 @@
 #include <crm/msg_xml.h>
 #include <crm/common/ipc.h>
 #include <crm/common/xml.h>
+#include <crm/common/xml_internal.h>
 
-#define CIB_FLAG_DIRTY 0x00001
-#define CIB_FLAG_LIVE  0x00002
+enum cib_file_flags {
+    cib_file_flag_dirty = (1 << 0),
+    cib_file_flag_live  = (1 << 1),
+};
 
 typedef struct cib_file_opaque_s {
-    int flags;
+    uint32_t flags; // Group of enum cib_file_flags
     char *filename;
-
 } cib_file_opaque_t;
 
+#define cib_set_file_flags(cibfile, flags_to_set) do {                  \
+        (cibfile)->flags = pcmk__set_flags_as(__func__, __LINE__,       \
+                                              LOG_TRACE, "CIB file",    \
+                                              cibfile->filename,        \
+                                              (cibfile)->flags,         \
+                                              (flags_to_set),           \
+                                              #flags_to_set);           \
+    } while (0)
+
+#define cib_clear_file_flags(cibfile, flags_to_clear) do {              \
+        (cibfile)->flags = pcmk__clear_flags_as(__func__, __LINE__,     \
+                                                LOG_TRACE, "CIB file",  \
+                                                cibfile->filename,      \
+                                                (cibfile)->flags,       \
+                                                (flags_to_clear),       \
+                                                #flags_to_clear);       \
+    } while (0)
+
 int cib_file_perform_op(cib_t * cib, const char *op, const char *host, const char *section,
                         xmlNode * data, xmlNode ** output_data, int call_options);
 
@@ -75,22 +97,25 @@ static gboolean
 cib_file_verify_digest(xmlNode *root, const char *sigfile)
 {
     gboolean passed = FALSE;
-    char *expected = crm_read_contents(sigfile);
+    char *expected;
+    int rc = pcmk__file_contents(sigfile, &expected);
 
-    if (expected == NULL) {
-        switch (errno) {
-            case 0:
+    switch (rc) {
+        case pcmk_rc_ok:
+            if (expected == NULL) {
                 crm_err("On-disk digest at %s is empty", sigfile);
                 return FALSE;
-            case ENOENT:
-                crm_warn("No on-disk digest present at %s", sigfile);
-                return TRUE;
-            default:
-                crm_perror(LOG_ERR, "Could not read on-disk digest from %s", sigfile);
-                return FALSE;
-        }
+            }
+            break;
+        case ENOENT:
+            crm_warn("No on-disk digest present at %s", sigfile);
+            return TRUE;
+        default:
+            crm_err("Could not read on-disk digest from %s: %s",
+                    sigfile, pcmk_rc_str(rc));
+            return FALSE;
     }
-    passed = crm_digest_verify(root, expected);
+    passed = pcmk__verify_digest(root, expected);
     free(expected);
     return passed;
 }
@@ -142,7 +167,7 @@ cib_file_read_and_verify(const char *filename, const char *sigfile, xmlNode **ro
 
     /* If sigfile is not specified, use original file name plus .sig */
     if (sigfile == NULL) {
-        sigfile = local_sigfile = crm_concat(filename, "sig", '.');
+        sigfile = local_sigfile = crm_strdup_printf("%s.sig", filename);
     }
 
     /* Verify that digests match */
@@ -184,16 +209,16 @@ cib_file_is_live(const char *filename)
 
     if (filename != NULL) {
         // Canonicalize file names for true comparison
-        char *real_filename = crm_compat_realpath(filename);
+        char *real_filename = NULL;
 
-        if (real_filename != NULL) {
-            char *real_livename;
+        if (pcmk__real_path(filename, &real_filename) == pcmk_rc_ok) {
+            char *real_livename = NULL;
 
-            real_livename = crm_compat_realpath(CRM_CONFIG_DIR "/" CIB_LIVE_NAME);
-            if (real_livename && !strcmp(real_filename, real_livename)) {
-                same = TRUE;
+            if (pcmk__real_path(CRM_CONFIG_DIR "/" CIB_LIVE_NAME,
+                                &real_livename) == pcmk_rc_ok) {
+                same = !strcmp(real_filename, real_livename);
+                free(real_livename);
             }
-            free(real_livename);
             free(real_filename);
         }
     }
@@ -221,17 +246,21 @@ static int
 cib_file_backup(const char *cib_dirname, const char *cib_filename)
 {
     int rc = 0;
-    char *cib_path = crm_concat(cib_dirname, cib_filename, '/');
-    char *cib_digest = crm_concat(cib_path, "sig", '.');
+    unsigned int seq;
+    char *cib_path = crm_strdup_printf("%s/%s", cib_dirname, cib_filename);
+    char *cib_digest = crm_strdup_printf("%s.sig", cib_path);
+    char *backup_path;
+    char *backup_digest;
 
-    /* Figure out what backup file sequence number to use */
-    int seq = get_last_sequence(cib_dirname, CIB_SERIES);
-    char *backup_path = generate_series_filename(cib_dirname, CIB_SERIES, seq,
-                                                 CIB_SERIES_BZIP);
-    char *backup_digest = crm_concat(backup_path, "sig", '.');
-
-    CRM_ASSERT((cib_path != NULL) && (cib_digest != NULL)
-               && (backup_path != NULL) && (backup_digest != NULL));
+    // Determine backup and digest file names
+    if (pcmk__read_series_sequence(cib_dirname, CIB_SERIES,
+                                   &seq) != pcmk_rc_ok) {
+        // @TODO maybe handle errors better ...
+        seq = 0;
+    }
+    backup_path = pcmk__series_filename(cib_dirname, CIB_SERIES, seq,
+                                        CIB_SERIES_BZIP);
+    backup_digest = crm_strdup_printf("%s.sig", backup_path);
 
     /* Remove the old backups if they exist */
     unlink(backup_path);
@@ -251,8 +280,11 @@ cib_file_backup(const char *cib_dirname, const char *cib_filename)
 
     /* Update the last counter and ensure everything is sync'd to media */
     } else {
-        write_last_sequence(cib_dirname, CIB_SERIES, seq + 1, CIB_SERIES_MAX);
+        pcmk__write_series_sequence(cib_dirname, CIB_SERIES, ++seq,
+                                    CIB_SERIES_MAX);
         if (cib_do_chown) {
+            int rc2;
+
             if ((chown(backup_path, cib_file_owner, cib_file_group) < 0)
                     && (errno != ENOENT)) {
                 crm_perror(LOG_ERR, "Could not set owner of %s", backup_path);
@@ -263,15 +295,15 @@ cib_file_backup(const char *cib_dirname, const char *cib_filename)
                 crm_perror(LOG_ERR, "Could not set owner of %s", backup_digest);
                 rc = -1;
             }
-            if (crm_chown_last_sequence(cib_dirname, CIB_SERIES, cib_file_owner,
-                                        cib_file_group) < 0) {
-                crm_perror(LOG_ERR,
-                           "Could not set owner of %s last sequence file",
-                           cib_dirname);
+            rc2 = pcmk__chown_series_sequence(cib_dirname, CIB_SERIES,
+                                              cib_file_owner, cib_file_group);
+            if (rc2 != pcmk_rc_ok) {
+                crm_err("Could not set owner of sequence file in %s: %s",
+                        cib_dirname, pcmk_rc_str(rc2));
                 rc = -1;
             }
         }
-        crm_sync_directory(cib_dirname);
+        pcmk__sync_directory(cib_dirname);
         crm_info("Archived previous version as %s", backup_path);
     }
 
@@ -300,7 +332,7 @@ cib_file_prepare_xml(xmlNode *root)
 
     /* Always write out with num_updates=0 and current last-written timestamp */
     crm_xml_add(root, XML_ATTR_NUMUPDATES, "0");
-    crm_xml_add_last_written(root);
+    pcmk__xe_add_last_written(root);
 
     /* Delete status section before writing to file, because
      * we discard it on startup anyway, and users get confused by it */
@@ -338,8 +370,8 @@ cib_file_write_with_digest(xmlNode *cib_root, const char *cib_dirname,
                                                 XML_ATTR_GENERATION_ADMIN);
 
     /* Determine full CIB and signature pathnames */
-    char *cib_path = crm_concat(cib_dirname, cib_filename, '/');
-    char *digest_path = crm_concat(cib_path, "sig", '.');
+    char *cib_path = crm_strdup_printf("%s/%s", cib_dirname, cib_filename);
+    char *digest_path = crm_strdup_printf("%s.sig", cib_path);
 
     /* Create temporary file name patterns for writing out CIB and signature */
     char *tmp_cib = crm_strdup_printf("%s/cib.XXXXXX", cib_dirname);
@@ -418,8 +450,10 @@ cib_file_write_with_digest(xmlNode *cib_root, const char *cib_dirname,
         close(fd);
         goto cleanup;
     }
-    if (crm_write_sync(fd, digest) < 0) {
-        crm_perror(LOG_ERR, "Could not write digest to file %s", tmp_digest);
+    rc = pcmk__write_sync(fd, digest);
+    if (rc != pcmk_rc_ok) {
+        crm_err("Could not write digest to %s: %s",
+                tmp_digest, pcmk_rc_str(rc));
         exit_rc = pcmk_err_cib_save;
         close(fd);
         goto cleanup;
@@ -444,7 +478,7 @@ cib_file_write_with_digest(xmlNode *cib_root, const char *cib_dirname,
                    digest_path);
         exit_rc = pcmk_err_cib_save;
     }
-    crm_sync_directory(cib_dirname);
+    pcmk__sync_directory(cib_dirname);
 
   cleanup:
     free(cib_path);
@@ -472,7 +506,7 @@ cib_file_new(const char *cib_location)
     }
     private->flags = 0;
     if (cib_file_is_live(cib_location)) {
-        set_bit(private->flags, CIB_FLAG_LIVE);
+        cib_set_file_flags(private, cib_file_flag_live);
         crm_trace("File %s detected as live CIB", cib_location);
     }
     private->filename = strdup(cib_location);
@@ -555,15 +589,15 @@ cib_file_signon(cib_t * cib, const char *name, enum cib_conn_type type)
     }
 
     if (rc == pcmk_ok) {
-        crm_debug("%s: Opened connection to local file '%s'", name, private->filename);
+        crm_debug("Opened connection to local file '%s' for %s",
+                  private->filename, name);
         cib->state = cib_connected_command;
         cib->type = cib_command;
 
     } else {
-        fprintf(stderr, "%s: Connection to local file '%s' failed: %s\n",
-                name, private->filename, pcmk_strerror(rc));
+        crm_info("Connection to local file '%s' for %s failed: %s\n",
+                 private->filename, name, pcmk_strerror(rc));
     }
-
     return rc;
 }
 
@@ -668,17 +702,17 @@ cib_file_signoff(cib_t * cib)
     cib->type = cib_no_connection;
 
     /* If the in-memory CIB has been changed, write it to disk */
-    if (is_set(private->flags, CIB_FLAG_DIRTY)) {
+    if (pcmk_is_set(private->flags, cib_file_flag_dirty)) {
 
         /* If this is the live CIB, write it out with a digest */
-        if (is_set(private->flags, CIB_FLAG_LIVE)) {
+        if (pcmk_is_set(private->flags, cib_file_flag_live)) {
             if (cib_file_write_live(private->filename) < 0) {
                 rc = pcmk_err_generic;
             }
 
         /* Otherwise, it's a simple write */
         } else {
-            gboolean do_bzip = crm_ends_with_ext(private->filename, ".bz2");
+            gboolean do_bzip = pcmk__ends_with_ext(private->filename, ".bz2");
 
             if (write_xml_file(in_mem_cib, private->filename, do_bzip) <= 0) {
                 rc = pcmk_err_generic;
@@ -687,7 +721,7 @@ cib_file_signoff(cib_t * cib)
 
         if (rc == pcmk_ok) {
             crm_info("Wrote CIB to %s", private->filename);
-            clear_bit(private->flags, CIB_FLAG_DIRTY);
+            cib_clear_file_flags(private, cib_file_flag_dirty);
         } else {
             crm_err("Could not write CIB to %s", private->filename);
         }
@@ -769,8 +803,17 @@ cib_file_perform_op_delegate(cib_t * cib, const char *op, const char *host, cons
     static int max_msg_types = DIMOF(cib_file_ops);
     cib_file_opaque_t *private = cib->variant_opaque;
 
-    crm_info("%s on %s", op, section);
-    call_options |= (cib_no_mtime | cib_inhibit_bcast | cib_scope_local);
+#if ENABLE_ACL
+    crm_info("Handling %s operation for %s as %s",
+             (op? op : "invalid"), (section? section : "entire CIB"),
+             (user_name? user_name : "default user"));
+#else
+    crm_info("Handling %s operation for %s",
+             (op? op : "invalid"), (section? section : "entire CIB"));
+#endif
+
+    cib__set_call_options(call_options, "file operation",
+                          cib_no_mtime|cib_inhibit_bcast|cib_scope_local);
 
     if (cib->state == cib_disconnected) {
         return -ENOTCONN;
@@ -785,7 +828,7 @@ cib_file_perform_op_delegate(cib_t * cib, const char *op, const char *host, cons
     }
 
     for (lpc = 0; lpc < max_msg_types; lpc++) {
-        if (safe_str_eq(op, cib_file_ops[lpc].op)) {
+        if (pcmk__str_eq(op, cib_file_ops[lpc].op, pcmk__str_casei)) {
             fn = &(cib_file_ops[lpc].fn);
             query = cib_file_ops[lpc].read_only;
             break;
@@ -802,11 +845,10 @@ cib_file_perform_op_delegate(cib_t * cib, const char *op, const char *host, cons
     if(user_name) {
         crm_xml_add(request, XML_ACL_TAG_USER, user_name);
     }
-    crm_trace("Performing %s operation as %s", op, user_name);
 #endif
 
     /* Mirror the logic in cib_prepare_common() */
-    if (section != NULL && data != NULL && crm_str_eq(crm_element_name(data), XML_TAG_CIB, TRUE)) {
+    if (section != NULL && data != NULL && pcmk__str_eq(crm_element_name(data), XML_TAG_CIB, pcmk__str_none)) {
         data = get_object_root(section, data);
     }
 
@@ -826,7 +868,7 @@ cib_file_perform_op_delegate(cib_t * cib, const char *op, const char *host, cons
         xml_log_patchset(LOG_DEBUG, "cib:diff", cib_diff);
         free_xml(in_mem_cib);
         in_mem_cib = result_cib;
-        set_bit(private->flags, CIB_FLAG_DIRTY);
+        cib_set_file_flags(private, cib_file_flag_dirty);
     }
 
     free_xml(cib_diff);
