@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -21,6 +21,7 @@
 #include <crm/crm.h>
 #include <crm/msg_xml.h>
 #include <crm/common/xml.h>
+#include <crm/common/xml_internal.h>
 #include "crmcommon_private.h"
 
 #define MAX_XPATH_LEN	4096
@@ -31,7 +32,7 @@ typedef struct xml_acl_s {
 } xml_acl_t;
 
 static void
-__xml_acl_free(void *data)
+free_acl(void *data)
 {
     if (data) {
         xml_acl_t *acl = data;
@@ -44,11 +45,11 @@ __xml_acl_free(void *data)
 void
 pcmk__free_acls(GList *acls)
 {
-    g_list_free_full(acls, __xml_acl_free);
+    g_list_free_full(acls, free_acl);
 }
 
 static GList *
-__xml_acl_create(xmlNode *xml, GList *acls, enum xml_private_flags mode)
+create_acl(xmlNode *xml, GList *acls, enum xml_private_flags mode)
 {
     xml_acl_t *acl = NULL;
 
@@ -143,12 +144,12 @@ __xml_acl_create(xmlNode *xml, GList *acls, enum xml_private_flags mode)
  * \return New head of (possibly modified) acls
  */
 static GList *
-__xml_acl_parse_entry(xmlNode *acl_top, xmlNode *acl_entry, GList *acls)
+parse_acl_entry(xmlNode *acl_top, xmlNode *acl_entry, GList *acls)
 {
     xmlNode *child = NULL;
 
-    for (child = __xml_first_child_element(acl_entry); child;
-         child = __xml_next_element(child)) {
+    for (child = pcmk__xe_first_child(acl_entry); child;
+         child = pcmk__xe_next(child)) {
         const char *tag = crm_element_name(child);
         const char *kind = crm_element_value(child, XML_ACL_ATTR_KIND);
 
@@ -167,8 +168,8 @@ __xml_acl_parse_entry(xmlNode *acl_top, xmlNode *acl_entry, GList *acls)
             if (ref_role) {
                 xmlNode *role = NULL;
 
-                for (role = __xml_first_child_element(acl_top); role;
-                     role = __xml_next_element(role)) {
+                for (role = pcmk__xe_first_child(acl_top); role;
+                     role = pcmk__xe_next(role)) {
                     if (!strcmp(XML_ACL_TAG_ROLE, (const char *) role->name)) {
                         const char *role_id = crm_element_value(role,
                                                                 XML_ATTR_ID);
@@ -176,7 +177,7 @@ __xml_acl_parse_entry(xmlNode *acl_top, xmlNode *acl_entry, GList *acls)
                         if (role_id && strcmp(ref_role, role_id) == 0) {
                             crm_trace("Unpacking referenced role '%s' in ACL <%s> element",
                                       role_id, crm_element_name(acl_entry));
-                            acls = __xml_acl_parse_entry(acl_top, role, acls);
+                            acls = parse_acl_entry(acl_top, role, acls);
                             break;
                         }
                     }
@@ -184,13 +185,13 @@ __xml_acl_parse_entry(xmlNode *acl_top, xmlNode *acl_entry, GList *acls)
             }
 
         } else if (strcmp(XML_ACL_TAG_READ, tag) == 0) {
-            acls = __xml_acl_create(child, acls, xpf_acl_read);
+            acls = create_acl(child, acls, xpf_acl_read);
 
         } else if (strcmp(XML_ACL_TAG_WRITE, tag) == 0) {
-            acls = __xml_acl_create(child, acls, xpf_acl_write);
+            acls = create_acl(child, acls, xpf_acl_write);
 
         } else if (strcmp(XML_ACL_TAG_DENY, tag) == 0) {
-            acls = __xml_acl_create(child, acls, xpf_acl_deny);
+            acls = create_acl(child, acls, xpf_acl_deny);
 
         } else {
             crm_warn("Ignoring unknown ACL %s '%s'",
@@ -224,15 +225,15 @@ __xml_acl_parse_entry(xmlNode *acl_top, xmlNode *acl_entry, GList *acls)
 */
 
 static const char *
-__xml_acl_to_text(enum xml_private_flags flags)
+acl_to_text(enum xml_private_flags flags)
 {
-    if (is_set(flags, xpf_acl_deny)) {
+    if (pcmk_is_set(flags, xpf_acl_deny)) {
         return "deny";
 
-    } else if (is_set(flags, xpf_acl_write) || is_set(flags, xpf_acl_create)) {
+    } else if (pcmk_any_flags_set(flags, xpf_acl_write|xpf_acl_create)) {
         return "read/write";
 
-    } else if (is_set(flags, xpf_acl_read)) {
+    } else if (pcmk_is_set(flags, xpf_acl_read)) {
         return "read";
     }
     return "none";
@@ -245,7 +246,7 @@ pcmk__apply_acl(xmlNode *xml)
     xml_private_t *p = xml->doc->_private;
     xmlXPathObjectPtr xpathObj = NULL;
 
-    if (xml_acl_enabled(xml) == FALSE) {
+    if (!xml_acl_enabled(xml)) {
         crm_trace("Skipping ACLs for user '%s' because not enabled for this XML",
                   p->user);
         return;
@@ -264,41 +265,29 @@ pcmk__apply_acl(xmlNode *xml)
 
             p = match->_private;
             crm_trace("Applying %s ACL to %s matched by %s",
-                      __xml_acl_to_text(acl->mode), path, acl->xpath);
+                      acl_to_text(acl->mode), path, acl->xpath);
 
 #ifdef SUSE_ACL_COMPAT
-            if (is_not_set(p->flags, acl->mode)
-                && (is_set(p->flags, xpf_acl_read)
-                    || is_set(p->flags, xpf_acl_write)
-                    || is_set(p->flags, xpf_acl_deny))) {
-                crm_config_warn("Configuration element %s is matched by "
-                                "multiple ACL rules, only the first applies "
-                                "('%s' wins over '%s')",
-                                path, __xml_acl_to_text(p->flags),
-                                __xml_acl_to_text(acl->mode));
+            if (!pcmk_is_set(p->flags, acl->mode)
+                && pcmk_any_flags_set(p->flags,
+                                      xpf_acl_read|xpf_acl_write|xpf_acl_deny)) {
+                pcmk__config_warn("Configuration element %s is matched by "
+                                  "multiple ACL rules, only the first applies "
+                                  "('%s' wins over '%s')",
+                                  path, acl_to_text(p->flags),
+                                  acl_to_text(acl->mode));
                 free(path);
                 continue;
             }
 #endif
-            p->flags |= acl->mode;
+            pcmk__set_xml_flags(p, acl->mode);
             free(path);
         }
         crm_trace("Applied %s ACL %s (%d match%s)",
-                  __xml_acl_to_text(acl->mode), acl->xpath, max,
+                  acl_to_text(acl->mode), acl->xpath, max,
                   ((max == 1)? "" : "es"));
         freeXpathObject(xpathObj);
     }
-
-    p = xml->_private;
-    if (is_not_set(p->flags, xpf_acl_read)
-        && is_not_set(p->flags, xpf_acl_write)) {
-
-        p->flags |= xpf_acl_deny;
-        p = xml->doc->_private;
-        crm_info("Applied default deny ACL for user '%s' to <%s>",
-                 p->user, crm_element_name(xml));
-    }
-
 }
 
 /*!
@@ -321,13 +310,13 @@ pcmk__unpack_acl(xmlNode *source, xmlNode *target, const char *user)
     }
 
     p = target->doc->_private;
-    if (pcmk_acl_required(user) == FALSE) {
+    if (!pcmk_acl_required(user)) {
         crm_trace("Not unpacking ACLs because not required for user '%s'",
                   user);
 
     } else if (p->acls == NULL) {
         xmlNode *acls = get_xpath_object("//" XML_CIB_TAG_ACLS,
-                                         source, LOG_TRACE);
+                                         source, LOG_NEVER);
 
         free(p->user);
         p->user = strdup(user);
@@ -335,8 +324,8 @@ pcmk__unpack_acl(xmlNode *source, xmlNode *target, const char *user)
         if (acls) {
             xmlNode *child = NULL;
 
-            for (child = __xml_first_child_element(acls); child;
-                 child = __xml_next_element(child)) {
+            for (child = pcmk__xe_first_child(acls); child;
+                 child = pcmk__xe_next(child)) {
                 const char *tag = crm_element_name(child);
 
                 if (!strcmp(tag, XML_ACL_TAG_USER)
@@ -345,7 +334,7 @@ pcmk__unpack_acl(xmlNode *source, xmlNode *target, const char *user)
 
                     if (id && strcmp(id, user) == 0) {
                         crm_debug("Unpacking ACLs for user '%s'", id);
-                        p->acls = __xml_acl_parse_entry(acls, child, p->acls);
+                        p->acls = parse_acl_entry(acls, child, p->acls);
                     }
                 }
             }
@@ -355,44 +344,36 @@ pcmk__unpack_acl(xmlNode *source, xmlNode *target, const char *user)
 }
 
 static inline bool
-__xml_acl_mode_test(enum xml_private_flags allowed,
-                    enum xml_private_flags requested)
+test_acl_mode(enum xml_private_flags allowed, enum xml_private_flags requested)
 {
-    if (is_set(allowed, xpf_acl_deny)) {
-        return FALSE;
+    if (pcmk_is_set(allowed, xpf_acl_deny)) {
+        return false;
 
-    } else if (is_set(allowed, requested)) {
-        return TRUE;
+    } else if (pcmk_all_flags_set(allowed, requested)) {
+        return true;
 
-    } else if (is_set(requested, xpf_acl_read)
-               && is_set(allowed, xpf_acl_write)) {
-        return TRUE;
+    } else if (pcmk_is_set(requested, xpf_acl_read)
+               && pcmk_is_set(allowed, xpf_acl_write)) {
+        return true;
 
-    } else if (is_set(requested, xpf_acl_create)
-               && is_set(allowed, xpf_acl_write)) {
-        return TRUE;
-
-    } else if (is_set(requested, xpf_acl_create)
-               && is_set(allowed, xpf_created)) {
-        return TRUE;
+    } else if (pcmk_is_set(requested, xpf_acl_create)
+               && pcmk_any_flags_set(allowed, xpf_acl_write|xpf_created)) {
+        return true;
     }
-    return FALSE;
+    return false;
 }
 
-/* rc = TRUE if orig_cib has been filtered
- * That means '*result' rather than 'xml' should be exploited afterwards
- */
 static bool
-__xml_purge_attributes(xmlNode *xml)
+purge_xml_attributes(xmlNode *xml)
 {
     xmlNode *child = NULL;
     xmlAttr *xIter = NULL;
-    bool readable_children = FALSE;
+    bool readable_children = false;
     xml_private_t *p = xml->_private;
 
-    if (__xml_acl_mode_test(p->flags, xpf_acl_read)) {
+    if (test_acl_mode(p->flags, xpf_acl_read)) {
         crm_trace("%s[@id=%s] is readable", crm_element_name(xml), ID(xml));
-        return TRUE;
+        return true;
     }
 
     xIter = xml->properties;
@@ -408,15 +389,15 @@ __xml_purge_attributes(xmlNode *xml)
         xmlUnsetProp(xml, tmp->name);
     }
 
-    child = __xml_first_child(xml);
+    child = pcmk__xml_first_child(xml);
     while ( child != NULL ) {
         xmlNode *tmp = child;
 
-        child = __xml_next(child);
-        readable_children |= __xml_purge_attributes(tmp);
+        child = pcmk__xml_next(child);
+        readable_children |= purge_xml_attributes(tmp);
     }
 
-    if (readable_children == FALSE) {
+    if (!readable_children) {
         free_xml(xml); /* Nothing readable under here, purge completely */
     }
     return readable_children;
@@ -431,7 +412,8 @@ __xml_purge_attributes(xmlNode *xml)
  * \param[in]  xml         XML to be copied
  * \param[out] result      Copy of XML portions readable via ACLs
  *
- * \return TRUE if xml exists and ACLs are required for user, otherwise FALSE
+ * \return true if xml exists and ACLs are required for user, false otherwise
+ * \note If this returns true, caller should use \p result rather than \p xml
  */
 bool
 xml_acl_filtered_copy(const char *user, xmlNode *acl_source, xmlNode *xml,
@@ -439,24 +421,23 @@ xml_acl_filtered_copy(const char *user, xmlNode *acl_source, xmlNode *xml,
 {
     GListPtr aIter = NULL;
     xmlNode *target = NULL;
-    xml_private_t *p = NULL;
     xml_private_t *doc = NULL;
 
     *result = NULL;
-    if (xml == NULL || pcmk_acl_required(user) == FALSE) {
+    if ((xml == NULL) || !pcmk_acl_required(user)) {
         crm_trace("Not filtering XML because ACLs not required for user '%s'",
                   user);
-        return FALSE;
+        return false;
     }
 
     crm_trace("Filtering XML copy using user '%s' ACLs", user);
     target = copy_xml(xml);
     if (target == NULL) {
-        return TRUE;
+        return true;
     }
 
     pcmk__unpack_acl(acl_source, target, user);
-    pcmk__set_xml_flag(target, xpf_acl_enabled);
+    pcmk__set_xml_doc_flag(target, xpf_acl_enabled);
     pcmk__apply_acl(target);
 
     doc = target->doc->_private;
@@ -475,28 +456,27 @@ xml_acl_filtered_copy(const char *user, xmlNode *acl_source, xmlNode *xml,
             for(lpc = 0; lpc < max; lpc++) {
                 xmlNode *match = getXpathResult(xpathObj, lpc);
 
-                if (__xml_purge_attributes(match) == FALSE && match == target) {
+                if (!purge_xml_attributes(match) && (match == target)) {
                     crm_trace("ACLs deny user '%s' access to entire XML document",
                               user);
                     freeXpathObject(xpathObj);
-                    return TRUE;
+                    return true;
                 }
             }
-            crm_trace("ACLs deny user '%s' access to %s (%d match%s)",
-                      user, acl->xpath, max, ((max == 1)? "" : "es"));
+            crm_trace("ACLs deny user '%s' access to %s (%d %s)",
+                      user, acl->xpath, max,
+                      pcmk__plural_alt(max, "match", "matches"));
             freeXpathObject(xpathObj);
         }
     }
 
-    p = target->_private;
-    if (is_set(p->flags, xpf_acl_deny)
-        && (__xml_purge_attributes(target) == FALSE)) {
+    if (!purge_xml_attributes(target)) {
         crm_trace("ACLs deny user '%s' access to entire XML document", user);
-        return TRUE;
+        return true;
     }
 
     if (doc->acls) {
-        g_list_free_full(doc->acls, __xml_acl_free);
+        g_list_free_full(doc->acls, free_acl);
         doc->acls = NULL;
 
     } else {
@@ -510,7 +490,7 @@ xml_acl_filtered_copy(const char *user, xmlNode *acl_source, xmlNode *xml,
         *result = target;
     }
 
-    return TRUE;
+    return true;
 }
 
 /*!
@@ -523,7 +503,7 @@ xml_acl_filtered_copy(const char *user, xmlNode *acl_source, xmlNode *xml,
  *
  * \param[in] xml  XML element to check
  *
- * \return TRUE if XML element is implicitly allowed, FALSE otherwise
+ * \return true if XML element is implicitly allowed, false otherwise
  */
 static bool
 implicitly_allowed(xmlNode *xml)
@@ -532,18 +512,18 @@ implicitly_allowed(xmlNode *xml)
 
     for (xmlAttr *prop = xml->properties; prop != NULL; prop = prop->next) {
         if (strcmp((const char *) prop->name, XML_ATTR_ID) != 0) {
-            return FALSE;
+            return false;
         }
     }
 
     path = xml_get_path(xml);
     if (strstr(path, "/" XML_CIB_TAG_ACLS "/") != NULL) {
         free(path);
-        return FALSE;
+        return false;
     }
     free(path);
 
-    return TRUE;
+    return true;
 }
 
 #define display_id(xml) (ID(xml)? ID(xml) : "<unset>")
@@ -559,14 +539,14 @@ implicitly_allowed(xmlNode *xml)
  *
  * \param[in,out] xml        XML to check
  * \param[in]     check_top  Whether to apply checks to argument itself
- *                           (if TRUE, xml might get freed)
+ *                           (if true, xml might get freed)
  */
 void
 pcmk__apply_creation_acl(xmlNode *xml, bool check_top)
 {
     xml_private_t *p = xml->_private;
 
-    if (is_set(p->flags, xpf_created)) {
+    if (pcmk_is_set(p->flags, xpf_created)) {
         if (implicitly_allowed(xml)) {
             crm_trace("Creation of <%s> scaffolding with id=\"%s\""
                       " is implicitly allowed",
@@ -589,10 +569,10 @@ pcmk__apply_creation_acl(xmlNode *xml, bool check_top)
         }
     }
 
-    for (xmlNode *cIter = __xml_first_child(xml); cIter != NULL; ) {
+    for (xmlNode *cIter = pcmk__xml_first_child(xml); cIter != NULL; ) {
         xmlNode *child = cIter;
-        cIter = __xml_next(cIter); /* In case it is free'd */
-        pcmk__apply_creation_acl(child, TRUE);
+        cIter = pcmk__xml_next(cIter); /* In case it is free'd */
+        pcmk__apply_creation_acl(child, true);
     }
 }
 
@@ -602,9 +582,9 @@ xml_acl_denied(xmlNode *xml)
     if (xml && xml->doc && xml->doc->_private){
         xml_private_t *p = xml->doc->_private;
 
-        return is_set(p->flags, xpf_acl_denied);
+        return pcmk_is_set(p->flags, xpf_acl_denied);
     }
-    return FALSE;
+    return false;
 }
 
 void
@@ -615,8 +595,8 @@ xml_acl_disable(xmlNode *xml)
 
         /* Catch anything that was created but shouldn't have been */
         pcmk__apply_acl(xml);
-        pcmk__apply_creation_acl(xml, FALSE);
-        clear_bit(p->flags, xpf_acl_enabled);
+        pcmk__apply_creation_acl(xml, false);
+        pcmk__clear_xml_flags(p, xpf_acl_enabled);
     }
 }
 
@@ -626,9 +606,9 @@ xml_acl_enabled(xmlNode *xml)
     if (xml && xml->doc && xml->doc->_private){
         xml_private_t *p = xml->doc->_private;
 
-        return is_set(p->flags, xpf_acl_enabled);
+        return pcmk_is_set(p->flags, xpf_acl_enabled);
     }
-    return FALSE;
+    return false;
 }
 
 bool
@@ -639,7 +619,7 @@ pcmk__check_acl(xmlNode *xml, const char *name, enum xml_private_flags mode)
     CRM_ASSERT(xml->doc->_private);
 
 #if ENABLE_ACL
-    if (pcmk__tracking_xml_changes(xml, FALSE) && xml_acl_enabled(xml)) {
+    if (pcmk__tracking_xml_changes(xml, false) && xml_acl_enabled(xml)) {
         int offset = 0;
         xmlNode *parent = xml;
         char buffer[MAX_XPATH_LEN];
@@ -655,9 +635,9 @@ pcmk__check_acl(xmlNode *xml, const char *name, enum xml_private_flags mode)
 
         if (docp->acls == NULL) {
             crm_trace("User '%s' without ACLs denied %s access to %s",
-                      docp->user, __xml_acl_to_text(mode), buffer);
-            pcmk__set_xml_flag(xml, xpf_acl_denied);
-            return FALSE;
+                      docp->user, acl_to_text(mode), buffer);
+            pcmk__set_xml_doc_flag(xml, xpf_acl_denied);
+            return false;
         }
 
         /* Walk the tree upwards looking for xml_acl_* flags
@@ -675,51 +655,59 @@ pcmk__check_acl(xmlNode *xml, const char *name, enum xml_private_flags mode)
 
         while (parent && parent->_private) {
             xml_private_t *p = parent->_private;
-            if (__xml_acl_mode_test(p->flags, mode)) {
-                return TRUE;
+            if (test_acl_mode(p->flags, mode)) {
+                return true;
 
-            } else if (is_set(p->flags, xpf_acl_deny)) {
-                crm_trace("Parent ACL denies user '%s' %s access to %s",
-                          docp->user, __xml_acl_to_text(mode), buffer);
-                pcmk__set_xml_flag(xml, xpf_acl_denied);
-                return FALSE;
+            } else if (pcmk_is_set(p->flags, xpf_acl_deny)) {
+                crm_trace("%sACL denies user '%s' %s access to %s",
+                          (parent != xml) ? "Parent " : "", docp->user,
+                          acl_to_text(mode), buffer);
+                pcmk__set_xml_doc_flag(xml, xpf_acl_denied);
+                return false;
             }
             parent = parent->parent;
         }
 
         crm_trace("Default ACL denies user '%s' %s access to %s",
-                  docp->user, __xml_acl_to_text(mode), buffer);
-        pcmk__set_xml_flag(xml, xpf_acl_denied);
-        return FALSE;
+                  docp->user, acl_to_text(mode), buffer);
+        pcmk__set_xml_doc_flag(xml, xpf_acl_denied);
+        return false;
     }
 #endif
 
-    return TRUE;
+    return true;
 }
 
+/*!
+ * \brief Check whether ACLs are required for a given user
+ *
+ * \param[in]  User name to check
+ *
+ * \return true if the user requires ACLs, false otherwise
+ */
 bool
 pcmk_acl_required(const char *user)
 {
 #if ENABLE_ACL
-    if ((user == NULL) || (*user == '\0')) {
+    if (pcmk__str_empty(user)) {
         crm_trace("ACLs not required because no user set");
-        return FALSE;
+        return false;
 
     } else if (!strcmp(user, CRM_DAEMON_USER) || !strcmp(user, "root")) {
         crm_trace("ACLs not required for privileged user %s", user);
-        return FALSE;
+        return false;
     }
     crm_trace("ACLs required for %s", user);
-    return TRUE;
+    return true;
 #else
     crm_trace("ACLs not required because not supported by this build");
-    return FALSE;
+    return false;
 #endif
 }
 
 #if ENABLE_ACL
 char *
-uid2username(uid_t uid)
+pcmk__uid2username(uid_t uid)
 {
     struct passwd *pwent = getpwuid(uid);
 
@@ -730,15 +718,34 @@ uid2username(uid_t uid)
     return strdup(pwent->pw_name);
 }
 
+/*!
+ * \internal
+ * \brief Set the ACL user field properly on an XML request
+ *
+ * Multiple user names are potentially involved in an XML request: the effective
+ * user of the current process; the user name known from an IPC client
+ * connection; and the user name obtained from the request itself, whether by
+ * the current standard XML attribute name or an older legacy attribute name.
+ * This function chooses the appropriate one that should be used for ACLs, sets
+ * it in the request (using the standard attribute name, and the legacy name if
+ * given), and returns it.
+ *
+ * \param[in,out] request    XML request to update
+ * \param[in]     field      Alternate name for ACL user name XML attribute
+ * \param[in]     peer_user  User name as known from IPC connection
+ *
+ * \return ACL user name actually used
+ */
 const char *
-crm_acl_get_set_user(xmlNode *request, const char *field, const char *peer_user)
+pcmk__update_acl_user(xmlNode *request, const char *field,
+                      const char *peer_user)
 {
     static const char *effective_user = NULL;
     const char *requested_user = NULL;
     const char *user = NULL;
 
     if (effective_user == NULL) {
-        effective_user = uid2username(geteuid());
+        effective_user = pcmk__uid2username(geteuid());
         if (effective_user == NULL) {
             effective_user = strdup("#unprivileged");
             CRM_CHECK(effective_user != NULL, return NULL);
@@ -756,7 +763,7 @@ crm_acl_get_set_user(xmlNode *request, const char *field, const char *peer_user)
         requested_user = crm_element_value(request, field);
     }
 
-    if (is_privileged(effective_user) == FALSE) {
+    if (!pcmk__is_privileged(effective_user)) {
         /* We're not running as a privileged user, set or overwrite any existing
          * value for $XML_ACL_TAG_USER
          */
@@ -772,7 +779,7 @@ crm_acl_get_set_user(xmlNode *request, const char *field, const char *peer_user)
         /* No user known, trusting 'requested_user' */
         user = requested_user;
 
-    } else if (is_privileged(peer_user) == FALSE) {
+    } else if (!pcmk__is_privileged(peer_user)) {
         /* The peer is not a privileged user, set or overwrite any existing
          * value for $XML_ACL_TAG_USER
          */
