@@ -1,19 +1,10 @@
 /*
- * Copyright (C) 2015 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2015-2020 the Pacemaker project contributors
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * The version control history for this file may have further details.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_internal.h>
@@ -22,10 +13,12 @@
 #include <unistd.h>
 #include <string.h>
 #include <stdlib.h>
+#include <md5.h>
 
 #include <crm/crm.h>
 #include <crm/msg_xml.h>
 #include <crm/common/xml.h>
+#include "crmcommon_private.h"
 
 #define BEST_EFFORT_STATUS 0
 
@@ -43,9 +36,9 @@ dump_xml_for_digest(xmlNode * an_xml_node)
     int offset = 0, max = 0;
 
     /* for compatibility with the old result which is used for v1 digests */
-    crm_buffer_add_char(&buffer, &offset, &max, ' ');
-    crm_xml_dump(an_xml_node, 0, &buffer, &offset, &max, 0);
-    crm_buffer_add_char(&buffer, &offset, &max, '\n');
+    pcmk__buffer_add_char(&buffer, &offset, &max, ' ');
+    pcmk__xml2text(an_xml_node, 0, &buffer, &offset, &max, 0);
+    pcmk__buffer_add_char(&buffer, &offset, &max, '\n');
 
     return buffer;
 }
@@ -119,7 +112,8 @@ calculate_xml_digest_v2(xmlNode * source, gboolean do_filter)
          */
 
     } else {
-        crm_xml_dump(source, do_filter ? xml_log_option_filtered : 0, &buffer, &offset, &max, 0);
+        pcmk__xml2text(source, (do_filter? xml_log_option_filtered : 0),
+                       &buffer, &offset, &max, 0);
     }
 
     CRM_ASSERT(buffer != NULL);
@@ -131,7 +125,7 @@ calculate_xml_digest_v2(xmlNode * source, gboolean do_filter)
     }
     if (digest_cs && digest_cs->targets) {
         char *trace_file = crm_strdup_printf("%s/digest-%s",
-                                             crm_get_tmpdir(), digest);
+                                             pcmk__get_tmpdir(), digest);
 
         crm_trace("Saving %s.%s.%s to %s",
                   crm_element_value(source, XML_ATTR_GENERATION_ADMIN),
@@ -215,27 +209,27 @@ calculate_xml_versioned_digest(xmlNode * input, gboolean sort, gboolean do_filte
 
 /*!
  * \internal
- * \brief Return whether calculated digest of XML tree matches expected digest
+ * \brief Check whether calculated digest of given XML matches expected digest
  *
- * \param[in] input Root of XML to digest
- * \param[in] expected Expected digest in on-disk format
+ * \param[in] input     Root of XML tree to digest
+ * \param[in] expected  Expected digest in on-disk format
  *
- * \return TRUE if digests match, FALSE otherwise or on error
+ * \return true if digests match, false on mismatch or error
  */
-gboolean
-crm_digest_verify(xmlNode *input, const char *expected)
+bool
+pcmk__verify_digest(xmlNode *input, const char *expected)
 {
     char *calculated = NULL;
-    gboolean passed;
+    bool passed;
 
     if (input != NULL) {
         calculated = calculate_on_disk_digest(input);
         if (calculated == NULL) {
             crm_perror(LOG_ERR, "Could not calculate digest for comparison");
-            return FALSE;
+            return false;
         }
     }
-    passed = safe_str_eq(expected, calculated);
+    passed = pcmk__str_eq(expected, calculated, pcmk__str_casei);
     if (passed) {
         crm_trace("Digest comparison passed: %s", calculated);
     } else {
@@ -245,3 +239,58 @@ crm_digest_verify(xmlNode *input, const char *expected)
     free(calculated);
     return passed;
 }
+
+/*!
+ * \internal
+ * \brief Check whether an XML attribute should be excluded from CIB digests
+ *
+ * \param[in] name  XML attribute name
+ *
+ * \return true if XML attribute should be excluded from CIB digest calculation
+ */
+bool
+pcmk__xa_filterable(const char *name)
+{
+    static const char *filter[] = {
+        XML_ATTR_ORIGIN,
+        XML_CIB_ATTR_WRITTEN,
+        XML_ATTR_UPDATE_ORIG,
+        XML_ATTR_UPDATE_CLIENT,
+        XML_ATTR_UPDATE_USER,
+    };
+
+    for (int i = 0; i < DIMOF(filter); i++) {
+        if (strcmp(name, filter[i]) == 0) {
+            return true;
+        }
+    }
+    return false;
+}
+
+char *
+crm_md5sum(const char *buffer)
+{
+    int lpc = 0, len = 0;
+    char *digest = NULL;
+    unsigned char raw_digest[MD5_DIGEST_SIZE];
+
+    if (buffer == NULL) {
+        buffer = "";
+    }
+    len = strlen(buffer);
+
+    crm_trace("Beginning digest of %d bytes", len);
+    digest = malloc(2 * MD5_DIGEST_SIZE + 1);
+    if (digest) {
+        md5_buffer(buffer, len, raw_digest);
+        for (lpc = 0; lpc < MD5_DIGEST_SIZE; lpc++) {
+            sprintf(digest + (2 * lpc), "%02x", raw_digest[lpc]);
+        }
+        digest[(2 * MD5_DIGEST_SIZE)] = 0;
+        crm_trace("Digest %s.", digest);
+
+    } else {
+        crm_err("Could not create digest");
+    }
+    return digest;
+}
