@@ -1,5 +1,7 @@
 /*
- * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2020 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -12,6 +14,7 @@
 #endif
 
 #include <stdlib.h>
+#include <string.h>
 #include <signal.h>
 #include <errno.h>
 
@@ -20,7 +23,7 @@
 #include <crm/crm.h>
 #include <crm/common/xml.h>
 #include <crm/common/mainloop.h>
-#include <crm/common/ipcs.h>
+#include <crm/common/ipc_internal.h>
 
 #include <qb/qbarray.h>
 
@@ -110,48 +113,6 @@ crm_trigger_finalize(GSource * source)
     crm_trace("Trigger %p destroyed", source);
 }
 
-#if 0
-struct _GSourceCopy
-{
-  gpointer callback_data;
-  GSourceCallbackFuncs *callback_funcs;
-
-  const GSourceFuncs *source_funcs;
-  guint ref_count;
-
-  GMainContext *context;
-
-  gint priority;
-  guint flags;
-  guint source_id;
-
-  GSList *poll_fds;
-  
-  GSource *prev;
-  GSource *next;
-
-  char    *name;
-
-  void *priv;
-};
-
-static int
-g_source_refcount(GSource * source)
-{
-    /* Duplicating the contents of private header files is a necessary evil */
-    if (source) {
-        struct _GSourceCopy *evil = (struct _GSourceCopy*)source;
-        return evil->ref_count;
-    }
-    return 0;
-}
-#else
-static int g_source_refcount(GSource * source)
-{
-    return 0;
-}
-#endif
-
 static GSourceFuncs crm_trigger_funcs = {
     crm_trigger_prepare,
     crm_trigger_check,
@@ -178,10 +139,7 @@ mainloop_setup_trigger(GSource * source, int priority, int (*dispatch) (gpointer
     g_source_set_priority(source, priority);
     g_source_set_can_recurse(source, FALSE);
 
-    crm_trace("Setup %p with ref-count=%u", source, g_source_refcount(source));
     trigger->id = g_source_attach(source, NULL);
-    crm_trace("Attached %p with ref-count=%u", source, g_source_refcount(source));
-
     return trigger;
 }
 
@@ -228,10 +186,6 @@ mainloop_destroy_trigger(crm_trigger_t * source)
 
     gs = (GSource *)source;
 
-    if(g_source_refcount(gs) > 2) {
-        crm_info("Trigger %p is still referenced %u times", gs, g_source_refcount(gs));
-    }
-
     g_source_destroy(gs); /* Remove from mainloop, ref_count-- */
     g_source_unref(gs); /* The caller no longer carries a reference to source
                          *
@@ -244,15 +198,29 @@ mainloop_destroy_trigger(crm_trigger_t * source)
     return TRUE;
 }
 
-typedef struct signal_s {
-    crm_trigger_t trigger;      /* must be first */
-    void (*handler) (int sig);
-    int signal;
+// Define a custom glib source for signal handling
 
+// Data structure for custom glib source
+typedef struct signal_s {
+    crm_trigger_t trigger;      // trigger that invoked source (must be first)
+    void (*handler) (int sig);  // signal handler
+    int signal;                 // signal that was received
 } crm_signal_t;
 
+// Table to associate signal handlers with signal numbers
 static crm_signal_t *crm_signals[NSIG];
 
+/*!
+ * \internal
+ * \brief Dispatch an event from custom glib source for signals
+ *
+ * Given an signal event, clear the event trigger and call any registered
+ * signal handler.
+ *
+ * \param[in] source    glib source that triggered this dispatch
+ * \param[in] callback  (ignored)
+ * \param[in] userdata  (ignored)
+ */
 static gboolean
 crm_signal_dispatch(GSource * source, GSourceFunc callback, gpointer userdata)
 {
@@ -271,6 +239,15 @@ crm_signal_dispatch(GSource * source, GSourceFunc callback, gpointer userdata)
     return TRUE;
 }
 
+/*!
+ * \internal
+ * \brief Handle a signal by setting a trigger for signal source
+ *
+ * \param[in] sig  Signal number that was received
+ *
+ * \note This is the true signal handler for the mainloop signal source, and
+ *       must be async-safe.
+ */
 static void
 mainloop_signal_handler(int sig)
 {
@@ -279,6 +256,7 @@ mainloop_signal_handler(int sig)
     }
 }
 
+// Functions implementing our custom glib source for signal handling
 static GSourceFuncs crm_signal_funcs = {
     crm_trigger_prepare,
     crm_trigger_check,
@@ -286,16 +264,29 @@ static GSourceFuncs crm_signal_funcs = {
     crm_trigger_finalize,
 };
 
-gboolean
-crm_signal(int sig, void (*dispatch) (int sig))
+/*!
+ * \internal
+ * \brief Set a true signal handler
+ *
+ * signal()-like interface to sigaction()
+ *
+ * \param[in] sig       Signal number to register handler for
+ * \param[in] dispatch  Signal handler
+ *
+ * \return The previous value of the signal handler, or SIG_ERR on error
+ * \note The dispatch function must be async-safe.
+ */
+sighandler_t
+crm_signal_handler(int sig, sighandler_t dispatch)
 {
     sigset_t mask;
     struct sigaction sa;
     struct sigaction old;
 
     if (sigemptyset(&mask) < 0) {
-        crm_perror(LOG_ERR, "Call to sigemptyset failed");
-        return FALSE;
+        crm_err("Could not set handler for signal %d: %s",
+                sig, pcmk_strerror(errno));
+        return SIG_ERR;
     }
 
     memset(&sa, 0, sizeof(struct sigaction));
@@ -304,11 +295,11 @@ crm_signal(int sig, void (*dispatch) (int sig))
     sa.sa_mask = mask;
 
     if (sigaction(sig, &sa, &old) < 0) {
-        crm_perror(LOG_ERR, "Could not install signal handler for signal %d", sig);
-        return FALSE;
+        crm_err("Could not set handler for signal %d: %s",
+                sig, pcmk_strerror(errno));
+        return SIG_ERR;
     }
-
-    return TRUE;
+    return old.sa_handler;
 }
 
 static void
@@ -322,6 +313,17 @@ mainloop_destroy_signal_entry(int sig)
     mainloop_destroy_trigger((crm_trigger_t *) tmp);
 }
 
+/*!
+ * \internal
+ * \brief Add a signal handler to a mainloop
+ *
+ * \param[in] sig       Signal number to handle
+ * \param[in] dispatch  Signal handler function
+ *
+ * \note The true signal handler merely sets a mainloop trigger to call this
+ *       dispatch function via the mainloop. Therefore, the dispatch function
+ *       does not need to be async-safe.
+ */
 gboolean
 mainloop_add_signal(int sig, void (*dispatch) (int sig))
 {
@@ -358,7 +360,7 @@ mainloop_add_signal(int sig, void (*dispatch) (int sig))
     crm_signals[sig]->handler = dispatch;
     crm_signals[sig]->signal = sig;
 
-    if (crm_signal(sig, mainloop_signal_handler) == FALSE) {
+    if (crm_signal_handler(sig, mainloop_signal_handler) == SIG_ERR) {
         mainloop_destroy_signal_entry(sig);
         return FALSE;
     }
@@ -383,7 +385,7 @@ mainloop_destroy_signal(int sig)
         crm_err("Signal %d is out of range", sig);
         return FALSE;
 
-    } else if (crm_signal(sig, NULL) == FALSE) {
+    } else if (crm_signal_handler(sig, NULL) == SIG_ERR) {
         crm_perror(LOG_ERR, "Could not uninstall signal handler for signal %d", sig);
         return FALSE;
 
@@ -449,6 +451,65 @@ gio_poll_destroy(gpointer data)
     }
 }
 
+/*!
+ * \internal
+ * \brief Convert libqb's poll priority into GLib's one
+ *
+ * \param[in] prio  libqb's poll priority (#QB_LOOP_MED assumed as fallback)
+ *
+ * \return  best matching GLib's priority
+ */
+static gint
+conv_prio_libqb2glib(enum qb_loop_priority prio)
+{
+    gint ret = G_PRIORITY_DEFAULT;
+    switch (prio) {
+        case QB_LOOP_LOW:
+            ret = G_PRIORITY_LOW;
+            break;
+        case QB_LOOP_HIGH:
+            ret = G_PRIORITY_HIGH;
+            break;
+        default:
+            crm_trace("Invalid libqb's loop priority %d, assuming QB_LOOP_MED",
+                      prio);
+            /* fall-through */
+        case QB_LOOP_MED:
+            break;
+    }
+    return ret;
+}
+
+/*!
+ * \internal
+ * \brief Convert libqb's poll priority to rate limiting spec
+ *
+ * \param[in] prio  libqb's poll priority (#QB_LOOP_MED assumed as fallback)
+ *
+ * \return  best matching rate limiting spec
+ */
+static enum qb_ipcs_rate_limit
+conv_libqb_prio2ratelimit(enum qb_loop_priority prio)
+{
+    /* this is an inversion of what libqb's qb_ipcs_request_rate_limit does */
+    enum qb_ipcs_rate_limit ret = QB_IPCS_RATE_NORMAL;
+    switch (prio) {
+        case QB_LOOP_LOW:
+            ret = QB_IPCS_RATE_SLOW;
+            break;
+        case QB_LOOP_HIGH:
+            ret = QB_IPCS_RATE_FAST;
+            break;
+        default:
+            crm_trace("Invalid libqb's loop priority %d, assuming QB_LOOP_MED",
+                      prio);
+            /* fall-through */
+        case QB_LOOP_MED:
+            break;
+    }
+    return ret;
+}
+
 static int32_t
 gio_poll_dispatch_update(enum qb_loop_priority p, int32_t fd, int32_t evts,
                          void *data, qb_ipcs_dispatch_fn_t fn, int32_t add)
@@ -495,8 +556,8 @@ gio_poll_dispatch_update(enum qb_loop_priority p, int32_t fd, int32_t evts,
     adaptor->p = p;
     adaptor->is_used++;
     adaptor->source =
-        g_io_add_watch_full(channel, G_PRIORITY_DEFAULT, evts, gio_read_socket, adaptor,
-                            gio_poll_destroy);
+        g_io_add_watch_full(channel, conv_prio_libqb2glib(p), evts,
+                            gio_read_socket, adaptor, gio_poll_destroy);
 
     /* Now that mainloop now holds a reference to channel,
      * thanks to g_io_add_watch_full(), drop ours from g_io_channel_unix_new().
@@ -580,7 +641,15 @@ pick_ipc_type(enum qb_ipc_type requested)
 
 qb_ipcs_service_t *
 mainloop_add_ipc_server(const char *name, enum qb_ipc_type type,
-                        struct qb_ipcs_service_handlers * callbacks)
+                        struct qb_ipcs_service_handlers *callbacks)
+{
+    return mainloop_add_ipc_server_with_prio(name, type, callbacks, QB_LOOP_MED);
+}
+
+qb_ipcs_service_t *
+mainloop_add_ipc_server_with_prio(const char *name, enum qb_ipc_type type,
+                                  struct qb_ipcs_service_handlers *callbacks,
+                                  enum qb_loop_priority prio)
 {
     int rc = 0;
     qb_ipcs_service_t *server = NULL;
@@ -589,9 +658,17 @@ mainloop_add_ipc_server(const char *name, enum qb_ipc_type type,
         gio_map = qb_array_create_2(64, sizeof(struct gio_to_qb_poll), 1);
     }
 
-    crm_client_init();
     server = qb_ipcs_create(name, 0, pick_ipc_type(type), callbacks);
 
+    if (server == NULL) {
+        crm_err("Could not create %s IPC server: %s (%d)", name, pcmk_strerror(rc), rc);
+        return NULL;
+    }
+
+    if (prio != QB_LOOP_MED) {
+        qb_ipcs_request_rate_limit(server, conv_libqb_prio2ratelimit(prio));
+    }
+
 #ifdef HAVE_IPCS_GET_BUFFER_SIZE
     /* All clients should use at least ipc_buffer_max as their buffer size */
     qb_ipcs_enforce_buffer_size(server, crm_ipc_default_buffer_size());
@@ -757,32 +834,66 @@ mainloop_gio_destroy(gpointer c)
     free(c_name);
 }
 
-mainloop_io_t *
-mainloop_add_ipc_client(const char *name, int priority, size_t max_size, void *userdata,
-                        struct ipc_client_callbacks *callbacks)
+/*!
+ * \brief Connect to IPC and add it as a main loop source
+ *
+ * \param[in]  ipc        IPC connection to add
+ * \param[in]  priority   Event source priority to use for connection
+ * \param[in]  userdata   Data to register with callbacks
+ * \param[in]  callbacks  Dispatch and destroy callbacks for connection
+ * \param[out] source     Newly allocated event source
+ *
+ * \return Standard Pacemaker return code
+ *
+ * \note On failure, the caller is still responsible for ipc. On success, the
+ *       caller should call mainloop_del_ipc_client() when source is no longer
+ *       needed, which will lead to the disconnection of the IPC later in the
+ *       main loop if it is connected. However the IPC disconnects,
+ *       mainloop_gio_destroy() will free ipc and source after calling the
+ *       destroy callback.
+ */
+int
+pcmk__add_mainloop_ipc(crm_ipc_t *ipc, int priority, void *userdata,
+                       struct ipc_client_callbacks *callbacks,
+                       mainloop_io_t **source)
 {
-    mainloop_io_t *client = NULL;
-    crm_ipc_t *conn = crm_ipc_new(name, max_size);
+    CRM_CHECK((ipc != NULL) && (callbacks != NULL), return EINVAL);
 
-    if (conn && crm_ipc_connect(conn)) {
-        int32_t fd = crm_ipc_get_fd(conn);
-
-        client = mainloop_add_fd(name, priority, fd, userdata, NULL);
+    if (!crm_ipc_connect(ipc)) {
+        return ENOTCONN;
     }
+    *source = mainloop_add_fd(crm_ipc_name(ipc), priority, crm_ipc_get_fd(ipc),
+                              userdata, NULL);
+    if (*source == NULL) {
+        int rc = errno;
 
-    if (client == NULL) {
-        crm_perror(LOG_TRACE, "Connection to %s failed", name);
-        if (conn) {
-            crm_ipc_close(conn);
-            crm_ipc_destroy(conn);
+        crm_ipc_close(ipc);
+        return rc;
+    }
+    (*source)->ipc = ipc;
+    (*source)->destroy_fn = callbacks->destroy;
+    (*source)->dispatch_fn_ipc = callbacks->dispatch;
+    return pcmk_rc_ok;
+}
+
+mainloop_io_t *
+mainloop_add_ipc_client(const char *name, int priority, size_t max_size,
+                        void *userdata, struct ipc_client_callbacks *callbacks)
+{
+    crm_ipc_t *ipc = crm_ipc_new(name, max_size);
+    mainloop_io_t *source = NULL;
+    int rc = pcmk__add_mainloop_ipc(ipc, priority, userdata, callbacks,
+                                    &source);
+
+    if (rc != pcmk_rc_ok) {
+        if (crm_log_level == LOG_STDOUT) {
+            fprintf(stderr, "Connection to %s failed: %s",
+                    name, pcmk_rc_str(rc));
         }
+        crm_ipc_destroy(ipc);
         return NULL;
     }
-
-    client->ipc = conn;
-    client->destroy_fn = callbacks->destroy;
-    client->dispatch_fn_ipc = callbacks->dispatch;
-    return client;
+    return source;
 }
 
 void
@@ -938,7 +1049,7 @@ child_timeout_callback(gpointer p)
     }
 
     rc = child_kill_helper(child);
-    if (rc == ESRCH) {
+    if (rc == -ESRCH) {
         /* Nothing left to do. pid doesn't exist */
         return FALSE;
     }
@@ -950,7 +1061,7 @@ child_timeout_callback(gpointer p)
     return FALSE;
 }
 
-static gboolean
+static bool
 child_waitpid(mainloop_child_t *child, int flags)
 {
     int rc = 0;
@@ -958,65 +1069,73 @@ child_waitpid(mainloop_child_t *child, int flags)
     int signo = 0;
     int status = 0;
     int exitcode = 0;
+    bool callback_needed = true;
 
     rc = waitpid(child->pid, &status, flags);
-    if(rc == 0) {
-        crm_perror(LOG_DEBUG, "wait(%d) = %d", child->pid, rc);
-        return FALSE;
+    if (rc == 0) { // WNOHANG in flags, and child status is not available
+        crm_trace("Child process %d (%s) still active",
+                  child->pid, child->desc);
+        callback_needed = false;
 
-    } else if(rc != child->pid) {
+    } else if (rc != child->pid) {
+        /* According to POSIX, possible conditions:
+         * - child->pid was non-positive (process group or any child),
+         *   and rc is specific child
+         * - errno ECHILD (pid does not exist or is not child)
+         * - errno EINVAL (invalid flags)
+         * - errno EINTR (caller interrupted by signal)
+         *
+         * @TODO Handle these cases more specifically.
+         */
         signo = SIGCHLD;
         exitcode = 1;
-        status = 1;
-        crm_perror(LOG_ERR, "Call to waitpid(%d) failed", child->pid);
+        crm_notice("Wait for child process %d (%s) interrupted: %s",
+                   child->pid, child->desc, pcmk_strerror(errno));
 
-    } else {
-        crm_trace("Managed process %d exited: %p", child->pid, child);
+    } else if (WIFEXITED(status)) {
+        exitcode = WEXITSTATUS(status);
+        crm_trace("Child process %d (%s) exited with status %d",
+                  child->pid, child->desc, exitcode);
 
-        if (WIFEXITED(status)) {
-            exitcode = WEXITSTATUS(status);
-            crm_trace("Managed process %d (%s) exited with rc=%d", child->pid, child->desc, exitcode);
+    } else if (WIFSIGNALED(status)) {
+        signo = WTERMSIG(status);
+        crm_trace("Child process %d (%s) exited with signal %d (%s)",
+                  child->pid, child->desc, signo, strsignal(signo));
 
-        } else if (WIFSIGNALED(status)) {
-            signo = WTERMSIG(status);
-            crm_trace("Managed process %d (%s) exited with signal=%d", child->pid, child->desc, signo);
-        }
-#ifdef WCOREDUMP
-        if (WCOREDUMP(status)) {
-            core = 1;
-            crm_err("Managed process %d (%s) dumped core", child->pid, child->desc);
-        }
+#ifdef WCOREDUMP // AIX, SunOS, maybe others
+    } else if (WCOREDUMP(status)) {
+        core = 1;
+        crm_err("Child process %d (%s) dumped core",
+                child->pid, child->desc);
 #endif
+
+    } else { // flags must contain WUNTRACED and/or WCONTINUED to reach this
+        crm_trace("Child process %d (%s) stopped or continued",
+                  child->pid, child->desc);
+        callback_needed = false;
     }
 
-    if (child->callback) {
+    if (callback_needed && child->callback) {
         child->callback(child, child->pid, core, signo, exitcode);
     }
-    return TRUE;
+    return callback_needed;
 }
 
 static void
 child_death_dispatch(int signal)
 {
-    GListPtr iter = child_list;
-    gboolean exited;
-
-    while(iter) {
-        GListPtr saved = NULL;
+    for (GList *iter = child_list; iter; ) {
+        GList *saved = iter;
         mainloop_child_t *child = iter->data;
-        exited = child_waitpid(child, WNOHANG);
 
-        saved = iter;
         iter = iter->next;
-
-        if (exited == FALSE) {
-            continue;
+        if (child_waitpid(child, WNOHANG)) {
+            crm_trace("Removing completed process %d from child list",
+                      child->pid);
+            child_list = g_list_remove_link(child_list, saved);
+            g_list_free(saved);
+            child_free(child);
         }
-        crm_trace("Removing process entry %p for %d", child, child->pid);
-
-        child_list = g_list_remove_link(child_list, saved);
-        g_list_free(saved);
-        child_free(child);
     }
 }
 
@@ -1032,7 +1151,7 @@ child_signal_init(gpointer p)
     return FALSE;
 }
 
-int
+gboolean
 mainloop_child_kill(pid_t pid)
 {
     GListPtr iter;
@@ -1055,16 +1174,13 @@ mainloop_child_kill(pid_t pid)
 
     rc = child_kill_helper(match);
     if(rc == -ESRCH) {
-        /* It's gone, but hasn't shown up in waitpid() yet
-         *
-         * Wait until we get SIGCHLD and let child_death_dispatch()
-         * clean it up as normal (so we get the correct return
-         * code/status)
-         *
-         * The blocking alternative would be to call:
-         *    child_waitpid(match, 0);
+        /* It's gone, but hasn't shown up in waitpid() yet. Wait until we get
+         * SIGCHLD and let handler clean it up as normal (so we get the correct
+         * return code/status). The blocking alternative would be to call
+         * child_waitpid(match, 0).
          */
-        crm_trace("Waiting for child %d to be reaped by child_death_dispatch()", match->pid);
+        crm_trace("Waiting for signal that child process %d completed",
+                  match->pid);
         return TRUE;
 
     } else if(rc != 0) {
@@ -1074,7 +1190,7 @@ mainloop_child_kill(pid_t pid)
         waitflags = WNOHANG;
     }
 
-    if (child_waitpid(match, waitflags) == FALSE) {
+    if (!child_waitpid(match, waitflags)) {
         /* not much we can do if this occurs */
         return FALSE;
     }
@@ -1086,6 +1202,10 @@ mainloop_child_kill(pid_t pid)
 
 /* Create/Log a new tracked process
  * To track a process group, use -pid
+ *
+ * @TODO Using a non-positive pid (i.e. any child, or process group) would
+ *       likely not be useful since we will free the child after the first
+ *       completed process.
  */
 void
 mainloop_child_add_with_flags(pid_t pid, int timeout, const char *desc, void *privatedata, enum mainloop_child_flags flags, 
@@ -1137,7 +1257,8 @@ struct mainloop_timer_s {
         void *userdata;
 };
 
-static gboolean mainloop_timer_cb(gpointer user_data)
+static gboolean
+mainloop_timer_cb(gpointer user_data)
 {
     int id = 0;
     bool repeat = FALSE;
@@ -1167,7 +1288,8 @@ static gboolean mainloop_timer_cb(gpointer user_data)
     return repeat;
 }
 
-bool mainloop_timer_running(mainloop_timer_t *t)
+bool
+mainloop_timer_running(mainloop_timer_t *t)
 {
     if(t && t->id != 0) {
         return TRUE;
@@ -1175,7 +1297,8 @@ bool mainloop_timer_running(mainloop_timer_t *t)
     return FALSE;
 }
 
-void mainloop_timer_start(mainloop_timer_t *t)
+void
+mainloop_timer_start(mainloop_timer_t *t)
 {
     mainloop_timer_stop(t);
     if(t && t->period_ms > 0) {
@@ -1184,7 +1307,8 @@ void mainloop_timer_start(mainloop_timer_t *t)
     }
 }
 
-void mainloop_timer_stop(mainloop_timer_t *t)
+void
+mainloop_timer_stop(mainloop_timer_t *t)
 {
     if(t && t->id != 0) {
         crm_trace("Stopping timer %s", t->name);
@@ -1193,7 +1317,8 @@ void mainloop_timer_stop(mainloop_timer_t *t)
     }
 }
 
-guint mainloop_timer_set_period(mainloop_timer_t *t, guint period_ms)
+guint
+mainloop_timer_set_period(mainloop_timer_t *t, guint period_ms)
 {
     guint last = 0;
 
@@ -1208,7 +1333,6 @@ guint mainloop_timer_set_period(mainloop_timer_t *t, guint period_ms)
     return last;
 }
 
-
 mainloop_timer_t *
 mainloop_timer_add(const char *name, guint period_ms, bool repeat, GSourceFunc cb, void *userdata)
 {
@@ -1254,6 +1378,28 @@ drain_timeout_cb(gpointer user_data)
     return FALSE;
 }
 
+/*!
+ * \brief Drain some remaining main loop events then quit it
+ *
+ * \param[in] mloop  Main loop to drain and quit
+ * \param[in] n      Drain up to this many pending events
+ */
+void
+pcmk_quit_main_loop(GMainLoop *mloop, unsigned int n)
+{
+    if ((mloop != NULL) && g_main_loop_is_running(mloop)) {
+        GMainContext *ctx = g_main_loop_get_context(mloop);
+
+        /* Drain up to n events in case some memory clean-up is pending
+         * (helpful to reduce noise in valgrind output).
+         */
+        for (int i = 0; (i < n) && g_main_context_pending(ctx); ++i) {
+            g_main_context_dispatch(ctx);
+        }
+        g_main_loop_quit(mloop);
+    }
+}
+
 /*!
  * \brief Process main loop events while a certain condition is met
  *
@@ -1289,3 +1435,17 @@ pcmk_drain_main_loop(GMainLoop *mloop, guint timer_ms, bool (*check)(guint))
         g_source_remove(timer);
     }
 }
+
+// Deprecated functions kept only for backward API compatibility
+gboolean crm_signal(int sig, void (*dispatch) (int sig));
+
+/*
+ * \brief Use crm_signal_handler() instead
+ * \deprecated
+ */
+gboolean
+crm_signal(int sig, void (*dispatch) (int sig))
+{
+    return crm_signal_handler(sig, dispatch) != SIG_ERR;
+}
+
