@@ -1,5 +1,7 @@
 /*
- * Copyright 2004-2019 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2020 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -17,6 +19,7 @@
 #include <crm/crm.h>
 #include <crm/msg_xml.h>
 #include <crm/common/xml.h>
+#include <crm/common/xml_internal.h>
 #include <crm/common/iso8601_internal.h>
 #include "crmcommon_private.h"
 
@@ -293,7 +296,8 @@ pcmk_format_nvpair(const char *name, const char *value, const char *units) {
  */
 char *
 pcmk_format_named_time(const char *name, time_t epoch_time) {
-    const char *now_str = crm_now_string(&epoch_time);
+    const char *now_str = pcmk__epoch2str(&epoch_time);
+
     return crm_strdup_printf("%s=\"%s\"", name, now_str ? now_str : "");
 }
 
@@ -588,7 +592,7 @@ crm_element_value_ll(const xmlNode *data, const char *name, long long *dest)
     value = crm_element_value(data, name);
     if (value) {
         errno = 0;
-        *dest = crm_int_helper(value, NULL);
+        *dest = crm_parse_ll(value, NULL);
         if (errno == 0) {
             return 0;
         }
@@ -611,11 +615,24 @@ int
 crm_element_value_ms(const xmlNode *data, const char *name, guint *dest)
 {
     const char *value = NULL;
+    long long value_ll;
 
     CRM_CHECK(dest != NULL, return -1);
+    *dest = 0;
+
     value = crm_element_value(data, name);
-    *dest = crm_parse_ms(value);
-    return errno? -1 : 0;
+    if (value == NULL) {
+        return pcmk_ok;
+    }
+
+    errno = 0;
+    value_ll = crm_parse_ll(value, NULL);
+    if ((errno != 0) || (value_ll < 0) || (value_ll > G_MAXUINT)) {
+        return -1;
+    }
+
+    *dest = (guint) value_ll;
+    return pcmk_ok;
 }
 
 /*!
@@ -919,8 +936,8 @@ xml2list(xmlNode *parent)
         g_hash_table_insert(nvpair_hash, strdup(p_name), strdup(p_value));
     }
 
-    for (child = __xml_first_child(nvpair_list); child != NULL;
-         child = __xml_next(child)) {
+    for (child = pcmk__xml_first_child(nvpair_list); child != NULL;
+         child = pcmk__xml_next(child)) {
 
         if (strcmp((const char *)child->name, XML_TAG_PARAM) == 0) {
             const char *key = crm_element_value(child, XML_NVPAIR_ATTR_NAME);
