@@ -1,5 +1,5 @@
 /*
- * Copyright 2019 the Pacemaker project contributors
+ * Copyright 2019-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -8,10 +8,11 @@
  */
 
 #include <stdlib.h>
-#include <crm/crm.h>
-#include <crm/common/output.h>
 #include <glib.h>
 
+#include <crm/crm.h>
+#include <crm/common/output_internal.h>
+
 GOptionEntry pcmk__none_output_entries[] = {
     { NULL }
 };
@@ -89,6 +90,11 @@ none_end_list(pcmk__output_t *out) {
     /* This function intentionally left blank */
 }
 
+static bool
+none_is_quiet(pcmk__output_t *out) {
+    return out->quiet;
+}
+
 pcmk__output_t *
 pcmk__mk_none_output(char **argv) {
     pcmk__output_t *retval = calloc(1, sizeof(pcmk__output_t));
@@ -99,7 +105,6 @@ pcmk__mk_none_output(char **argv) {
 
     retval->fmt_name = "none";
     retval->request = argv == NULL ? NULL : g_strjoinv(" ", argv);
-    retval->supports_quiet = true;
 
     retval->init = none_init;
     retval->free_priv = none_free_priv;
@@ -120,5 +125,7 @@ pcmk__mk_none_output(char **argv) {
     retval->increment_list = none_increment_list;
     retval->end_list = none_end_list;
 
+    retval->is_quiet = none_is_quiet;
+
     return retval;
 }
