@@ -1,24 +1,17 @@
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2020 the Pacemaker project contributors
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * The version control history for this file may have further details.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_internal.h>
 #include <stdio.h>
 #include <string.h>
+#include <crm/msg_xml.h>
+#include "crmcommon_private.h"
 
 /*
  * From xpath2.c
@@ -234,25 +227,32 @@ get_xpath_object(const char *xpath, xmlNode * xml_obj, int error_level)
     max = numXpathResults(xpathObj);
 
     if (max < 1) {
-        do_crm_log(error_level, "No match for %s in %s", xpath, crm_str(nodePath));
-        crm_log_xml_explicit(xml_obj, "Unexpected Input");
+        if (error_level < LOG_NEVER) {
+            do_crm_log(error_level, "No match for %s in %s",
+                       xpath, crm_str(nodePath));
+            crm_log_xml_explicit(xml_obj, "Unexpected Input");
+        }
 
     } else if (max > 1) {
-        int lpc = 0;
+        if (error_level < LOG_NEVER) {
+            int lpc = 0;
 
-        do_crm_log(error_level, "Too many matches for %s in %s", xpath, crm_str(nodePath));
+            do_crm_log(error_level, "Too many matches for %s in %s",
+                       xpath, crm_str(nodePath));
 
-        for (lpc = 0; lpc < max; lpc++) {
-            xmlNode *match = getXpathResult(xpathObj, lpc);
+            for (lpc = 0; lpc < max; lpc++) {
+                xmlNode *match = getXpathResult(xpathObj, lpc);
 
-            CRM_LOG_ASSERT(match != NULL);
-            if(match != NULL) {
-                matchNodePath = (char *)xmlGetNodePath(match);
-                do_crm_log(error_level, "%s[%d] = %s", xpath, lpc, crm_str(matchNodePath));
-                free(matchNodePath);
+                CRM_LOG_ASSERT(match != NULL);
+                if (match != NULL) {
+                    matchNodePath = (char *) xmlGetNodePath(match);
+                    do_crm_log(error_level, "%s[%d] = %s",
+                               xpath, lpc, crm_str(matchNodePath));
+                    free(matchNodePath);
+                }
             }
+            crm_log_xml_explicit(xml_obj, "Bad Input");
         }
-        crm_log_xml_explicit(xml_obj, "Bad Input");
 
     } else {
         result = getXpathResult(xpathObj, 0);
@@ -263,3 +263,37 @@ get_xpath_object(const char *xpath, xmlNode * xml_obj, int error_level)
 
     return result;
 }
+
+int
+pcmk__element_xpath(const char *prefix, xmlNode *xml, char *buffer,
+                    int offset, size_t buffer_size)
+{
+    const char *id = ID(xml);
+
+    if(offset == 0 && prefix == NULL && xml->parent) {
+        offset = pcmk__element_xpath(NULL, xml->parent, buffer, offset,
+                                     buffer_size);
+    }
+
+    if(id) {
+        offset += snprintf(buffer + offset, buffer_size - offset,
+                           "/%s[@id='%s']", (const char *) xml->name, id);
+    } else if(xml->name) {
+        offset += snprintf(buffer + offset, buffer_size - offset,
+                           "/%s", (const char *) xml->name);
+    }
+
+    return offset;
+}
+
+char *
+xml_get_path(xmlNode *xml)
+{
+    int offset = 0;
+    char buffer[PCMK__BUFFER_SIZE];
+
+    if (pcmk__element_xpath(NULL, xml, buffer, offset, sizeof(buffer)) > 0) {
+        return strdup(buffer);
+    }
+    return NULL;
+}
