@@ -1,5 +1,7 @@
 /*
- * Copyright 2010-2019 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2010-2020 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -34,6 +36,10 @@
 #  include <systemd.h>
 #endif
 
+#if SUPPORT_NAGIOS
+#  include <services_nagios.h>
+#endif
+
 /* TODO: Develop a rollover strategy */
 
 static int operations = 0;
@@ -100,9 +106,9 @@ init_recurring_actions(void)
 static inline gboolean
 inflight_systemd_or_upstart(svc_action_t *op)
 {
-    return (safe_str_eq(op->standard, PCMK_RESOURCE_CLASS_SYSTEMD)
-            || safe_str_eq(op->standard, PCMK_RESOURCE_CLASS_UPSTART))
-            && (g_list_find(inflight_ops, op) != NULL);
+    return pcmk__strcase_any_of(op->standard, PCMK_RESOURCE_CLASS_SYSTEMD,
+                           PCMK_RESOURCE_CLASS_UPSTART, NULL) &&
+           g_list_find(inflight_ops, op) != NULL;
 }
 
 /*!
@@ -169,28 +175,29 @@ resources_action_create(const char *name, const char *standard,
      * build the svc_action_t instance.
      */
 
-    if (crm_strlen_zero(name)) {
+    if (pcmk__str_empty(name)) {
         crm_err("Cannot create operation without resource name");
         goto return_error;
     }
 
-    if (crm_strlen_zero(standard)) {
+    if (pcmk__str_empty(standard)) {
         crm_err("Cannot create operation for %s without resource class", name);
         goto return_error;
     }
     ra_caps = pcmk_get_ra_caps(standard);
 
-    if (is_set(ra_caps, pcmk_ra_cap_provider) && crm_strlen_zero(provider)) {
+    if (pcmk_is_set(ra_caps, pcmk_ra_cap_provider)
+        && pcmk__str_empty(provider)) {
         crm_err("Cannot create operation for %s without provider", name);
         goto return_error;
     }
 
-    if (crm_strlen_zero(agent)) {
+    if (pcmk__str_empty(agent)) {
         crm_err("Cannot create operation for %s without agent name", name);
         goto return_error;
     }
 
-    if (crm_strlen_zero(action)) {
+    if (pcmk__str_empty(action)) {
         crm_err("Cannot create operation for %s without operation name", name);
         goto return_error;
     }
@@ -208,19 +215,21 @@ resources_action_create(const char *name, const char *standard,
     op->agent = strdup(agent);
     op->sequence = ++operations;
     op->flags = flags;
-    op->id = generate_op_key(name, action, interval_ms);
+    op->id = pcmk__op_key(name, action, interval_ms);
+
+    if (pcmk_is_set(ra_caps, pcmk_ra_cap_status)
+        && pcmk__str_eq(action, "monitor", pcmk__str_casei)) {
 
-    if (is_set(ra_caps, pcmk_ra_cap_status) && safe_str_eq(action, "monitor")) {
         op->action = strdup("status");
     } else {
         op->action = strdup(action);
     }
 
-    if (is_set(ra_caps, pcmk_ra_cap_provider)) {
+    if (pcmk_is_set(ra_caps, pcmk_ra_cap_provider)) {
         op->provider = strdup(provider);
     }
 
-    if (is_set(ra_caps, pcmk_ra_cap_params)) {
+    if (pcmk_is_set(ra_caps, pcmk_ra_cap_params)) {
         op->params = params;
         params = NULL; // so we don't free them in this function
     }
@@ -249,7 +258,7 @@ resources_action_create(const char *name, const char *standard,
         op->opaque->exec = dup_file_path(op->agent, NAGIOS_PLUGIN_DIR);
         op->opaque->args[0] = strdup(op->opaque->exec);
 
-        if (safe_str_eq(op->action, "monitor") && (op->interval_ms == 0)) {
+        if (pcmk__str_eq(op->action, "monitor", pcmk__str_casei) && (op->interval_ms == 0)) {
             /* Invoke --version for a nagios probe */
             op->opaque->args[1] = strdup("--version");
 
@@ -265,7 +274,7 @@ resources_action_create(const char *name, const char *standard,
             while (g_hash_table_iter_next(&iter, (gpointer *) & key, (gpointer *) & value) &&
                    index <= args_size - 3) {
 
-                if (safe_str_eq(key, XML_ATTR_CRM_VERSION) || strstr(key, CRM_META "_")) {
+                if (pcmk__str_eq(key, XML_ATTR_CRM_VERSION, pcmk__str_casei) || strstr(key, CRM_META "_")) {
                     continue;
                 }
                 op->opaque->args[index++] = crm_strdup_printf("--%s", key);
@@ -541,7 +550,7 @@ gboolean
 services_action_cancel(const char *name, const char *action, guint interval_ms)
 {
     gboolean cancelled = FALSE;
-    char *id = generate_op_key(name, action, interval_ms);
+    char *id = pcmk__op_key(name, action, interval_ms);
     svc_action_t *op = NULL;
 
     /* We can only cancel a recurring action */
@@ -564,11 +573,11 @@ services_action_cancel(const char *name, const char *action, guint interval_ms)
      * goes away.
      */
     if (op->pid != 0) {
-        crm_info("Terminating in-flight op %s (pid %d) early because it was cancelled",
+        crm_info("Terminating in-flight op %s[%d] early because it was cancelled",
                  id, op->pid);
         cancelled = mainloop_child_kill(op->pid);
         if (cancelled == FALSE) {
-            crm_err("Termination of %s (pid %d) failed", id, op->pid);
+            crm_err("Termination of %s[%d] failed", id, op->pid);
         }
         goto done;
     }
@@ -608,7 +617,7 @@ gboolean
 services_action_kick(const char *name, const char *action, guint interval_ms)
 {
     svc_action_t * op = NULL;
-    char *id = generate_op_key(name, action, interval_ms);
+    char *id = pcmk__op_key(name, action, interval_ms);
 
     init_recurring_actions();
     op = g_hash_table_lookup(recurring_actions, id);
@@ -750,7 +759,7 @@ services_action_async_fork_notify(svc_action_t * op,
         g_hash_table_replace(recurring_actions, op->id, op);
     }
 
-    if (is_not_set(op->flags, SVC_ACTION_NON_BLOCKED)
+    if (!pcmk_is_set(op->flags, SVC_ACTION_NON_BLOCKED)
         && op->rsc && is_op_blocked(op->rsc)) {
         blocked_ops = g_list_append(blocked_ops, op);
         return TRUE;
@@ -776,7 +785,7 @@ is_op_blocked(const char *rsc)
 
     for (gIter = inflight_ops; gIter != NULL; gIter = gIter->next) {
         op = gIter->data;
-        if (safe_str_eq(op->rsc, rsc)) {
+        if (pcmk__str_eq(op->rsc, rsc, pcmk__str_casei)) {
             return TRUE;
         }
     }
@@ -825,57 +834,6 @@ handle_blocked_ops(void)
     processing_blocked_ops = FALSE;
 }
 
-#if SUPPORT_NAGIOS
-static int
-nagios_get_metadata(const char *type, char **output)
-{
-    int rc = pcmk_ok;
-    FILE *file_strm = NULL;
-    int start = 0, length = 0, read_len = 0;
-    char *metadata_file = crm_strdup_printf("%s/%s.xml",
-                                            NAGIOS_METADATA_DIR, type);
-
-    file_strm = fopen(metadata_file, "r");
-    if (file_strm == NULL) {
-        crm_err("Metadata file %s does not exist", metadata_file);
-        free(metadata_file);
-        return -EIO;
-    }
-
-    /* see how big the file is */
-    start = ftell(file_strm);
-    fseek(file_strm, 0L, SEEK_END);
-    length = ftell(file_strm);
-    fseek(file_strm, 0L, start);
-
-    CRM_ASSERT(length >= 0);
-    CRM_ASSERT(start == ftell(file_strm));
-
-    if (length <= 0) {
-        crm_info("%s was not valid", metadata_file);
-        free(*output);
-        *output = NULL;
-        rc = -EIO;
-
-    } else {
-        crm_trace("Reading %d bytes from file", length);
-        *output = calloc(1, (length + 1));
-        read_len = fread(*output, 1, length, file_strm);
-        if (read_len != length) {
-            crm_err("Calculated and read bytes differ: %d vs. %d",
-                    length, read_len);
-            free(*output);
-            *output = NULL;
-            rc = -EIO;
-        }
-    }
-
-    fclose(file_strm);
-    free(metadata_file);
-    return rc;
-}
-#endif
-
 static gboolean
 action_get_metadata(svc_action_t *op)
 {
@@ -902,13 +860,13 @@ action_get_metadata(svc_action_t *op)
         return FALSE;
     }
 
-    if (safe_str_eq(class, PCMK_RESOURCE_CLASS_LSB)) {
+    if (pcmk__str_eq(class, PCMK_RESOURCE_CLASS_LSB, pcmk__str_casei)) {
         return (services__get_lsb_metadata(op->agent, &op->stdout_data) >= 0);
     }
 
 #if SUPPORT_NAGIOS
-    if (safe_str_eq(class, PCMK_RESOURCE_CLASS_NAGIOS)) {
-        return (nagios_get_metadata(op->agent, &op->stdout_data) >= 0);
+    if (pcmk__str_eq(class, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)) {
+        return services__get_nagios_metadata(op->agent, &op->stdout_data) >= 0;
     }
 #endif
 
@@ -927,7 +885,7 @@ services_action_sync(svc_action_t * op)
 
     op->synchronous = true;
 
-    if (safe_str_eq(op->action, "meta-data")) {
+    if (pcmk__str_eq(op->action, "meta-data", pcmk__str_casei)) {
         /* Synchronous meta-data operations are handled specially. Since most
          * resource classes don't provide any meta-data, it has to be
          * synthesized from available information about the agent.
@@ -939,7 +897,7 @@ services_action_sync(svc_action_t * op)
     } else {
         rc = action_exec_helper(op);
     }
-    crm_trace(" > " CRM_OP_FMT ": %s = %d",
+    crm_trace(" > " PCMK__OP_FMT ": %s = %d",
               op->rsc, op->action, op->interval_ms, op->opaque->exec, op->rc);
     if (op->stdout_data) {
         crm_trace(" >  stdout: %s", op->stdout_data);
@@ -985,7 +943,7 @@ resources_list_standards(void)
 #endif
 
 #if SUPPORT_NAGIOS
-    agents = resources_os_list_nagios_agents();
+    agents = services__list_nagios_agents();
     if (agents) {
         standards = g_list_append(standards,
                                   strdup(PCMK_RESOURCE_CLASS_NAGIOS));
@@ -999,7 +957,7 @@ resources_list_standards(void)
 GList *
 resources_list_providers(const char *standard)
 {
-    if (is_set(pcmk_get_ra_caps(standard), pcmk_ra_cap_provider)) {
+    if (pcmk_is_set(pcmk_get_ra_caps(standard), pcmk_ra_cap_provider)) {
         return resources_os_list_ocf_providers();
     }
 
@@ -1055,7 +1013,7 @@ resources_list_agents(const char *standard, const char *provider)
 #endif
 #if SUPPORT_NAGIOS
     } else if (strcasecmp(standard, PCMK_RESOURCE_CLASS_NAGIOS) == 0) {
-        return resources_os_list_nagios_agents();
+        return services__list_nagios_agents();
 #endif
     }
 
@@ -1073,7 +1031,7 @@ resources_agent_exists(const char *standard, const char *provider, const char *a
 
     standards = resources_list_standards();
     for (iter = standards; iter != NULL; iter = iter->next) {
-        if (crm_str_eq(iter->data, standard, TRUE)) {
+        if (pcmk__str_eq(iter->data, standard, pcmk__str_none)) {
             rc = TRUE;
             break;
         }
@@ -1085,11 +1043,11 @@ resources_agent_exists(const char *standard, const char *provider, const char *a
 
     rc = FALSE;
 
-    has_providers = is_set(pcmk_get_ra_caps(standard), pcmk_ra_cap_provider);
+    has_providers = pcmk_is_set(pcmk_get_ra_caps(standard), pcmk_ra_cap_provider);
     if (has_providers == TRUE && provider != NULL) {
         providers = resources_list_providers(standard);
         for (iter = providers; iter != NULL; iter = iter->next) {
-            if (crm_str_eq(iter->data, provider, TRUE)) {
+            if (pcmk__str_eq(iter->data, provider, pcmk__str_none)) {
                 rc = TRUE;
                 break;
             }
@@ -1102,7 +1060,7 @@ resources_agent_exists(const char *standard, const char *provider, const char *a
         goto done;
     }
 
-    if (safe_str_eq(standard, PCMK_RESOURCE_CLASS_SERVICE)) {
+    if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_SERVICE, pcmk__str_casei)) {
         if (services__lsb_agent_exists(agent)) {
             rc = TRUE;
 #if SUPPORT_SYSTEMD
@@ -1118,24 +1076,24 @@ resources_agent_exists(const char *standard, const char *provider, const char *a
             rc = FALSE;
         }
 
-    } else if (safe_str_eq(standard, PCMK_RESOURCE_CLASS_OCF)) {
+    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_OCF, pcmk__str_casei)) {
         rc = services__ocf_agent_exists(provider, agent);
 
-    } else if (safe_str_eq(standard, PCMK_RESOURCE_CLASS_LSB)) {
+    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_LSB, pcmk__str_casei)) {
         rc = services__lsb_agent_exists(agent);
 
 #if SUPPORT_SYSTEMD
-    } else if (safe_str_eq(standard, PCMK_RESOURCE_CLASS_SYSTEMD)) {
+    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_SYSTEMD, pcmk__str_casei)) {
         rc = systemd_unit_exists(agent);
 #endif
 
 #if SUPPORT_UPSTART
-    } else if (safe_str_eq(standard, PCMK_RESOURCE_CLASS_UPSTART)) {
+    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_UPSTART, pcmk__str_casei)) {
         rc = upstart_job_exists(agent);
 #endif
 
 #if SUPPORT_NAGIOS
-    } else if (safe_str_eq(standard, PCMK_RESOURCE_CLASS_NAGIOS)) {
+    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)) {
         rc = services__nagios_agent_exists(agent);
 #endif
 
