@@ -1,19 +1,8 @@
 /*
- * Copyright (C) 2010 - 2011, Red Hat, Inc.
+ * Copyright 2010-2018 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
 #ifndef SERVICES_PRIVATE__H
@@ -36,12 +25,15 @@ struct svc_action_private_s {
 
     guint repeat_timer;
     void (*callback) (svc_action_t * op);
+    void (*fork_callback) (svc_action_t * op);
 
     int stderr_fd;
     mainloop_io_t *stderr_gsource;
 
     int stdout_fd;
     mainloop_io_t *stdout_gsource;
+
+    int stdin_fd;
 #if SUPPORT_DBUS
     DBusPendingCall* pending;
     unsigned timerid;
@@ -54,9 +46,6 @@ GList *services_os_get_directory_list(const char *root, gboolean files, gboolean
 G_GNUC_INTERNAL
 gboolean services_os_action_execute(svc_action_t * op);
 
-G_GNUC_INTERNAL
-GList *resources_os_list_lsb_agents(void);
-
 G_GNUC_INTERNAL
 GList *resources_os_list_ocf_providers(void);
 
@@ -64,7 +53,7 @@ G_GNUC_INTERNAL
 GList *resources_os_list_ocf_agents(const char *provider);
 
 G_GNUC_INTERNAL
-GList *resources_os_list_nagios_agents(void);
+gboolean services__ocf_agent_exists(const char *provider, const char *agent);
 
 G_GNUC_INTERNAL
 gboolean cancel_recurring_action(svc_action_t * op);
