@@ -1,5 +1,7 @@
 /*
- * Copyright 2012-2018 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2012-2020 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -100,7 +102,7 @@ static gboolean
 systemd_init(void)
 {
     static int need_init = 1;
-    /* http://dbus.freedesktop.org/doc/api/html/group__DBusConnection.html */
+    // https://dbus.freedesktop.org/doc/api/html/group__DBusConnection.html
 
     if (systemd_proxy
         && dbus_connection_get_is_connected(systemd_proxy) == FALSE) {
@@ -234,7 +236,7 @@ systemd_mask_error(svc_action_t *op, const char *error)
        || strstr(error, "org.freedesktop.systemd1.LoadFailed")
        || strstr(error, "org.freedesktop.systemd1.NoSuchUnit")) {
 
-        if (safe_str_eq(op->action, "stop")) {
+        if (pcmk__str_eq(op->action, "stop", pcmk__str_casei)) {
             crm_trace("Masking %s failure for %s: unknown services are stopped", op->action, op->rsc);
             op->rc = PCMK_OCF_OK;
             return TRUE;
@@ -263,7 +265,12 @@ systemd_loadunit_result(DBusMessage *reply, svc_action_t * op)
         }
         dbus_error_free(&error);
 
-    } else if(pcmk_dbus_type_check(reply, NULL, DBUS_TYPE_OBJECT_PATH, __FUNCTION__, __LINE__)) {
+    } else if (!pcmk_dbus_type_check(reply, NULL, DBUS_TYPE_OBJECT_PATH,
+                                     __func__, __LINE__)) {
+        crm_err("Could not load systemd unit %s for %s: "
+                "systemd reply has unexpected type", op->agent, op->id);
+
+    } else {
         dbus_message_get_args (reply, NULL,
                                DBUS_TYPE_OBJECT_PATH, &path,
                                DBUS_TYPE_INVALID);
@@ -358,6 +365,31 @@ systemd_unit_by_name(const gchar * arg_name, svc_action_t *op)
     return NULL;
 }
 
+/*!
+ * \internal
+ * \brief Compare two strings alphabetically (case-insensitive)
+ *
+ * \param[in] a  First string to compare
+ * \param[in] b  Second string to compare
+ *
+ * \return 0 if strings are equal, -1 if a < b, 1 if a > b
+ *
+ * \note Usable as a GCompareFunc with g_list_sort().
+ *       NULL is considered less than non-NULL.
+ */
+static gint
+sort_str(gconstpointer a, gconstpointer b)
+{
+    if (!a && !b) {
+        return 0;
+    } else if (!a) {
+        return -1;
+    } else if (!b) {
+        return 1;
+    }
+    return strcasecmp(a, b);
+}
+
 GList *
 systemd_unit_listall(void)
 {
@@ -388,7 +420,7 @@ systemd_unit_listall(void)
         return NULL;
     }
     if (!pcmk_dbus_type_check(reply, &args, DBUS_TYPE_ARRAY,
-                              __FUNCTION__, __LINE__)) {
+                              __func__, __LINE__)) {
         crm_err("Could not list systemd unit files: systemd reply has invalid arguments");
         dbus_message_unref(reply);
         return NULL;
@@ -403,14 +435,14 @@ systemd_unit_listall(void)
         char *unit_name = NULL;
         char *basename = NULL;
 
-        if(!pcmk_dbus_type_check(reply, &unit, DBUS_TYPE_STRUCT, __FUNCTION__, __LINE__)) {
-            crm_debug("ListUnitFiles reply has unexpected type");
+        if(!pcmk_dbus_type_check(reply, &unit, DBUS_TYPE_STRUCT, __func__, __LINE__)) {
+            crm_warn("Skipping systemd reply argument with unexpected type");
             continue;
         }
 
         dbus_message_iter_recurse(&unit, &elem);
-        if(!pcmk_dbus_type_check(reply, &elem, DBUS_TYPE_STRING, __FUNCTION__, __LINE__)) {
-            crm_debug("ListUnitFiles reply does not contain a string");
+        if(!pcmk_dbus_type_check(reply, &elem, DBUS_TYPE_STRING, __func__, __LINE__)) {
+            crm_warn("Skipping systemd reply argument with no string");
             continue;
         }
 
@@ -451,7 +483,7 @@ systemd_unit_listall(void)
     dbus_message_unref(reply);
 
     crm_trace("Found %d manageable systemd unit files", nfiles);
-    units = g_list_sort(units, crm_alpha_sort);
+    units = g_list_sort(units, sort_str);
     return units;
 }
 
@@ -524,7 +556,7 @@ systemd_exec_result(DBusMessage *reply, svc_action_t *op)
         dbus_error_free(&error);
 
     } else {
-        if(!pcmk_dbus_type_check(reply, NULL, DBUS_TYPE_OBJECT_PATH, __FUNCTION__, __LINE__)) {
+        if(!pcmk_dbus_type_check(reply, NULL, DBUS_TYPE_OBJECT_PATH, __func__, __LINE__)) {
             crm_warn("Call to %s passed but return type was unexpected", op->action);
             op->rc = PCMK_OCF_OK;
 
@@ -696,7 +728,7 @@ systemd_unit_exec_with_unit(svc_action_t * op, const char *unit)
 
     CRM_ASSERT(unit);
 
-    if (safe_str_eq(op->action, "monitor") || safe_str_eq(method, "status")) {
+    if (pcmk__str_eq(op->action, "monitor", pcmk__str_casei) || pcmk__str_eq(method, "status", pcmk__str_casei)) {
         DBusPendingCall *pending = NULL;
         char *state;
 
@@ -805,7 +837,7 @@ systemd_unit_exec(svc_action_t * op)
     crm_debug("Performing %ssynchronous %s op on systemd unit %s named '%s'",
               op->synchronous ? "" : "a", op->action, op->agent, op->rsc);
 
-    if (safe_str_eq(op->action, "meta-data")) {
+    if (pcmk__str_eq(op->action, "meta-data", pcmk__str_casei)) {
         // @TODO Implement an async meta-data call in executor API
         op->stdout_data = systemd_unit_metadata(op->agent, op->timeout);
         op->rc = PCMK_OCF_OK;
