@@ -86,7 +86,7 @@ upstart_job_by_name(const gchar * arg_name, gchar ** out_unit, int timeout)
         crm_err("Could not issue %s for %s: %s", method, arg_name, error.message);
         dbus_error_free(&error);
 
-    } else if(!pcmk_dbus_type_check(reply, NULL, DBUS_TYPE_OBJECT_PATH, __FUNCTION__, __LINE__)) {
+    } else if(!pcmk_dbus_type_check(reply, NULL, DBUS_TYPE_OBJECT_PATH, __func__, __LINE__)) {
         crm_err("Invalid return type for %s", method);
 
     } else {
@@ -186,7 +186,7 @@ upstart_job_listall(void)
         return NULL;
     }
 
-    if(!pcmk_dbus_type_check(reply, &args, DBUS_TYPE_ARRAY, __FUNCTION__, __LINE__)) {
+    if(!pcmk_dbus_type_check(reply, &args, DBUS_TYPE_ARRAY, __func__, __LINE__)) {
         crm_err("Call to %s failed: Message has invalid arguments", method);
         dbus_message_unref(reply);
         return NULL;
@@ -198,7 +198,8 @@ upstart_job_listall(void)
         const char *job = NULL;
         char *path = NULL;
 
-        if(!pcmk_dbus_type_check(reply, &unit, DBUS_TYPE_OBJECT_PATH, __FUNCTION__, __LINE__)) {
+        if(!pcmk_dbus_type_check(reply, &unit, DBUS_TYPE_OBJECT_PATH, __func__, __LINE__)) {
+            crm_warn("Skipping Upstart reply argument with unexpected type");
             continue;
         }
 
@@ -268,13 +269,13 @@ get_first_instance(const gchar * job, int timeout)
         goto done;
     }
 
-    if(!pcmk_dbus_type_check(reply, &args, DBUS_TYPE_ARRAY, __FUNCTION__, __LINE__)) {
+    if(!pcmk_dbus_type_check(reply, &args, DBUS_TYPE_ARRAY, __func__, __LINE__)) {
         crm_err("Call to %s failed: Message has invalid arguments", method);
         goto done;
     }
 
     dbus_message_iter_recurse(&args, &unit);
-    if(pcmk_dbus_type_check(reply, &unit, DBUS_TYPE_OBJECT_PATH, __FUNCTION__, __LINE__)) {
+    if(pcmk_dbus_type_check(reply, &unit, DBUS_TYPE_OBJECT_PATH, __func__, __LINE__)) {
         DBusBasicValue value;
 
         dbus_message_iter_get_basic(&unit, &value);
@@ -341,18 +342,18 @@ upstart_mask_error(svc_action_t *op, const char *error)
 {
     crm_trace("Could not issue %s for %s: %s", op->action, op->rsc, error);
     if(strstr(error, UPSTART_06_API ".Error.UnknownInstance")) {
-        if(safe_str_eq(op->action, "stop")) {
+        if(pcmk__str_eq(op->action, "stop", pcmk__str_casei)) {
             crm_trace("Masking %s failure for %s: unknown services are stopped", op->action, op->rsc);
             op->rc = PCMK_OCF_OK;
 
-        } else if(safe_str_eq(op->action, "start")) {
+        } else if(pcmk__str_eq(op->action, "start", pcmk__str_casei)) {
             crm_trace("Mapping %s failure for %s: unknown services are not installed", op->action, op->rsc);
             op->rc = PCMK_OCF_NOT_INSTALLED;
             op->status = PCMK_LRM_OP_NOT_INSTALLED;
         }
         return TRUE;
 
-    } else if (safe_str_eq(op->action, "start")
+    } else if (pcmk__str_eq(op->action, "start", pcmk__str_casei)
                && strstr(error, UPSTART_06_API ".Error.AlreadyStarted")) {
         crm_trace("Mapping %s failure for %s: starting a started resource is allowed", op->action, op->rsc);
         op->rc = PCMK_OCF_OK;
@@ -387,7 +388,7 @@ upstart_async_dispatch(DBusPendingCall *pending, void *user_data)
         op->rc = PCMK_OCF_OK;
 
     } else {
-        if(!pcmk_dbus_type_check(reply, NULL, DBUS_TYPE_OBJECT_PATH, __FUNCTION__, __LINE__)) {
+        if(!pcmk_dbus_type_check(reply, NULL, DBUS_TYPE_OBJECT_PATH, __func__, __LINE__)) {
             crm_warn("Call to %s passed but return type was unexpected", op->action);
             op->rc = PCMK_OCF_OK;
 
@@ -429,7 +430,7 @@ upstart_job_exec(svc_action_t * op)
     op->rc = PCMK_OCF_UNKNOWN_ERROR;
     CRM_ASSERT(upstart_init());
 
-    if (safe_str_eq(op->action, "meta-data")) {
+    if (pcmk__str_eq(op->action, "meta-data", pcmk__str_casei)) {
         op->stdout_data = upstart_job_metadata(op->agent);
         op->rc = PCMK_OCF_OK;
         goto cleanup;
@@ -447,8 +448,7 @@ upstart_job_exec(svc_action_t * op)
         goto cleanup;
     }
 
-    if (safe_str_eq(op->action, "monitor") || safe_str_eq(action, "status")) {
-
+    if (pcmk__strcase_any_of(op->action, "monitor", "status", NULL)) {
         char *path = get_first_instance(job, op->timeout);
 
         op->rc = PCMK_OCF_NOT_RUNNING;
@@ -532,7 +532,7 @@ upstart_job_exec(svc_action_t * op)
         /* No return vaue */
         op->rc = PCMK_OCF_OK;
 
-    } else if(!pcmk_dbus_type_check(reply, NULL, DBUS_TYPE_OBJECT_PATH, __FUNCTION__, __LINE__)) {
+    } else if(!pcmk_dbus_type_check(reply, NULL, DBUS_TYPE_OBJECT_PATH, __func__, __LINE__)) {
         crm_warn("Call to %s passed but return type was unexpected", op->action);
         op->rc = PCMK_OCF_OK;
 
