@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -8,6 +8,7 @@
  */
 
 #include <crm_resource.h>
+#include <crm/common/xml_internal.h>
 
 #define cons_string(x) x?x:"NA"
 void
@@ -17,8 +18,8 @@ cli_resource_print_cts_constraints(pe_working_set_t * data_set)
     xmlNode *lifetime = NULL;
     xmlNode *cib_constraints = get_object_root(XML_CIB_TAG_CONSTRAINTS, data_set->input);
 
-    for (xml_obj = __xml_first_child_element(cib_constraints); xml_obj != NULL;
-         xml_obj = __xml_next_element(xml_obj)) {
+    for (xml_obj = pcmk__xe_first_child(cib_constraints); xml_obj != NULL;
+         xml_obj = pcmk__xe_next(xml_obj)) {
         const char *id = crm_element_value(xml_obj, XML_ATTR_ID);
 
         if (id == NULL) {
@@ -31,7 +32,7 @@ cli_resource_print_cts_constraints(pe_working_set_t * data_set)
             continue;
         }
 
-        if (safe_str_eq(XML_CONS_TAG_RSC_DEPEND, crm_element_name(xml_obj))) {
+        if (pcmk__str_eq(XML_CONS_TAG_RSC_DEPEND, crm_element_name(xml_obj), pcmk__str_casei)) {
             printf("Constraint %s %s %s %s %s %s %s\n",
                    crm_element_name(xml_obj),
                    cons_string(crm_element_value(xml_obj, XML_ATTR_ID)),
@@ -41,14 +42,14 @@ cli_resource_print_cts_constraints(pe_working_set_t * data_set)
                    cons_string(crm_element_value(xml_obj, XML_COLOC_ATTR_SOURCE_ROLE)),
                    cons_string(crm_element_value(xml_obj, XML_COLOC_ATTR_TARGET_ROLE)));
 
-        } else if (safe_str_eq(XML_CONS_TAG_RSC_LOCATION, crm_element_name(xml_obj))) {
+        } else if (pcmk__str_eq(XML_CONS_TAG_RSC_LOCATION, crm_element_name(xml_obj), pcmk__str_casei)) {
             /* unpack_location(xml_obj, data_set); */
         }
     }
 }
 
 void
-cli_resource_print_cts(resource_t * rsc)
+cli_resource_print_cts(pe_resource_t * rsc)
 {
     GListPtr lpc = NULL;
     const char *host = NULL;
@@ -58,7 +59,7 @@ cli_resource_print_cts(resource_t * rsc)
     const char *rclass = crm_element_value(rsc->xml, XML_AGENT_ATTR_CLASS);
     pe_node_t *node = pe__current_node(rsc);
 
-    if (safe_str_eq(rclass, PCMK_RESOURCE_CLASS_STONITH)) {
+    if (pcmk__str_eq(rclass, PCMK_RESOURCE_CLASS_STONITH, pcmk__str_casei)) {
         needs_quorum = FALSE;
     } else {
         // @TODO check requires in resource meta-data and rsc_defaults
@@ -75,7 +76,7 @@ cli_resource_print_cts(resource_t * rsc)
            rsc->flags);
 
     for (lpc = rsc->children; lpc != NULL; lpc = lpc->next) {
-        resource_t *child = (resource_t *) lpc->data;
+        pe_resource_t *child = (pe_resource_t *) lpc->data;
 
         cli_resource_print_cts(child);
     }
@@ -83,7 +84,7 @@ cli_resource_print_cts(resource_t * rsc)
 
 
 void
-cli_resource_print_raw(resource_t * rsc)
+cli_resource_print_raw(pe_resource_t * rsc)
 {
     GListPtr lpc = NULL;
     GListPtr children = rsc->children;
@@ -93,12 +94,13 @@ cli_resource_print_raw(resource_t * rsc)
     }
 
     for (lpc = children; lpc != NULL; lpc = lpc->next) {
-        resource_t *child = (resource_t *) lpc->data;
+        pe_resource_t *child = (pe_resource_t *) lpc->data;
 
         cli_resource_print_raw(child);
     }
 }
 
+// \return Standard Pacemaker return code
 int
 cli_resource_print_list(pe_working_set_t * data_set, bool raw)
 {
@@ -108,9 +110,9 @@ cli_resource_print_list(pe_working_set_t * data_set, bool raw)
     int opts = pe_print_printf | pe_print_rsconly | pe_print_pending;
 
     for (lpc = data_set->resources; lpc != NULL; lpc = lpc->next) {
-        resource_t *rsc = (resource_t *) lpc->data;
+        pe_resource_t *rsc = (pe_resource_t *) lpc->data;
 
-        if (is_set(rsc->flags, pe_rsc_orphan)
+        if (pcmk_is_set(rsc->flags, pe_rsc_orphan)
             && rsc->fns->active(rsc, TRUE) == FALSE) {
             continue;
         }
@@ -120,17 +122,18 @@ cli_resource_print_list(pe_working_set_t * data_set, bool raw)
 
     if (found == 0) {
         printf("NO resources configured\n");
-        return -ENXIO;
+        return ENXIO;
     }
 
-    return 0;
+    return pcmk_rc_ok;
 }
 
+// \return Standard Pacemaker return code
 int
 cli_resource_print_operations(const char *rsc_id, const char *host_uname, bool active,
                          pe_working_set_t * data_set)
 {
-    resource_t *rsc = NULL;
+    pe_resource_t *rsc = NULL;
     int opts = pe_print_printf | pe_print_rsconly | pe_print_suppres_nl | pe_print_pending;
     GListPtr ops = find_operations(rsc_id, host_uname, active, data_set);
     GListPtr lpc = NULL;
@@ -159,17 +162,17 @@ cli_resource_print_operations(const char *rsc_id, const char *host_uname, bool a
 
         if (crm_element_value_epoch(xml_op, XML_RSC_OP_LAST_CHANGE,
                                     &last_change) == pcmk_ok) {
-            fprintf(stdout, ", last-rc-change=%s, exec=%sms",
+            fprintf(stdout, ", " XML_RSC_OP_LAST_CHANGE "=%s, exec=%sms",
                     crm_strip_trailing_newline(ctime(&last_change)),
                     crm_element_value(xml_op, XML_RSC_OP_T_EXEC));
         }
         fprintf(stdout, "): %s\n", services_lrm_status_str(status));
     }
-    return pcmk_ok;
+    return pcmk_rc_ok;
 }
 
 void
-cli_resource_print_location(resource_t * rsc, const char *prefix)
+cli_resource_print_location(pe_resource_t * rsc, const char *prefix)
 {
     GListPtr lpc = NULL;
     GListPtr list = rsc->rsc_location;
@@ -185,7 +188,7 @@ cli_resource_print_location(resource_t * rsc, const char *prefix)
         GListPtr lpc2 = NULL;
 
         for (lpc2 = cons->node_list_rh; lpc2 != NULL; lpc2 = lpc2->next) {
-            node_t *node = (node_t *) lpc2->data;
+            pe_node_t *node = (pe_node_t *) lpc2->data;
             char *score = score2char(node->weight);
 
             fprintf(stdout, "%s: Node %-*s (score=%s, id=%s)\n",
@@ -196,7 +199,7 @@ cli_resource_print_location(resource_t * rsc, const char *prefix)
 }
 
 void
-cli_resource_print_colocation(resource_t * rsc, bool dependents, bool recursive, int offset)
+cli_resource_print_colocation(pe_resource_t * rsc, bool dependents, bool recursive, int offset)
 {
     char *prefix = NULL;
     GListPtr lpc = NULL;
@@ -209,25 +212,25 @@ cli_resource_print_colocation(resource_t * rsc, bool dependents, bool recursive,
         list = rsc->rsc_cons_lhs;
     }
 
-    if (is_set(rsc->flags, pe_rsc_allocating)) {
+    if (pcmk_is_set(rsc->flags, pe_rsc_allocating)) {
         /* Break colocation loops */
         printf("loop %s\n", rsc->id);
         free(prefix);
         return;
     }
 
-    set_bit(rsc->flags, pe_rsc_allocating);
+    pe__set_resource_flags(rsc, pe_rsc_allocating);
     for (lpc = list; lpc != NULL; lpc = lpc->next) {
         rsc_colocation_t *cons = (rsc_colocation_t *) lpc->data;
 
         char *score = NULL;
-        resource_t *peer = cons->rsc_rh;
+        pe_resource_t *peer = cons->rsc_rh;
 
         if (dependents) {
             peer = cons->rsc_lh;
         }
 
-        if (is_set(peer->flags, pe_rsc_allocating)) {
+        if (pcmk_is_set(peer->flags, pe_rsc_allocating)) {
             if (dependents == FALSE) {
                 fprintf(stdout, "%s%-*s (id=%s - loop)\n", prefix, 80 - (4 * offset), peer->id,
                         cons->id);
@@ -258,8 +261,9 @@ cli_resource_print_colocation(resource_t * rsc, bool dependents, bool recursive,
     free(prefix);
 }
 
+// \return Standard Pacemaker return code
 int
-cli_resource_print(resource_t *rsc, pe_working_set_t *data_set, bool expanded)
+cli_resource_print(pe_resource_t *rsc, pe_working_set_t *data_set, bool expanded)
 {
     char *rsc_xml = NULL;
     int opts = pe_print_printf | pe_print_pending;
@@ -270,17 +274,19 @@ cli_resource_print(resource_t *rsc, pe_working_set_t *data_set, bool expanded)
                                  rsc->orig_xml : rsc->xml);
     fprintf(stdout, "%sxml:\n%s\n", expanded ? "" : "raw ", rsc_xml);
     free(rsc_xml);
-    return 0;
+    return pcmk_rc_ok;
 }
 
+// \return Standard Pacemaker return code
 int
-cli_resource_print_attribute(resource_t *rsc, const char *attr, pe_working_set_t * data_set)
+cli_resource_print_attribute(pe_resource_t *rsc, const char *attr, const char *attr_set_type,
+                             pe_working_set_t * data_set)
 {
-    int rc = -ENXIO;
+    int rc = ENXIO;
     unsigned int count = 0;
     GHashTable *params = NULL;
     const char *value = NULL;
-    node_t *current = pe__find_active_on(rsc, &count, NULL);
+    pe_node_t *current = pe__find_active_on(rsc, &count, NULL);
 
     if (count > 1) {
         CMD_ERR("%s is active on more than one node,"
@@ -290,10 +296,10 @@ cli_resource_print_attribute(resource_t *rsc, const char *attr, pe_working_set_t
 
     params = crm_str_table_new();
 
-    if (safe_str_eq(attr_set_type, XML_TAG_ATTR_SETS)) {
+    if (pcmk__str_eq(attr_set_type, XML_TAG_ATTR_SETS, pcmk__str_casei)) {
         get_rsc_attributes(params, rsc, current, data_set);
 
-    } else if (safe_str_eq(attr_set_type, XML_TAG_META_SETS)) {
+    } else if (pcmk__str_eq(attr_set_type, XML_TAG_META_SETS, pcmk__str_casei)) {
         /* No need to redirect to the parent */
         get_meta_attributes(params, rsc, current, data_set);
 
@@ -306,7 +312,7 @@ cli_resource_print_attribute(resource_t *rsc, const char *attr, pe_working_set_t
     value = g_hash_table_lookup(params, attr);
     if (value != NULL) {
         fprintf(stdout, "%s\n", value);
-        rc = 0;
+        rc = pcmk_rc_ok;
 
     } else {
         CMD_ERR("Attribute '%s' not found for '%s'", attr, rsc->id);
@@ -316,15 +322,15 @@ cli_resource_print_attribute(resource_t *rsc, const char *attr, pe_working_set_t
     return rc;
 }
 
-
+// \return Standard Pacemaker return code
 int
-cli_resource_print_property(resource_t *rsc, const char *attr, pe_working_set_t * data_set)
+cli_resource_print_property(pe_resource_t *rsc, const char *attr, pe_working_set_t * data_set)
 {
     const char *value = crm_element_value(rsc->xml, attr);
 
     if (value != NULL) {
         fprintf(stdout, "%s\n", value);
-        return 0;
+        return pcmk_rc_ok;
     }
-    return -ENXIO;
+    return ENXIO;
 }
