@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2019 the Pacemaker project contributors
+ * Copyright 2012-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -41,14 +41,14 @@ const char *attr_value = NULL;
 const char *attr_id = NULL;
 const char *set_name = NULL;
 const char *attr_default = NULL;
+const char *xml_file = NULL;
 char ticket_cmd = 'S';
-char *xml_file = NULL;
 int cib_options = cib_sync_call;
 
-static ticket_t *
+static pe_ticket_t *
 find_ticket(const char *ticket_id, pe_working_set_t * data_set)
 {
-    ticket_t *ticket = NULL;
+    pe_ticket_t *ticket = NULL;
 
     ticket = g_hash_table_lookup(data_set->tickets, ticket_id);
 
@@ -71,7 +71,7 @@ print_date(time_t time)
 }
 
 static int
-print_ticket(ticket_t * ticket, gboolean raw, gboolean details)
+print_ticket(pe_ticket_t * ticket, gboolean raw, gboolean details)
 {
     if (raw) {
         fprintf(stdout, "%s\n", ticket->id);
@@ -96,8 +96,7 @@ print_ticket(ticket_t * ticket, gboolean raw, gboolean details)
                 fprintf(stdout, ", ");
             }
             fprintf(stdout, "%s=", name);
-            if (crm_str_eq(name, "last-granted", TRUE)
-                || crm_str_eq(name, "expires", TRUE)) {
+            if (pcmk__str_any_of(name, "last-granted", "expires", NULL)) {
                 print_date(crm_parse_int(value, 0));
             } else {
                 fprintf(stdout, "%s", value);
@@ -122,7 +121,7 @@ static int
 print_ticket_list(pe_working_set_t * data_set, gboolean raw, gboolean details)
 {
     GHashTableIter iter;
-    ticket_t *ticket = NULL;
+    pe_ticket_t *ticket = NULL;
 
     g_hash_table_iter_init(&iter, data_set->tickets);
 
@@ -266,7 +265,7 @@ static int
 get_ticket_state_attr(const char *ticket_id, const char *attr_name, const char **attr_value,
                       pe_working_set_t * data_set)
 {
-    ticket_t *ticket = NULL;
+    pe_ticket_t *ticket = NULL;
 
     CRM_ASSERT(attr_value != NULL);
     *attr_value = NULL;
@@ -295,7 +294,7 @@ ticket_warning(const char *ticket_id, const char *action)
     const char *word = NULL;
 
     warning = calloc(1, text_max);
-    if (safe_str_eq(action, "grant")) {
+    if (pcmk__str_eq(action, "grant", pcmk__str_casei)) {
         offset += snprintf(warning + offset, text_max - offset,
                            "This command cannot help you verify whether '%s' has been already granted elsewhere.\n",
                            ticket_id);
@@ -355,7 +354,7 @@ allow_modification(const char *ticket_id, GListPtr attr_delete,
     for(list_iter = attr_delete; list_iter; list_iter = list_iter->next) {
         const char *key = (const char *)list_iter->data;
 
-        if (safe_str_eq(key, "granted")) {
+        if (pcmk__str_eq(key, "granted", pcmk__str_casei)) {
             ticket_warning(ticket_id, "revoke");
             return FALSE;
         }
@@ -379,7 +378,7 @@ modify_ticket_state(const char * ticket_id, GListPtr attr_delete, GHashTable * a
     char *key = NULL;
     char *value = NULL;
 
-    ticket_t *ticket = NULL;
+    pe_ticket_t *ticket = NULL;
 
     rc = find_ticket_state(cib, ticket_id, &ticket_state_xml);
     if (rc == pcmk_ok) {
@@ -413,7 +412,7 @@ modify_ticket_state(const char * ticket_id, GListPtr attr_delete, GHashTable * a
     while (g_hash_table_iter_next(&hash_iter, (gpointer *) & key, (gpointer *) & value)) {
         crm_xml_add(ticket_state_xml, key, value);
 
-        if (safe_str_eq(key, "granted")
+        if (pcmk__str_eq(key, "granted", pcmk__str_casei)
             && (ticket == NULL || ticket->granted == FALSE)
             && crm_is_true(value)) {
 
@@ -424,7 +423,7 @@ modify_ticket_state(const char * ticket_id, GListPtr attr_delete, GHashTable * a
         }
     }
 
-    if (found && g_list_length(attr_delete)) {
+    if (found && (attr_delete != NULL)) {
         crm_log_xml_debug(xml_top, "Replace");
         rc = cib->cmds->replace(cib, XML_CIB_TAG_STATUS, ticket_state_xml, cib_options);
 
@@ -465,79 +464,245 @@ delete_ticket_state(const char *ticket_id, cib_t * cib)
     return rc;
 }
 
-/* *INDENT-OFF* */
-static struct crm_option long_options[] = {
-    /* Top-level Options */
-    {"help",    0, 0, '?', "\t\tThis text"},
-    {"version", 0, 0, '$', "\t\tVersion information"  },
-    {"verbose", 0, 0, 'V', "\t\tIncrease debug output"},
-    {"quiet",   0, 0, 'Q', "\t\tPrint only the value on stdout\n"},
-
-    {"ticket",  1, 0, 't', "\tTicket ID" },
-
-    {"-spacer-",   1, 0, '-', "\nQueries:"},
-    {"info",       0, 0, 'l', "\t\tDisplay the information of ticket(s)"},
-    {"details",    0, 0, 'L', "\t\tDisplay the details of ticket(s)"},
-    {"raw",        0, 0, 'w', "\t\tDisplay the IDs of ticket(s)"},
-    {"query-xml",  0, 0, 'q', "\tQuery the XML of ticket(s)"},
-    {"constraints",0, 0, 'c', "\tDisplay the rsc_ticket constraints that apply to ticket(s)"},
-
-    {"-spacer-",   1, 0, '-', "\nCommands:"},
-    {"grant",      0, 0, 'g', "\t\tGrant a ticket to this cluster site"},
-    {"revoke",     0, 0, 'r', "\t\tRevoke a ticket from this cluster site"},
-    {"standby",    0, 0, 's', "\t\tTell this cluster site this ticket is standby"},
-    {"activate",   0, 0, 'a', "\tTell this cluster site this ticket is active"},
-    
-    {"-spacer-",   1, 0, '-', "\nAdvanced Commands:"},
-    {"get-attr",   1, 0, 'G', "\tDisplay the named attribute for a ticket"},
-    {"set-attr",   1, 0, 'S', "\tSet the named attribute for a ticket"},
-    {"delete-attr",1, 0, 'D', "\tDelete the named attribute for a ticket"},
-    {"cleanup",    0, 0, 'C', "\t\tDelete all state of a ticket at this cluster site"},
-    
-    {"-spacer-",   1, 0, '-', "\nAdditional Options:"},
-    {"attr-value", 1, 0, 'v', "\tAttribute value to use with -S"},
-    {"default",    1, 0, 'd', "\t(Advanced) The default attribute value to display if none is found. For use with -G"},
-    {"force",      0, 0, 'f', "\t\t(Advanced) Force the action to be performed"},
-    {"xml-file",   1, 0, 'x', NULL, 1},
+static pcmk__cli_option_t long_options[] = {
+    // long option, argument type, storage, short option, description, flags
+    {
+        "help", no_argument, NULL, '?',
+        "\t\tThis text", pcmk__option_default
+    },
+    {
+        "version", no_argument, NULL, '$',
+        "\t\tVersion information", pcmk__option_default
+    },
+    {
+        "verbose", no_argument, NULL, 'V',
+        "\t\tIncrease debug output", pcmk__option_default
+    },
+    {
+        "quiet", no_argument, NULL, 'Q',
+        "\t\tPrint only the value on stdout\n", pcmk__option_default
+    },
+    {
+        "ticket", required_argument, NULL, 't',
+        "\tTicket ID", pcmk__option_default
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "\nQueries:", pcmk__option_default
+    },
+    {
+        "info", no_argument, NULL, 'l',
+        "\t\tDisplay the information of ticket(s)", pcmk__option_default
+    },
+    {
+        "details", no_argument, NULL, 'L',
+        "\t\tDisplay the details of ticket(s)", pcmk__option_default
+    },
+    {
+        "raw", no_argument, NULL, 'w',
+        "\t\tDisplay the IDs of ticket(s)", pcmk__option_default
+    },
+    {
+        "query-xml", no_argument, NULL, 'q',
+        "\tQuery the XML of ticket(s)", pcmk__option_default
+    },
+    {
+        "constraints", no_argument, NULL, 'c',
+        "\tDisplay the rsc_ticket constraints that apply to ticket(s)",
+        pcmk__option_default
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "\nCommands:", pcmk__option_default
+    },
+    {
+        "grant", no_argument, NULL, 'g',
+        "\t\tGrant a ticket to this cluster site", pcmk__option_default
+    },
+    {
+        "revoke", no_argument, NULL, 'r',
+        "\t\tRevoke a ticket from this cluster site", pcmk__option_default
+    },
+    {
+        "standby", no_argument, NULL, 's',
+        "\t\tTell this cluster site this ticket is standby",
+        pcmk__option_default
+    },
+    {
+        "activate", no_argument, NULL, 'a',
+        "\tTell this cluster site this ticket is active", pcmk__option_default
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "\nAdvanced Commands:", pcmk__option_default
+    },
+    {
+        "get-attr", required_argument, NULL, 'G',
+        "\tDisplay the named attribute for a ticket", pcmk__option_default
+    },
+    {
+        "set-attr", required_argument, NULL, 'S',
+        "\tSet the named attribute for a ticket", pcmk__option_default
+    },
+    {
+        "delete-attr", required_argument, NULL, 'D',
+        "\tDelete the named attribute for a ticket", pcmk__option_default
+    },
+    {
+        "cleanup", no_argument, NULL, 'C',
+        "\t\tDelete all state of a ticket at this cluster site",
+        pcmk__option_default
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "\nAdditional Options:", pcmk__option_default
+    },
+    {
+        "attr-value", required_argument, NULL, 'v',
+        "\tAttribute value to use with -S", pcmk__option_default
+    },
+    {
+        "default", required_argument, NULL, 'd',
+        "\t(Advanced) Default attribute value to display if none is found "
+            "(for use with -G)",
+        pcmk__option_default
+    },
+    {
+        "force", no_argument, NULL, 'f',
+        "\t\t(Advanced) Force the action to be performed", pcmk__option_default
+    },
+    {
+        "xml-file", required_argument, NULL, 'x',
+        NULL, pcmk__option_hidden
+    },
 
     /* legacy options */
-    {"set-name",   1, 0, 'n', "\t(Advanced) ID of the instance_attributes object to change"},
-    {"nvpair",     1, 0, 'i', "\t(Advanced) ID of the nvpair object to change/delete"},
-    
-    {"-spacer-",	1, 0, '-', "\nExamples:", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', "Display the info of tickets:", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', " crm_ticket --info", pcmk_option_example},
-    {"-spacer-",	1, 0, '-', "Display the detailed info of tickets:", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', " crm_ticket --details", pcmk_option_example},
-    {"-spacer-",	1, 0, '-', "Display the XML of 'ticketA':", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', " crm_ticket --ticket ticketA --query-xml", pcmk_option_example},
-    {"-spacer-",	1, 0, '-', "Display the rsc_ticket constraints that apply to 'ticketA':", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', " crm_ticket --ticket ticketA --constraints", pcmk_option_example},
-
-    {"-spacer-",	1, 0, '-', "Grant 'ticketA' to this cluster site:", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', " crm_ticket --ticket ticketA --grant", pcmk_option_example},
-    {"-spacer-",	1, 0, '-', "Revoke 'ticketA' from this cluster site:", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', " crm_ticket --ticket ticketA --revoke", pcmk_option_example},
-    {"-spacer-",	1, 0, '-', "Make 'ticketA' standby:", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', "The cluster site will treat a granted 'ticketA' as 'standby'."},
-    {"-spacer-",	1, 0, '-', "The dependent resources will be stopped or demoted gracefully without triggering loss-policies", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', " crm_ticket --ticket ticketA --standby", pcmk_option_example},
-    {"-spacer-",	1, 0, '-', "Activate 'ticketA' from being standby:", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', " crm_ticket --ticket ticketA --activate", pcmk_option_example},
-
-    {"-spacer-",	1, 0, '-', "Get the value of the 'granted' attribute for 'ticketA':", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', " crm_ticket --ticket ticketA --get-attr granted", pcmk_option_example},
-    {"-spacer-",	1, 0, '-', "Set the value of the 'standby' attribute for 'ticketA':", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', " crm_ticket --ticket ticketA --set-attr standby --attr-value true", pcmk_option_example},
-    {"-spacer-",	1, 0, '-', "Delete the 'granted' attribute for 'ticketA':", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', " crm_ticket --ticket ticketA --delete-attr granted", pcmk_option_example},
-    {"-spacer-",	1, 0, '-', "Erase the operation history of 'ticketA' at this cluster site:", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', "The cluster site will 'forget' the existing ticket state.", pcmk_option_paragraph},
-    {"-spacer-",	1, 0, '-', " crm_ticket --ticket ticketA --cleanup", pcmk_option_example},
-    
-    {0, 0, 0, 0}
+    {
+        "set-name", required_argument, NULL, 'n',
+        "\t(Advanced) ID of the instance_attributes object to change",
+        pcmk__option_default
+    },
+    {
+        "nvpair", required_argument, NULL, 'i',
+        "\t(Advanced) ID of the nvpair object to change/delete",
+        pcmk__option_default
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "\nExamples:", pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "Display the info of tickets:", pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        " crm_ticket --info", pcmk__option_example
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "Display the detailed info of tickets:", pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        " crm_ticket --details", pcmk__option_example
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "Display the XML of 'ticketA':", pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        " crm_ticket --ticket ticketA --query-xml", pcmk__option_example
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "Display the rsc_ticket constraints that apply to 'ticketA':",
+        pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        " crm_ticket --ticket ticketA --constraints", pcmk__option_example
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "Grant 'ticketA' to this cluster site:", pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        " crm_ticket --ticket ticketA --grant", pcmk__option_example
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "Revoke 'ticketA' from this cluster site:", pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        " crm_ticket --ticket ticketA --revoke", pcmk__option_example
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "Make 'ticketA' standby (the cluster site will treat a granted "
+            "'ticketA' as 'standby', and the dependent resources will be "
+            "stopped or demoted gracefully without triggering loss-policies):",
+        pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        " crm_ticket --ticket ticketA --standby", pcmk__option_example
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "Activate 'ticketA' from being standby:", pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        " crm_ticket --ticket ticketA --activate", pcmk__option_example
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "Get the value of the 'granted' attribute for 'ticketA':",
+        pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        " crm_ticket --ticket ticketA --get-attr granted", pcmk__option_example
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "Set the value of the 'standby' attribute for 'ticketA':",
+        pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        " crm_ticket --ticket ticketA --set-attr standby --attr-value true",
+        pcmk__option_example
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "Delete the 'granted' attribute for 'ticketA':", pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        " crm_ticket --ticket ticketA --delete-attr granted",
+        pcmk__option_example
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "Erase the operation history of 'ticketA' at this cluster site:",
+        pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        "The cluster site will 'forget' the existing ticket state.",
+        pcmk__option_paragraph
+    },
+    {
+        "-spacer-", no_argument, NULL, '-',
+        " crm_ticket --ticket ticketA --cleanup", pcmk__option_example
+    },
+    { 0, 0, 0, 0 }
 };
-/* *INDENT-ON* */
 
 int
 main(int argc, char **argv)
@@ -559,15 +724,17 @@ main(int argc, char **argv)
     GHashTable *attr_set = crm_str_table_new();
 
     crm_log_init(NULL, LOG_CRIT, FALSE, FALSE, argc, argv, FALSE);
-    crm_set_options(NULL, "(query|command) [options]", long_options,
-                    "Perform tasks related to cluster tickets.\nAllows ticket attributes to be queried, modified and deleted.\n");
+    pcmk__set_cli_options(NULL, "<query>|<command> [options]", long_options,
+                          "perform tasks related to cluster tickets\n\n"
+                          "Allows ticket attributes to be queried, modified "
+                          "and deleted.\n");
 
     if (argc < 2) {
-        crm_help('?', CRM_EX_USAGE);
+        pcmk__cli_help('?', CRM_EX_USAGE);
     }
 
     while (1) {
-        flag = crm_get_option(argc, argv, &option_index);
+        flag = pcmk__next_cli_option(argc, argv, &option_index, NULL);
         if (flag == -1)
             break;
 
@@ -577,7 +744,7 @@ main(int argc, char **argv)
                 break;
             case '$':
             case '?':
-                crm_help(flag, CRM_EX_OK);
+                pcmk__cli_help(flag, CRM_EX_OK);
                 break;
             case 'Q':
                 BE_QUIET = TRUE;
@@ -644,7 +811,7 @@ main(int argc, char **argv)
                 do_force = TRUE;
                 break;
             case 'x':
-                xml_file = strdup(optarg);
+                xml_file = optarg;
                 break;
             case 'n':
                 set_name = optarg;
@@ -673,7 +840,7 @@ main(int argc, char **argv)
     }
 
     if (argerr) {
-        crm_help('?', CRM_EX_USAGE);
+        pcmk__cli_help('?', CRM_EX_USAGE);
     }
 
     data_set = pe_new_working_set();
@@ -682,7 +849,7 @@ main(int argc, char **argv)
         exit_code = CRM_EX_OSERR;
         goto bail;
     }
-    set_bit(data_set->flags, pe_flag_no_counts);
+    pe__set_working_set_flags(data_set, pe_flag_no_counts|pe_flag_no_compat);
 
     cib_conn = cib_new();
     if (cib_conn == NULL) {
@@ -693,7 +860,7 @@ main(int argc, char **argv)
 
     rc = cib_conn->cmds->signon(cib_conn, crm_system_name, cib_command);
     if (rc != pcmk_ok) {
-        CMD_ERR("Could not connect to the CIB manager: %s", pcmk_strerror(rc));
+        CMD_ERR("Could not connect to CIB: %s", pcmk_strerror(rc));
         exit_code = crm_errno2exit(rc);
         goto bail;
     }
@@ -737,7 +904,7 @@ main(int argc, char **argv)
         }
 
         if (ticket_id) {
-            ticket_t *ticket = find_ticket(ticket_id, data_set);
+            pe_ticket_t *ticket = find_ticket(ticket_id, data_set);
 
             if (ticket == NULL) {
                 CMD_ERR("No such ticket '%s'", ticket_id);
@@ -794,7 +961,7 @@ main(int argc, char **argv)
         }
 
         if (do_force == FALSE) {
-            ticket_t *ticket = NULL;
+            pe_ticket_t *ticket = NULL;
 
             ticket = find_ticket(ticket_id, data_set);
             if (ticket == NULL) {
@@ -824,14 +991,14 @@ main(int argc, char **argv)
         }
 
         if (attr_value
-            && (attr_name == NULL || strlen(attr_name) == 0)) {
+            && (pcmk__str_empty(attr_name))) {
             CMD_ERR("Must supply attribute name with -S for -v %s", attr_value);
             exit_code = CRM_EX_USAGE;
             goto bail;
         }
 
         if (attr_name
-            && (attr_value == NULL || strlen(attr_value) == 0)) {
+            && (pcmk__str_empty(attr_value))) {
             CMD_ERR("Must supply attribute value with -v for -S %s", attr_name);
             exit_code = CRM_EX_USAGE;
             goto bail;
@@ -854,7 +1021,7 @@ main(int argc, char **argv)
          * this is just for reporting usage errors
          */
 
-        if (attr_name == NULL || strlen(attr_name) == 0) {
+        if (pcmk__str_empty(attr_name)) {
             // We only get here if ticket_cmd was left as default
             CMD_ERR("Must supply a command");
             exit_code = CRM_EX_USAGE;
@@ -867,7 +1034,7 @@ main(int argc, char **argv)
             goto bail;
         }
 
-        if (attr_value == NULL || strlen(attr_value) == 0) {
+        if (pcmk__str_empty(attr_value)) {
             CMD_ERR("Must supply value with -v for -S %s", attr_name);
             exit_code = CRM_EX_USAGE;
             goto bail;
