@@ -549,7 +549,7 @@ main(int argc, char *argv[])
                 usage(argv[0]);
                 return CRM_EX_OK;
             } else if (!strcmp(*arg, "--version") || !strcmp(*arg, "-$")) {
-                crm_help('$', CRM_EX_OK);
+                pcmk__cli_help('$', CRM_EX_OK);
             }
         }
     }
@@ -563,7 +563,7 @@ main(int argc, char *argv[])
     }
 #endif
 
-    crm_make_daemon("ipmiservicelogd", TRUE, PCMK_RUN_DIR "/ipmiservicelogd.pid0");
+    pcmk__daemonize("ipmiservicelogd", PCMK_RUN_DIR "/ipmiservicelogd.pid0");
     crm_log_cli_init("ipmiservicelogd");
     // Maybe this should log like a daemon instead?
     // crm_log_init("ipmiservicelogd", LOG_INFO, TRUE, FALSE, argc, argv, FALSE);
